#!/bin/sh
#$Id: install.sh,v 1.25 2010/02/26 14:50:49 cguirao Exp $
#
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# COPYRIGHT (c) 2003 European Southern Observatory
# LICENSE
#
# NAME:      EditLaTex.sh
# TYPE:      Bourne Shell script
# PROJECT:   VLT Data Flow System
# AUTHOR:    CG - ESO/DMD/SEG
# DESCRIPTION: script to install recursively a package and all dependencies
# USAGE:
#   install.sh [-r <release>]
# EXAMPLES:
#   ./install.sh -r dfs-4_10
#------------------------------------------------------------------------------

checkout_and_build()
{
release="$2 $3"
subdir=`dirname $1`
module=`basename $1`
#
# If module is simple, simple check it out
# if module is a composite, i.e. aaa/bbb/ccc 
#    mkdir aaa; cd aaa ; cvs co bbb/ccc 
if [ "$subdir" != "." ]; then
  subdir=`echo $1 | sed 's/\([^\/]*\).*/\1/'` 
  module=`echo $1 | sed  's/[^\/]*\/\(.*\)/\1/'`
  mkdir -p $subdir
fi

cd $subdir
#
# if the module does not exist check it out
#
if [ ! -d "$module" ]; then 
  echo "cvs co $release $module"
  cvs co $release $module >> $LOGFILE 2>&1 ; result=$?
  if [ $result -ne 0 ]; then
    echo "$cmd [ERROR]: cvs co $release $module. See logfile $LOGFILE"; exit $result
  fi
fi

cd "$module"
if [ -f "SUCCESFULLY_INSTALLED" ]; then
     echo "$subdir/$module already installed"; return 0
#
# For MAVEN modules with pom-all.xml
#
elif [ -f pom-all.xml ]; then
     ThisDir=`pwd`
     echo "$module: mvn -f pom-all.xml package"
     mvn -f pom-all.xml package >>$LOGFILE 2>&1 ; result=$?
     if [ $result -ne 0 ]; then
       echo "$module: [ERROR]: mvn. See logfile $LOGFILE"; return $result
     fi
     echo "cd $INTROOT"
     echo "tar xvf $ThisDir/*.tar"
     cd $INTROOT; tar xvf $ThisDir/*.tar >>$LOGFILE 2>&1; result=$?
     if [ $result -ne 0 ]; then
       echo "$module: [ERROR]: install. See logfile $LOGFILE"; return $result
     fi
     cd $ThisDir
     touch SUCCESFULLY_INSTALLED
#
# For MAVEN modules with simple pom.xml
#
elif [ -f pom.xml ]; then
     ThisDir=`pwd`
     echo "$module: mvn package"
     mvn package >>$LOGFILE 2>&1 ; result=$?
     if [ $result -ne 0 ]; then
       echo "$module: [ERROR]: mvn. See logfile $LOGFILE"; return $result
     fi
     echo "cd $INTROOT"
     echo "tar xvf $ThisDir/*.tar"
     cd $INTROOT; tar xvf $ThisDir/*.tar >>$LOGFILE 2>&1; result=$?
     if [ $result -ne 0 ]; then
       echo "$module: [ERROR]: install. See logfile $LOGFILE"; return $result
     fi
     cd $ThisDir
     touch SUCCESFULLY_INSTALLED
#
# For VLT modules (e.g. p2pp,ot,mt)
#
elif [ -f "Makefile.$subdir" ]; then
     #
     # Let preserve the original INTROOT, and recreate INTROOT for OHS
     # packages as $ORIGINAL_INTROOT/INTROOT
     #
     ThisDir=`pwd`
     ORIGINAL_INTROOT=$INTROOT
     INTROOT=$INTROOT/INTROOT

     if [ -z "$CUSTOMER" ]; then
       echo "$subdir/$module: [WARNING]: environment CUSTOMER not defined. Using default SEG" 
       CUSTOMER=SEG; export CUSTOMER
     fi

     if [ ! -d "$INTROOT" ]; then
       mkdir $INTROOT
       echo "$subdir/$module: getTemplateForDirectory INTROOT $INTROOT"
       getTemplateForDirectory INTROOT $INTROOT >/dev/null 2>&1
     fi
     echo "$subdir/$module: make build"
     OHS=`pwd`; export OHS; \
     make build >> $LOGFILE 2>&1 ; result=$? 
     if [ $result -ne 0 ]; then 
       echo "$subdir/$module: [ERROR]: make build. See logfile $LOGFILE"; return $result
     fi

     DISTROOT=$ORIGINAL_INTROOT/OHS ; export DISTROOT
     if [ ! -d "$DISTROOT" ]; then
       mkdir $DISTROOT
     fi
     echo "$subdir/$module: make -f Makefile.$subdir distribution CUSTOMER=$CUSTOMER"
     make -f Makefile.$subdir distribution CUSTOMER=$CUSTOMER >> $LOGFILE 2>&1 ; result=$?; \
     if [ $result -ne 0 ]; then 
       echo "$subdir/$module: [ERROR]: make -f Makefile.$subdir distribution CUSTOMER=$CUSTOMER. See logfile $LOGFILE"; return $result
     fi
     cd $DISTROOT; gtar xzf $subdir-*-$CUSTOMER.tar.gz
     ln -s `basename $subdir-*-$CUSTOMER.tar.gz -$CUSTOMER.tar.gz` $subdir
     rm -rf ${subdir}*gz

     INTROOT=$ORIGINAL_INTROOT
     cd $ThisDir
     touch SUCCESFULLY_INSTALLED
#
# For Autotools modules
#
elif [ -f "Makefile" ]; then
     echo "$module: make"
     make >>$LOGFILE 2>&1 ; result=$?
     if [ $result -ne 0 ]; then
       echo "$module: [ERROR]: make. See logfile $LOGFILE"; return $result
     fi
     echo "$module: make install"
     make install >>$LOGFILE 2>&1 ; result=$?
     if [ $result -ne 0 ]; then 
       echo "$module [ERROR]: make install. See logfile $LOGFILE"; return $result
     fi
     touch SUCCESFULLY_INSTALLED
elif [ -x "configure" ]; then
     # 
     # If the module already pre-defines a prefix, use it
     #
     if egrep -q "^AC_PREFIX_DEFAULT" configure.ac ; then
       echo "$module: ./configure"
       ./configure >>$LOGFILE 2>&1 ; result=$?
     else
       echo "$module: ./configure --prefix=$INTROOT"
       ./configure --prefix=$INTROOT >>$LOGFILE 2>&1 ; result=$?
     fi
     if [ $result -ne 0 ]; then 
       echo "$module [ERROR]: ./configure. See logfile $LOGFILE"; return $result
     fi
     echo "$module: make"
     make >>$LOGFILE 2>&1 ; result=$?
     if [ $result -ne 0 ]; then
       echo "$module [ERROR]: make. See logfile $LOGFILE"; return $result
     fi
     echo "$module: make install"
     make install >>$LOGFILE 2>&1 ; result=$?
     if [ $result -ne 0 ]; then
       echo "$module [ERROR]: make install. See logfile $LOGFILE"; return $result
     fi
     touch SUCCESFULLY_INSTALLED
elif [ -x "bootstrap" ]; then
     echo "$module: ./bootstrap"
     ./bootstrap >> $LOGFILE 2>&1 ; result=$?
     if [ $result -ne 0 ]; then
       echo "$module [ERROR]: ./bootstrap. See logfile $LOGFILE"; return $result
     fi
     # 
     # If the module already pre-defines a prefix, use it
     #
     if [ "$subdir" = "Gasgano230" ]; then
       echo "$module: ./configure --prefix=$INTROOT/gasgano230"
       ./configure --prefix=$INTROOT/gasgano230  >>$LOGFILE 2>&1 ; result=$?
     elif egrep -q "^AC_PREFIX_DEFAULT" configure.ac ; then
       echo "$module: ./configure"
       ./configure >>$LOGFILE 2>&1 ; result=$?
     else
       echo "$module: ./configure --prefix=$INTROOT"
       ./configure --prefix=$INTROOT >>$LOGFILE 2>&1 ; result=$?
     fi
     if [ $result -ne 0 ]; then 
       echo "$module [ERROR]: ./configure. See logfile $LOGFILE"; return $result
     fi
     echo "$module: make install"
     make install >> $LOGFILE 2>&1 ; result=$?
     if [ $result -ne 0 ]; then 
       echo "$module [ERROR]: make install. See logfile $LOGFILE"; return $result
     fi
     touch SUCCESFULLY_INSTALLED
else 
     echo "$module [ERROR]: nothing to install"; return 1
fi

return 0
}
 

# main
# MAIN

keep_log=0
release=

cmd=`basename $0`
cmddir=`dirname $0`
LOGFILE=/tmp/${cmd}-${USER}.log
mainmodule=$1
echo "cmd: All commands are logged in file $LOGFILE"

# Move to the directory above the location of this script
cd $cmddir
if [ -z "$mainmodule" ]; then
  ThisDir=`pwd`
  mainmodule=`basename $ThisDir`
else
  shift
fi

THIS_CMD=`pwd`/$cmd
cd ..
workdir=`pwd`
echo "Working directory is $workdir"

dfs_release=`basename $workdir`
echo dfs_release=$dfs_release

 
#
# Read arguments
#
while test $# != 0
do
  case $1 in 
  -r) release="-r $2" ; shift; shift ;;
  -c) keep_log=1 ; shift ;;
  *) echo "$cmd: argument $1 unexpected. Exit"; exit 1;;
  esac
done

if [ -z "$INTROOT" ]; then
   echo "$cmd [ERROR]: INTROOT not defined"
   exit 1
elif [ ! -d "$INTROOT" ]; then
   echo "$cmd [WARNING]: $INTROOT does not exist. Creating it empty"
   echo "$cmd: mkdir -p $INTROOT"
   mkdir -p $INTROOT  ; result=$? 
   if [ $result != 0 ]; then 
     echo "$cmd [ERROR]: executing \"mkdir -p $INTROOT\""
     exit 1
   fi
fi
echo "$cmd: DFS installation on $INTROOT" 
echo -n "Do you want to continue [y/n]? (y):" 
read answ
if [ -z "$answ" ]; then
  answ=y
fi
if [ "$answ" != "y" -a "$answ" != "Y" ]; then
  exit 0
fi
echo "OK, let's go. This may take some time..."


# Loading the environment for this mainmodule 
if [ -f "$mainmodule/install.env" ]; then
  echo "Reading . $mainmodule/install.env"
  . $mainmodule/install.env
fi

#
# Remove the log if we start
#
if [ $keep_log -eq 0 ]; then
  rm -f $LOGFILE
fi

#
# Check DEPENDENCIES
#
if [ -f "$mainmodule/DEPENDENCIES" ] ; then
  # Remove first comment lines with #
  cat $mainmodule/DEPENDENCIES | sed -e 's/[      ]*//' -e 's/^#.*//' | while read line
  do
    line=$line
    if [ -z "$line" ]; then
       continue
    else
       echo "$mainmodule: depends on \"$line\""
       cd $workdir
       checkout_and_build $line ; result=$?
       if [ $result -ne 0 ]; then
         echo "$cmd [ERROR]: See logfile $LOGFILE"; exit $result
       fi
    fi
  done
  result=$?
  if [ $result -ne 0 ]; then exit $result; fi
else
  echo "$mainmodule: no DEPENDENCIES"
fi

#
# Finally after DEPENDENCIES, build this main module
#
checkout_and_build $mainmodule ; result=$?
exit $resul
