#ifndef _FITSIO_
#define _FITSIO_

/*
 * E.S.O. - VLT project 
 *
 * "@(#) $Id: FitsIO.h,v 1.1 2007/02/26 14:51:42 cguirao Exp $" 
 *
 * This class is a replacement of the "original" FITS manipulation class.
 * See previous CVS revisions for original code.
 * Only the interfaces used by OLAS are kept.
 *
 * who                 when           what
 * ----------------    -----------    -------
 * Stefano Zampieri    15 Sep 2005    Created
 */

#include "Mem.h"

extern "C" {
#include "error.h"
#include "fitsio.h"
}

using namespace std;

class FitsIO {
  
 private:
  fitsfile* fitsio_;     // handle to use for cfitsio C library routines
  
  // constructor
  FitsIO(fitsfile* fitsio);
  
  // error reporting methods
  static int cfitsio_error();
  static int cfitsio_error(const char *msg);
  static int keyword_not_found_warning(const char *keyword);
  
 public:
  
  // destructor
  ~FitsIO();
  
  // read a FITS file and return a pointer to an allocated FitsIO object
  // NULL if an error occurred
  static FitsIO* read(const char* filename, enum Mem::FileOptions opt = Mem::FILE_DEFAULTS);
  
  static FitsIO* initialize(Mem& header);
  
  // find and set value for the given FITS keyword and return 0 if OK (found)
  int get(const char* keyword, double& val) const;
  int get(const char* keyword, float& val) const;
  int get(const char* keyword, int& val) const;
  int get(const char* keyword, long& val) const;
  int get(const char* keyword, unsigned char& val) const;
  int get(const char* keyword, unsigned short& val) const;
  int get(const char* keyword, short& val) const;
  
  // find and return the value for the given FITS keyword, or NULL if not found
  char* get(const char* keyword) const;
  
  // same as get(const char*), but you supply the buffer to hold the result
  char* get(const char* keyword, char* buf, int bufsz) const;
  
  // these are static versions of the above that require the cfitsio handle
  static int get(fitsfile*, const char* keyword, double& val);
  static int get(fitsfile*, const char* keyword, float& val);
  static int get(fitsfile*, const char* keyword, int& val);
  static int get(fitsfile*, const char* keyword, long& val);
  static int get(fitsfile*, const char* keyword, unsigned char& val);
  static int get(fitsfile*, const char* keyword, unsigned short& val);
  static int get(fitsfile*, const char* keyword, short& val);
  
  // find and return the value for the given FITS keyword, or NULL if not found
  static char* get(fitsfile*, const char* keyword);
  
  // Insert the given FITS keyword and value and return 0 if OK
  // If there is not enough space in the header, the file size is
  // automatically increased.
  int put(const char* keyword, double val, const char* comment = NULL);
  int put(const char* keyword, float val, const char* comment = NULL);
  int put(const char* keyword, int val, const char* comment = NULL);
  int put(const char* keyword, const char* val, const char* comment = NULL);
  
  // write/update the checksum and datasum in all HDUs of the fits file
  int writeChecksum();
  
  // return name fits file name
  int filename(char *);
  
};

#endif
