#ifndef _MEM_
#define _MEM_

/*
 * E.S.O. - VLT project / ESO Archive
 *
 * "@(#) $Id: Mem.h,v 1.1 2007/02/26 14:51:42 cguirao Exp $" 
 *
 * The purpose of this class is to replace only the very basic functionalities
 * of the "original" Mem class, which was used to manage memory areas
 * and to mmap files.
 * This class is used by OLAS and it's not meant to be used by other (new)
 * applications
 * See previous CVS revisions for original code.
 * 
 * who                when          what
 * ----------------   -----------   -------------------------------------
 * Stefano Zampieri   15 Sep 2005   Created
 */


#include <string>
#include <iostream>
#include <fstream>

#include "error.h"
#include "util.h"

extern "C" {
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
}

using namespace std;

class Mem {
 public:
  // file options
  enum FileOptions {
    FILE_DEFAULTS = 0,   // File RDONLY
    FILE_RDWR = 1,       // File writable
  };
  
 private:
  string filename_;
  string linkname_;
  char *data_;
  long size_;
  int owner_;
  int status_;
  int options_;
  
 public:
  // constructor with filename. No memory is allocated.
  Mem (const char *filename, int options = FILE_DEFAULTS, int verbose = 0);
  // constructor with size of memory to allocate.
  Mem (int size);
  // simple constructor
  Mem ();
  // destructor
  ~Mem ();
  
  // member access methods
  const char* filename(int flag = 0) const;
  int status() const { return status_; }
  int length() const { return size(); }
  
  // if the constructor with filename was used, this
  // method can be used to load in memory the file contents
  int loadFileInMemory();
  // get pointer to data
  void * myptr() const;
  // set owner flag
  void owner(int);
  // get size
  int size() const;
  // get file options
  int options() const { return options_; }

};

#endif
