#ifndef miscErrNoCcs_H
#define miscErrNoCcs_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: miscNoCcs.h,v 1.1 2007/02/26 14:51:42 cguirao Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  01/11/96  created
* mnylund   25/11/03  errPrintStderr is added, SPR VLTSW20030409
* mnylund   11/10/04  vltPortGeneral.h is included, SPR VLTSW20040260
*/

/************************************************************************
 */

#ifdef __cplusplus
extern "C" {
#endif

#include <stdarg.h>
#include "vltPortGeneral.h"

#define errSTACK_SIZE      10
#define ccsMODULEID_LEN    7 
#define ccsPROCNAME_LEN    19
#define ccsFALSE           0
#define ccsTRUE            1           
#define modUNIX            "UNIX"
#define msgBODYMAXLEN      8000
#define USE(var)  static void *use_##var = ((void)&use_##var, (void *) &var);

#define errTOP     "top"
#define errBOTTOM  "bottom"

/* The macro __FILE_LINE__ produces a string in the format: "FileName:Line" */
/* with the name and line of the file where it has been called.             */
/* It is usefull in error messages                                          */
/* (the other macros defined here are just for support, since I have not    */
/* found a cleaner way to define __FILE_LINE__ macro                        */
#define _eccs_tostr(a) #a
#define _eccs_tostr_pass2(a) _eccs_tostr(a) 
#define __FILE_LINE__   __FILE__ ":" _eccs_tostr_pass2(__LINE__)

typedef char               ccsMODULEID[ccsMODULEID_LEN + 1];
typedef char               ccsPROCNAME[ccsPROCNAME_LEN + 1];

typedef enum {
    FAILURE = 1,
    SUCCESS 
} ccsCOMPL_STAT; 

typedef struct {
    vltBYTES64      errorMnemonic;
    vltBYTES32      timeStamp;      /* Time when the error occurred         */
				    /* - Subroutine, Function, etc ..       */ 
    ccsMODULEID     moduleId;       /* Addresses the error messages file    */
    vltBYTES256     runTimePar;     /* Run Time Information about the error */
} ccsSTACK_ELEM;

typedef struct {                
    ccsSTACK_ELEM errorStack[errSTACK_SIZE];   
    char          *timeStamp;
    char          *moduleId;
    char          *runTimePar;
    vltINT32      errorsInStack;
} ccsERROR;                         

ccsCOMPL_STAT errAdd(ccsERROR           *error, 
		     const ccsMODULEID  moduleId,   
		     char               *errorMnemonic, 
		     const char         *locId,      		     
		     char               *format,     
		     ...);

ccsCOMPL_STAT errSysAdd(ccsERROR           *error,
			const ccsMODULEID  moduleId,
			char               *errorMnemonic,
			const char         *locId,
			char               *format,
			char               *sysModule,
			...);

ccsCOMPL_STAT errIsReason(ccsERROR           *error,
			  const ccsMODULEID  moduleId,           
			  char               *errorMnemonic,
			  vltLOGICAL         *errFound,
			  ccsSTACK_ELEM      **errDescription);

vltLOGICAL errErrInStack(const char      *errorMnemonic,
			 const ccsERROR  *error);

vltINT8 errGetStackSize(ccsERROR       *error);

ccsCOMPL_STAT errAdd_v(ccsERROR            *error,
		       const ccsMODULEID   moduleId,
		       char                *errorMnemonic,                 
		       const char          *locId,
		       char                *format,
		       va_list             va_parList);

ccsCOMPL_STAT errSysAdd_v(ccsERROR            *error,
			  const ccsMODULEID   moduleId,
			  char                *errorMnemonic,              
			  const char          *locId,
			  char                *format,
			  char                *sysModule,
			  va_list             va_parList);

ccsCOMPL_STAT errPrint(ccsERROR  *error);

/* VLTSW20030409 */
ccsCOMPL_STAT errPrintStderr(ccsERROR  *error);

ccsCOMPL_STAT errResetStack(ccsERROR        *error);

ccsCOMPL_STAT errCloseStack(ccsERROR        *error);

ccsCOMPL_STAT errCopy(ccsERROR *errDest,
		      ccsERROR *errSrc);
 
ccsCOMPL_STAT errMergeStack(ccsERROR  *errDest,
			    ccsERROR  *errSrc,
			    char      *flag);

ccsCOMPL_STAT ccsInit(const ccsPROCNAME  procName,
		      vltUINT32          obituary,
		      void               (*breakHandler)(int),
		      void               (*killHandler)(int), 
		      ccsERROR           *error);

ccsCOMPL_STAT ccsExit (ccsERROR    *error);



#ifdef __cplusplus
}
#endif

#endif /*!miscErrNoCcs_H*/
