#ifndef oslxINS_DIR_H
#define oslxINS_DIR_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: oslxINS_DIR.h,v 1.1 2007/02/26 14:51:42 cguirao Exp $" 
*
* oslxINS_DIR.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  17/10/95  Created
*/

/************************************************************************
*  Header file for the OSLX class oslxINS_DIR
*----------------------------------------------------------------------
*/

#include "oslxPRIVATE.h"
#include "oslxFITS.h"

#ifndef __cplusplus
#error This is a C++ include file which cannot be used from plain C
#endif


typedef enum {
    slxTYPE_SETUP_FILE = 1,
    slxTYPE_CONFIG_FILE,
    slxTYPE_ALIAS_CONV_TABLE,
    slxTYPE_DICTIONARY,
    slxTYPE_FITS_FILE,
    slxTYPE_SHORT_FITS_FILE,
    slxTYPE_ALL_TYPES,
    slxTYPE_UNKNOWN
} slxFILE_TYPE;

typedef struct {
    vltINT32   userPerm;
    vltINT32   groupPerm;
    vltINT32   othersPerm;
} oslxFILE_STAT;


// Class to handle aliasing of setup keywords.
class oslxINS_DIR: public oslxBASE
{
  public:
    oslxINS_DIR();
    ~oslxINS_DIR();

    void          UseInsTree(const vltLOGICAL state);
    vltLOGICAL    UseInsTree();

    ccsCOMPL_STAT SetDictionary(const oslxDICTIONARY *dictionary);

    ccsCOMPL_STAT Clear();

    ccsCOMPL_STAT SetInsRoot(const slxFILENAME  insRoot);
    ccsCOMPL_STAT GetInsRoot(slxFILENAME insRoot);
    ccsCOMPL_STAT SetInsUser(const slxFILENAME  insRoot);
    ccsCOMPL_STAT SetInsSetupPath(const char  *insSetupPath);
    ccsCOMPL_STAT AddInsSetupPath(const slxFILENAME  setupPath);

    ccsCOMPL_STAT  GetInsUsers(char *msgBody = NULL);
    char           *GetNextUserDir(const vltLOGICAL  initialize = FALSE);
    vltINT32       NoOfInsUsers();

    ccsCOMPL_STAT  SearchFitsFile(const slxFILENAME baseName,
				  vltINT32          *numberFound,
				  const vltLOGICAL  globalSearch = FALSE,
				  const vltLOGICAL  stopFirstOccur = TRUE,
				  const vltLOGICAL  skipInsUserDir = FALSE,
				  const char        *pathEnvVar = NULL,
				  oslxKEYW_FILTER   *filter = NULL,
				  const vltLOGICAL  ignoreUnknownKeys = FALSE,
				  const vltLOGICAL  searchLocalFileList=FALSE);

    ccsCOMPL_STAT   GetSearchFileList(char **fileList);
    ccsCOMPL_STAT   GetSearchFileListRaw(char **fileList);
    ccsCOMPL_STAT   GetFormattedFileList(const char *format,
					 const char *fields,
					 char **fileList);
    ccsCOMPL_STAT   SetSearchFileList(const char *fileList);

    // Methods to manipulate the INS Directory structure
    // IMPL.: All the following methods are not yet developed
    ccsCOMPL_STAT  CreateUserBranch(const char          *user,
				    const char          *insRoot = NULL);
    ccsCOMPL_STAT  DeleteUserBranch(const char          *user,
				    const vltLOGICAL    force = FALSE,
				    const char          *insRoot = NULL);
    ccsCOMPL_STAT  CopyUserBranch(const char            *srcUser,
				  const char            *destUser,
				  const char            *insRoot = NULL);
    // Protects all directories and files in a User Branch
    ccsCOMPL_STAT  ProtectUserBranch(const char         *user,
				     const char         *userPerm = "rwx",
				     const char         *groupPerm = "rwx",
				     const char         *othersPerm = "rwx",
				     const char         *insRoot = NULL);
    ccsCOMPL_STAT  UnprotectUserBranch(const char       *user,
				       const char       *userPerm = "rwx",
				       const char       *groupPerm = "rwx",
				       const char       *othersPerm = "rwx",
				       const char       *insRoot = NULL);
    
    ccsCOMPL_STAT  CreateObsModeBranch(const char       *obsMode,
				       const char       *user,
				       const char       *insRoot = NULL);
    ccsCOMPL_STAT  DeleteObsModeBranch(const char       *obsMode,
				       const char       *user,
				       const vltLOGICAL force = FALSE,
				       const char       *insRoot = NULL);
    ccsCOMPL_STAT  ProtectObsModeBranch(const char      *obsMode,
					const char      *user,
					const char      *userPerm = "rwx",
					const char      *groupPerm = "rwx",
					const char      *othersPerm = "rwx",
					const char      *insRoot = NULL);
    ccsCOMPL_STAT  UnprotectObsModeBranch(const char    *obsMode,
					  const char    *user,
					  const char    *userPerm = "rwx",
					  const char    *groupPerm = "rwx",
					  const char    *othersPerm = "rwx",
					  const char    *insRoot = NULL);


    // Method to delete a file, or several files if wildcards are used
    ccsCOMPL_STAT  DeleteFile(const slxFILENAME   *fileName,
			      const vltLOGICAL    force = FALSE,
			      const char          *obsMode = NULL,
			      const char          *insUser = NULL,
			      const char          *insRoot = NULL);
    // The destination file can be a directory. Wildcards can be used
    // if a directory is specified as destination. If the destination file
    // exists, and "force" is not specified, the method will return with an
    // error.
    ccsCOMPL_STAT  CopyFile(const char              *srcFileName,
			    const char              *destFile,
			    const vltLOGICAL        force = FALSE,
			    const char              *obsMode = NULL,
			    const char              *insUser = NULL,
			    const char              *insRoot = NULL);
    ccsCOMPL_STAT  ProtectFile(const slxFILENAME    *fileName,
			       const char           *userPerm = "rwx",
			       const char           *groupPerm = "rwx",
			       const char           *othersPerm = "rwx",
			       const char           *obsMode = NULL,
			      const char            *insUser = NULL,
			      const char            *insRoot = NULL);
    ccsCOMPL_STAT  UnprotectFile(const slxFILENAME  *fileName,
				 const char         *userPerm = "rwx",
				 const char         *groupPerm = "rwx",
				 const char         *othersPerm = "rwx",
				 const char         *obsMode = NULL,
				 const char         *insUser = NULL,
				 const char         *insRoot = NULL);

    ccsCOMPL_STAT TestDuplicateFiles(const slxFILE_TYPE type, 
				     vltINT32       *numberFound, 
				     const char     *obsMode = NULL,
				     const char     *insUser = NULL,
				     const char     *insRoot = NULL);
    ccsCOMPL_STAT GetNextDuplicate(const vltLOGICAL first = FALSE);

    ccsCOMPL_STAT Statistics(const slxFILE_TYPE     type,
			     oslxFILE_STAT          *status,
			     const char             *obsMode = NULL,
			     const char             *insUser = NULL,
			     const char             *insRoot = NULL);
    ccsCOMPL_STAT FindSuspectFiles(oslxFILE_STAT    *status,
				   const char       *obsMode = NULL,
				   const char       *insUser = NULL,
				   const char       *insRoot = NULL);

  protected:
    vltLOGICAL       useInsTree;

    oslxDICTIONARY   *dictionary;
    vltLOGICAL       dicAvail;

    slxSTRING_LIST   *insUsers;
    slxSTRING_LIST   *insUserPtr;
    vltINT32         noOfInsUsers;

    oslxFITS         fits;

    slxSTRING_LIST   *fileListRaw;
    slxSTRING_LIST   *fileListFiltered;
    slxSTRING_LIST   *fileList;
    slxSTRING_LIST   *fileInfoList;
    vltINT32         updateFileInfoList;
    vltINT32         fileInfoListCount;  
    vltINT32         noFilesFound;
    slxDYN_BUF       fileSearchList;
};   // end-class oslxINS_DIR



#endif /*!oslxINS_DIR_H*/
