#ifndef slxDynBuf_H
#define slxDynBuf_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: slxDynBuf.h,v 1.1 2007/02/26 14:51:42 cguirao Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  04/10/96  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */
#ifdef __cplusplus
extern "C" {
#endif

#include "slxBasics.h"

/* SLX Dynamic Buffer. Keyword lines are stored with a possible comment in the
 * Short-FITS format. The buffer is maintained by the functions using the
 * handle, i.e. more bytes are allocated if needed by the various functions.
 * Separator in the buffer is _one_ '\n'. 
 */
typedef struct {
    char        *dynBuf;
    vltUINT32   bytesAlloc;
    vltUINT32   bytesStored;
    vltINT32    count;
} slxDYN_BUF;

/* Actually obsolete, but kept for backwards compatability
 */
typedef enum {
    slxDO_NOT_INIT = 0,
    slxINIT_DYN_BUF
} slxDYN_BUF_OP;

/* Functions to operate on SLX Dynamic Buffers
 */
ccsCOMPL_STAT slxInitDynBuf(slxDYN_BUF   *dynBuf,
			    ccsERROR     *error);

ccsCOMPL_STAT slxResetDynBuf(slxDYN_BUF  *dynBuf,
			     ccsERROR    *error);

ccsCOMPL_STAT slxSoftResetDynBuf(slxDYN_BUF   *dynBuf,
			         ccsERROR     *error);

ccsCOMPL_STAT slxDynBufAlloc(slxDYN_BUF       *dynBuf,
                             const vltUINT32  size,
                             ccsERROR         *error);

ccsCOMPL_STAT slxRewindDynBuf(slxDYN_BUF     *dynBuf,
			      const vltINT32 bytes,
			      ccsERROR       *error);

char *slxDynBuf(const slxDYN_BUF     *dynBuf);

vltUINT32 slxSizeDynBuf(const slxDYN_BUF     *dynBuf);

vltUINT32 slxAllocDynBuf(const slxDYN_BUF     *dynBuf);

char slxGetCharDynBuf(const slxDYN_BUF  *dynBuf,
		      const vltINT32    index);

ccsCOMPL_STAT slxSetCharDynBuf(slxDYN_BUF     *dynBuf,
			       const vltINT32 index,
			       char           chr,
			       ccsERROR       *error);

ccsCOMPL_STAT slxTruncateDynBuf(slxDYN_BUF     *dynBuf,
				const vltINT32 bytes,
				ccsERROR       *error);

ccsCOMPL_STAT slxGetNextLineBuf(slxDYN_BUF            *setupHandle,
				const slxDYN_BUF_OP   initBufPtr,
				slxKEYW_LINE          keywordLine,
				ccsERROR              *error);

ccsCOMPL_STAT slxGetNextLineBuf2(slxDYN_BUF            *setupHandle,
				 const slxDYN_BUF_OP   initDynBuf,
				 char                  **keywordLine,
				 ccsERROR              *error);

ccsCOMPL_STAT slxGetNextKeyBuf(slxDYN_BUF           *setupHandle,
                               const slxDYN_BUF_OP  initDynBuf,
                               char                 *keyword,
                               char                 *value,
                               ccsERROR             *error);

ccsCOMPL_STAT slxStoreLineBuf(const slxKEYW_LINE  keywordLine,
			      slxDYN_BUF          *setupHandle,
			      vltINT32            *lineNo,
			      ccsERROR            *error);

ccsCOMPL_STAT slxStoreBufDynBuf(slxDYN_BUF        *dynBuf,
                                const char        *buf,
                                const vltUINT32   bufLen,
                                ccsERROR          *error);

ccsCOMPL_STAT slxGetKeywordBuf(slxDYN_BUF       *setupHandle,
			       const char       *keyword,
			       const vltUINT16  keywordNumber,
			       slxKEYW_INFO     *keywordInfo,
			       ccsERROR         *error);

ccsCOMPL_STAT slxDynBufReadToCrt(slxDYN_BUF    *dynBuf,
 	                         char          **buffer,
                                 vltUINT32     *bufLen,
	 	                 ccsERROR      *error);

ccsCOMPL_STAT slxDynBufAddEOS(slxDYN_BUF     *dynBuf,
			      ccsERROR       *error);

ccsCOMPL_STAT slxGetXBytesBuf(slxDYN_BUF     *dynBuf,
			      slxDYN_BUF_OP  initialize,
			      vltINT32       reqLen,
			      char           *buffer,
			      vltINT32       *bufLen,
			      ccsERROR       *error);

#ifdef __cplusplus
}
#endif

#endif /*!slxDynBuf_H*/
