/******************************************************************************
 * E.S.O. - VLT project
 *
 * "@(#) $Id: ftuGetMoonPars.C,v 1.1 2004/09/06 14:38:35 cguirao Exp $"
 *
 * Who       When        What
 * --------  ----------  ----------------------------------------------
 * jknudstr  06/04/2001  Created
 */

/************************************************************************
 *   NAME
 *   
 * 
 *   SYNOPSIS
 *   
 * 
 *   DESCRIPTION
 *
 *   FILES
 *
 *   ENVIRONMENT
 *
 *   COMMANDS
 *
 *   RETURN VALUES
 *
 *   CAUTIONS 
 *
 *   EXAMPLES
 *
 *   SEE ALSO
 *
 *   BUGS   
 * 
 *------------------------------------------------------------------------
 */
#define _POSIX_SOURCE 1

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define PI           3.14159265358979323846
#define MOON_PERIOD  29.5307

static char *rcsId="@(#) $Id: ftuGetMoonPars.C,v 1.1 2004/09/06 14:38:35 cguirao Exp $"; 
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/*
  ftuGetMoonPars -ra 51605.12631260 -24.6279 32466.336
  -> 296.890855423210

  ftuGetMoonPars -dec 51605.02631260 -24.6279 32466.336
  -> -19.820300411747

  ftuGetMoonPars -dist 156.473333 -42.152222 156.378154351414
                       -42.182421066729
  -> 0.076737296227

  ftuGetMoonPars -alt 51605.12631260 -24.6279 32466.336 
  -> -42.182421066729

  ftuGetMoonPars -az 51605.12631260 -24.6279 32466.336
  -> 156.378154351414
 */


/* Prototypes for functions in the skycalc module. */
extern "C" {
    void lpmoon(double jd,
		double lat,
		double sid,
		double *ra,
		double *dec,
		double *dist);

    double subtend(double ra1,
		   double dec1,
		   double ra2,
		   double dec2);

    double altit(double dec,
		 double ha,
		 double lat,
		 double *az);

    float lun_age(double  jd, 
		  int     *nlun);
}

void correctUsage(int ret = -1)
{
    printf("\nCorrect usage is:\n");
    printf("\n> ftuGetMoonPars -ra  <pars> | -dec <pars> | -dist  <pars> |");
    printf("\n                 -alt <pars> | -az  <pars> | -phase <pars>\n\n");
    printf("\nParameters:\n");

    printf("\n-ra:    <pars> := MJD-OBS TEL.GEOLAT LST");
    printf("\n-dec:   <pars> := MJD-OBS TEL.GEOLAT LST\n");

    printf("\n-dist:  <pars> := RA DEC GEN.MOON.RA GEN.MOON.DEC\n");

    printf("\n-alt:   <pars> := MJD-OBS TEL.GEOLAT LST");
    printf("\n-az:    <pars> := MJD-OBS TEL.GEOLAT LST\n");

    printf("\n-phase: <pars> := MJD-OBS\n\n");
    if (ret != -1) exit(ret);
}

int main(int argc, char *argv[])
{
    if (argc < 2) correctUsage(1);

    int i = 1;
    if ((strcmp(argv[i], "-ra") == 0) ||
	(strcmp(argv[i], "-dec") == 0))
	{
	if (argc != 5) correctUsage(1);
	char  par[8];
	strcpy(par, argv[i++]);
	double jd      = (atof(argv[i++]) + 2400000.5);
	double lat     = atof(argv[i++]);
	double sid     = (atof(argv[i]) / 3600.0);
	double moonRa  = 0;
	double moonDec = 0;
	double dist    = 0;
	lpmoon(jd, lat, sid, &moonRa, &moonDec, &dist);
	if (strcmp(par, "-ra") == 0)
	    printf("%.12f", (moonRa * 15.0));
	else
	    printf("%.12f", moonDec);
	}
    else if (strcmp(argv[i], "-dist") == 0)
	{
	if (argc != 6) correctUsage(1);
	i++;
	double ra1  = atof(argv[i++]);
	double dec1 = atof(argv[i++]);
	double ra2  = atof(argv[i++]);
	double dec2 = atof(argv[i]);
	double dist = (subtend((ra1 / 15.0), dec1, 
			       (ra2 / 15.0), dec2) * (180.0 / PI));
	printf("%.12f", dist);
	}
    else if ((strcmp(argv[i], "-alt") == 0) ||
	     (strcmp(argv[i], "-az") == 0))
	{
	if (argc != 5) correctUsage(1);
	char  par[8];
	strcpy(par, argv[i++]);
	double jd      = (atof(argv[i++]) + 2400000.5);
	double lat     = atof(argv[i++]);
	double sid     = (atof(argv[i]) / 3600.0);
	double moonRa  = 0;
	double moonDec = 0;
	double dist    = 0;
	lpmoon(jd, lat, sid, &moonRa, &moonDec, &dist);
	double moonAlt, moonAz;
	moonAlt = altit(moonDec, (sid - moonRa), lat, &moonAz);
	if (strcmp(par, "-alt") == 0)
	    printf("%.12f", moonAlt);
	else
	    printf("%.12f", moonAz);
	}
    else if (strcmp(argv[i], "-phase") == 0)
	{
	if (argc != 3) correctUsage(1);
	double jd = (atof(argv[++i]) + 2400000.5);
	double moonPhase;
	int    nLun;
	moonPhase = (double)(lun_age(jd, &nLun) / MOON_PERIOD);
	printf("%.12f", moonPhase);
	}
    else
	correctUsage(1);

    exit(0);
}

// --- oOo --- 







