void ffswap2(short *svalues,  /* IO - pointer to shorts to be swapped       */
             long nvals)      /* I  - number of shorts to be swapped        */
/*
  swap the bytes in the input short integers: ( 0 1 -> 1 0 )
*/
{
    register char *cvalues;
    register long ii;

    union u_tag {
        char cvals[2];   /* equivalence an array of 4 bytes with */
        short sval;      /* a short */
    } u;
    
    cvalues = (char *) svalues;      /* copy the initial pointer value */
    
    for (ii = 0; ii < nvals;)
	{
        u.sval = svalues[ii++];  /* copy next short to temporary buffer */
	
        *cvalues++ = u.cvals[1]; /* copy the 2 bytes to output in turn */
        *cvalues++ = u.cvals[0];
	}
    return;
}

void ffcsum(char          *buf,      
	    long          nrec,  /* I - number of 2880-byte blocks to sum  */
	    unsigned long *sum)  /* IO - accumulated checksum              */
/*
  Calculate a 32-bit 1's complement checksum of the FITS 2880-byte blocks.
  This routine is based on the C algorithm developed by Rob
  Seaman at NOAO that was presented at the 1994 ADASS conference,  
  published in the Astronomical Society of the Pacific Conference Series.
  This uses a 32-bit 1's complement checksum in which the overflow bits
  are permuted back into the sum and therefore all bit positions are
  sampled evenly. 
*/
{
    long ii, jj;
    unsigned short *sbuf;
    unsigned long hi, lo, hicarry, locarry;
    
    sbuf = (unsigned short *)buf;

    /*
      Sum the specified number of FITS 2880-byte records.  This assumes that
      the FITSIO file pointer points to the start of the records to be summed.
      Read each FITS block as 1440 short values (do byte swapping if needed).
    */
    for (jj = 0; jj < nrec; jj++)
	{
#if __linux__
	/* reverse order of bytes in each value */
	ffswap2((short *)(sbuf + (jj * 1440)), 1440); 
#endif
      
	hi = (*sum >> 16);
	lo = *sum & 0xFFFF;
      
	for (ii = 0; ii < 1440; ii += 2)
	    {
	    hi += sbuf[ii];
	    lo += sbuf[ii+1];
	    }
	
	hicarry = hi >> 16;    /* fold carry bits in */
	locarry = lo >> 16;
	
	while (hicarry | locarry)
	    {
	    hi = (hi & 0xFFFF) + locarry;
	    lo = (lo & 0xFFFF) + hicarry;
	    hicarry = hi >> 16;
	    locarry = lo >> 16;
	    }
      
	*sum = (hi << 16) + lo;
	}
}

void ffesum(unsigned long sum,    /* I - accumulated checksum                */
	    int           complm, /* I - = 1 to encode complement of the sum */
	    char *ascii)          /* O - 16-char ASCII encoded checksum      */
/*
  encode the 32 bit checksum by converting every 
  2 bits of each byte into an ASCII character (32 bit word encoded 
  as 16 character string).   Only ASCII letters and digits are used
  to encode the values (no ASCII punctuation characters).
  
  If complm=TRUE, then the complement of the sum will be encoded.
  
  This routine is based on the C algorithm developed by Rob
  Seaman at NOAO that was presented at the 1994 ADASS conference,
  published in the Astronomical Society of the Pacific Conference Series.
*/
{
    unsigned int exclude[13] = { 0x3a, 0x3b, 0x3c, 0x3d, 0x3e, 0x3f, 0x40,
				 0x5b, 0x5c, 0x5d, 0x5e, 0x5f, 0x60 };
    unsigned long mask[4] = { 0xff000000, 0xff0000, 0xff00, 0xff  };
    
    int offset = 0x30;     /* ASCII 0 (zero) */
    
    unsigned long value;
    int byte, quotient, remainder, ch[4], check, ii, jj, kk;
    char asc[32];
    
    if (complm)
        value = 0xFFFFFFFF - sum;   /* complement each bit of the value */
    else
        value = sum;
    
    for (ii = 0; ii < 4; ii++)
	{
        byte = (value & mask[ii]) >> (24 - (8 * ii));
        quotient = byte / 4 + offset;
        remainder = byte % 4;
        for (jj = 0; jj < 4; jj++)
            ch[jj] = quotient;
	
        ch[0] += remainder;
	
        for (check = 1; check;)   /* avoid ASCII  punctuation */
            for (check = 0, kk = 0; kk < 13; kk++)
                for (jj = 0; jj < 4; jj += 2)
                    if ((unsigned char) ch[jj] == exclude[kk] ||
                        (unsigned char) ch[jj+1] == exclude[kk])
			{
                        ch[jj]++;
                        ch[jj+1]--;
                        check++;
			}
	
        for (jj = 0; jj < 4; jj++)        /* assign the bytes */
            asc[4*jj+ii] = ch[jj];
	}
    
    for (ii = 0; ii < 16; ii++)       /* shift the bytes 1 to the right */
        ascii[ii] = asc[(ii+15)%16];
    
    ascii[16] = '\0';
}

