/******************************************************************************
 * E.S.O. - Archive System
 *
 * "@(#) $Id: ftuCONDITIONS.C,v 1.2.2.1 2008/09/11 08:11:40 cguirao Exp $"
 *
 * Who       When        What
 * --------  ----------  ----------------------------------------------
 * jknudstr  26/01/2000  Created 
 */

/************************************************************************
 *
 *   NAME
 *   ftuCONDITIONS - Class to contain the conditions for the translation.
 * 
 *   SYNOPSIS
 *   
 *   PARENT CLASS
 * 
 *   DESCRIPTION
 *   The class is used to contain the conditions defined by input
 *   parameters for carrying out a translation.
 *
 *   PUBLIC METHODS
 *   ftuCONDITIONS()
 *   ~ftuCONDITIONS()
 *   Constructor/destructor methods.
 *
 *   void            Force(const utilLOGICAL force)
 *   utilLOGICAL     Force()
 *   Set/get the value of the Force Translation parameter.
 *
 *   void            TestFiles(const utilLOGICAL testFiles)
 *   utilLOGICAL     TestFiles()
 *   Set/get the value of the Generate Test Files (<filename>_test.fits)
 *   parameter.
 *
 *   void            Checksum(const utilLOGICAL checksum)
 *   utilLOGICAL     Checksum()
 *   Set/get the value of the Generate Checksum property.
 *    
 *   void            Statistics(const utilLOGICAL statistics)
 *   utilLOGICAL     Statistics()
 *   Set/get the value of Generate Statistics Property.
 *
 *   void            History(const utilLOGICAL history)
 *   utilLOGICAL     History()
 *   Set/get the value of the Generate Change History Property.
 *
 *   void            Comment(const utilLOGICAL comment)
 *   utilLOGICAL     Comment()
 *   Set/get the value of Add FTU Comment Property.
 *
 *   void            Version(const char   *version)
 *   char            *Version()
 *   Set/get the value of the version string.
 *
 *   void            VersionDate(const char   *versionDate)
 *   char            *VersionDate()
 *   Set/get the value of the Version Data string.
 *
 *   void            SkipFileOnError(const utilLOGICAL skipFileOnError)
 *   utilLOGICAL     SkipFileOnError()
 *   Set/get the value of the Skip-File-On-Error property.
 *
 *   void            Tolerant(const utilLOGICAL tolerant)
 *   utilLOGICAL     Tolerant()
 *   Set/get the value indicating whether to run in Tolerant Mode.
 *  
 *   void            Overwrite(const utilLOGICAL overwrite)
 *   utilLOGICAL     Overwrite()
 *   Set/get the flag indicating whether files should be overwritten
 *   when existing.
 *
 *
 *   PUBLIC DATA MEMBERS
 *
 *   PROTECTED METHODS
 *
 *   PROTECTED DATA MEMBERS
 *
 *   FILES
 *
 *   ENVIRONMENT
 *
 *   RETURN VALUES
 *
 *   CAUTIONS 
 *
 *   EXAMPLES
 *
 *   SEE ALSO
 *
 *   BUGS   
 * 
 *------------------------------------------------------------------------
 */

static const char *rcsId="@(#) $Id: ftuCONDITIONS.C,v 1.2.2.1 2008/09/11 08:11:40 cguirao Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

#include <string.h>

#include "ftuCONDITIONS.h"
#include "ftuVersion.h"


ftuCONDITIONS::ftuCONDITIONS() 
{
    force_           = FALSE;
    testFiles_       = FALSE;
    checksum_        = TRUE;
    statistics_      = TRUE;  
    history_         = TRUE;
    comment_         = TRUE;
    skipFileOnError_ = FALSE;
    tolerant_        = FALSE;
    overwrite_       = FALSE;
    *targetDir_      = '\0';
    *targetFile_     = '\0';
    repeat_          = 1;

    Version(ftuVERSION); 
    VersionDate(ftuVERSION_DATE); 
    
    /* The format of the RCS ID is something like:
     *
     * rcsId="@(#) $Id: <filename>,v 1.11 2000/12/21 16:36:53 \
     *        dfsmgr Exp $";
     */
    /*
    slxKEYW_LINE     subStrings[16];
    vltINT32         noOfSubStr;
    utilSTAT          stat;
    stat = slxSplit(" ", rcsId, 16, subStrings, &noOfSubStr, &ErrStack());
    if ((stat == FAILURE) || (noOfSubStr < 9))
	{
	Version("?.?");
	VersionDate("XXXX-XX-XX");
	ErrStackReset();
	}
    else
	{
	Version(subStrings[3]); 
	VersionDate(subStrings[4]); 
	}
    */
}

/*___oOo___*/
