#ifndef ftuFTT_H
#define ftuFTT_H
/******************************************************************************
 *
 * E.S.O. - Archive System
 *
 * "@(#) $Id: ftuFTT.h,v 1.1 2004/09/06 14:38:35 cguirao Exp $"
 *
 * Who       When        What
 * --------  ----------  ----------------------------------------------
 * jknudstr  20/01/2000  Created
 */

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "oslxFITS.h"
#include "oslxDICTIONARY.h"
#include "utilPAF.h"
#include "utilFITS_VAL.h"
#include "utilCHRONOMETER.h"

#include "ftuBASE.h"
#include "ftuTRANSLATION.h"
#include "ftuCONDITIONS.h"

extern "C" {
    #include "mmap_i.h"

    ftuINT8  median_pixelvalue_INT8(ftuINT8  *a, int n);
    ftuINT16 median_pixelvalue_INT16(ftuINT16 *a, int n);
    ftuINT32 median_pixelvalue_INT32(ftuINT32 *a, int n);
    ftuFP32  median_pixelvalue_FP32(ftuFP32  *a, int n);
    ftuFP64  median_pixelvalue_FP64(ftuFP64  *a, int n);
}

#define ftuSIMPLE_LINE   "SIMPLE  =                    T / Standard FITS format (NOST-100.0)                          "
#define ftuXTENSION_LINE "XTENSION=  'IMAGE  '           / Extension first keyword                                    "

#define ftuHDU_BLOCK   2880
#define ftuFITS_CARD   80


class fttFILENAME_SET
{
  public:
    fttFILENAME_SET  &SetTargetFileName(const slxFILENAME  fileName) {
	strcpy(targetFileName_, fileName);
	return *this;
    }

    char              *GetTargetFileName() {
	return targetFileName_;
    }
    
    fttFILENAME_SET  &SetTmpFileName(const slxFILENAME  fileName) {
	strcpy(tmpFileName_, fileName);
	return *this;
	}

    char              *GetTmpFileName() {
	return tmpFileName_;
    }
    
  private:
    slxFILENAME  targetFileName_;
    slxFILENAME  tmpFileName_;
};

// Callback to handle deletion of the fttFILENAME_SET class.
void fttFileNameSetHashDeleteEl(void *elPtr);
class fttFILENAME_SET_HASH: public utilHASH
{
  public:
    fttFILENAME_SET_HASH() {
	SetDeleteElFct(fttFileNameSetHashDeleteEl);
    }
};

// Callback to handle deletion of the oslxFITS class.
void fttOslxFitsDeleteEl(void *elPtr);
class fttFITS_HASH: public utilHASH
{
  public:
    fttFITS_HASH() {
	SetDeleteElFct(fttOslxFitsDeleteEl);
    }
};

#define ftuGET_SRC_HDR(hdrPtr) \
{ \
    if ((hdrPtr = fits.GetHdr(translation.GetSrcHdr() + 1)) == NULL) \
	{ \
	sprintf(errBuf_, "Error accessing header number %d in FITS file " \
		"%s.", translation.GetSrcHdr(), fits.FitsFileName()); \
	ErrAdd("ftu", ftuERR_GEN, utilERR_WARNING, errBuf_); \
	goto skip; \
	} \
}

#define ftuGET_TARG_HDR(hdrPtr) \
{ \
    if ((hdrPtr = fits.GetHdr(translation.GetTargHdr() + 1)) == NULL) \
	{ \
	sprintf(errBuf_, "Error accessing header number %d in FITS file " \
		"%s.", translation.GetTargHdr(), fits.FitsFileName()); \
	ErrAdd("ftu", ftuERR_GEN, utilERR_WARNING, errBuf_); \
	goto skip; \
	} \
}

class ftuFTT: public ftuBASE
{
  public:
    ftuFTT(ftuCONDITIONS  &conditions);
    ~ftuFTT();

    ftuCONDITIONS    &Conditions();

    char             *TestFileTag();

    utilSTAT         Load(const slxFILENAME  fttFile,
			  const int          recursiveCall);

    int              NoOfTranslations();

    ftuTRANSLATION   *GetTranslation(const int number);

    utilSTAT         Translate(const slxFILENAME    fileName);

    char             *ConcatString(const int          noOfValues,
				   const slxVALUE     pars[]);
    char             *AndLogical(const int            noOfValues,
				 const slxVALUE       pars[]);
    char             *OrLogical(const int             noOfValues,
				const slxVALUE        pars[]);
    char             *SumInteger(const int            noOfValues,
				 const slxVALUE       pars[]);
    char             *MultiplyInteger(const int       noOfValues,
				      const slxVALUE  pars[]);
    char             *SumDouble(const int             noOfValues,
				const slxVALUE        pars[]);
    char             *MultiplyDouble(const int        noOfValues,
				     const slxVALUE   pars[]);
    char             *Ut2IsoTime(const slxVALUE  ut);

    utilSTAT         Secs2Time(const double          ut,
			       const vltINT32        precision,
			       slxKEYW_LINE          utcTime);   

    utilSTAT          Date(const char  *date);
    char              *Date();

    char              *FttFileName();

    ftuFITS_HDR_TYPES HeaderType(oslxFITS   &fitsObj);

    utilSTAT          ImageInfo(oslxFITS   &fits,
				int        *imageSize,
				int        *totalImageSize,
				int        *bitPix,
				int        *naxis,
				double     *bscale,
				double     *bzero);

    utilSTAT          GetDataSize(oslxFITS               &fits,
				  int                    *dataSize);

    utilSTAT          GenStatistics(const int   bitPix,
				    const char  *buf,
				    const int   len,
				    double      bscale,
				    double      bzero,
				    double      *min,
				    double      *max,
				    double      *mean,
				    double      *rms,
				    double      *sum);
    
    utilLOGICAL        Split(oslxFITS  &fitsObj);

    utilLOGICAL        CreateTargetFitsFile();

    void               SetDumpHdr(const ftuHDR_DUMP_TYPES dump);
    ftuHDR_DUMP_TYPES  GetDumpHdr();

    utilSTAT           DumpFitsHeader(oslxFITS           &fitsObj,
				      const slxFILENAME  targetFileName);

    ccsCOMPL_STAT      CopyFitsObj(oslxFITS  &srcObj,
				   oslxFITS  &trgObj);

  protected:
  private:
    void             ResetNoOfTransCount();
    void             IncNoOfTransCount();
    int              GetNoOfTransCount();

    utilSTAT         AddFitsExt(const slxFILENAME orgFileName,
				slxFILENAME       tmpFileName);
    utilSTAT         RenameFile(const slxFILENAME  srcFileName,
				const slxFILENAME  targFileName,
				const int          issueWarning);

    utilSTAT         BuildFttDic();
    utilSTAT         BuildFitsDic();

    utilSTAT         CheckKeyTypes(const char *srcKey,
				   const char *targKey);

    utilSTAT         CheckKeyAndValTypes(const char    *key,
					 const char    *value,
					 utilLOGICAL   *match = NULL);

    utilLOGICAL      AllowKeyRemove(const char *key);

    utilSTAT         HandleIncludeKey(utilPAF &fttPaf); 
    utilSTAT         HandleDictionaryKey(utilPAF &fttPaf);
    utilSTAT         HandleDicKey(utilPAF &fttPaf);
    utilSTAT         HandleAddKey(utilPAF &fttPaf);
    utilSTAT         HandleChangeKey(utilPAF &fttPaf);
    utilSTAT         HandleRenameKey(utilPAF &fttPaf);
    utilSTAT         HandleMoveKey(utilPAF &fttPaf);
    utilSTAT         HandleCopyKey(utilPAF &fttPaf);
    utilSTAT         HandleRemoveKey(utilPAF &fttPaf);

    utilSTAT         HandleHdrKey(utilPAF &fttPaf);

    utilSTAT         HandleFileSplitRenamePathKey(utilPAF          &fttPaf,
						  ftuTRANSLATION   &trans,
						  const ftuACTION  transId,
						  const char       *initKey);

    utilSTAT         HandleFileSplitKey(utilPAF &fttPaf);

    utilSTAT         HandleFileRenameKey(utilPAF &fttPaf);

    utilSTAT         HandleFilePathKey(utilPAF &fttPaf);

    utilSTAT         HandleActionSkip(const char  *errMsg);

    utilSTAT         HandleActionAddChange(oslxFITS        &fits, 
					   ftuTRANSLATION  &translation,
					   ftuACTION       action);
    utilSTAT         HandleActionAdd(oslxFITS        &fits, 
				    ftuTRANSLATION  &translation);
    utilSTAT         HandleActionChange(oslxFITS        &fits, 
					ftuTRANSLATION  &translation);
    utilSTAT         HandleActionRename(oslxFITS        &fits, 
					ftuTRANSLATION  &translation);
    utilSTAT         HandleActionMove(oslxFITS        &fits, 
				      ftuTRANSLATION  &translation);
    utilSTAT         HandleActionCopy(oslxFITS        &fits, 
				      ftuTRANSLATION  &translation);
    utilSTAT         HandleActionRemove(oslxFITS        &fits, 
					ftuTRANSLATION  &translation);
    
    utilSTAT         GetValue(oslxFITS        &fits, 
			      const int       hdrNo,
			      const int       scanHdrs, 
			      ftuVALUE_SRC    &valueSource,
			      slxVALUE        value,
			      ftuTRANSLATION  &translation);
    
    utilSTAT         GenValue(oslxFITS         &fits, 
			      slxKEYW_LINE     targKey,
			      ftuTRANSLATION   &translation,
			      utilFITS_VAL     &fitsVal,
			      slxVALUE         rawValue);
    
    utilSTAT         GetFctValue(const char  *function,
				 slxVALUE    result);
    
    utilSTAT         ExecPlugIn(const char  *cmd,
				slxVALUE    result);

    utilSTAT         ComputeValue(const char     *function,
				 const int       noOfValues,
				 const slxVALUE  values[],
				 slxVALUE        result);

    utilSTAT         CheckExpKey(const slxKEYW_LINE  expKeys,
				 const slxKEYW_LINE  actualKey);

    utilSTAT         AddValueSrcs(utilPAF          &fttPaf,
				  ftuTRANSLATION   *transEl);

    void             TrimFileName(slxFILENAME  fileName);

    void             SubstituteChr(char   *str,
				   char   srcChr,
				   char   trgChr);

    int              SetFileIndex(const int   index);
    int              IncrFileIndex();
    void             ResetFileIndex();
    int              GetFileIndex();

    utilSTAT         GenFileName(const slxFILENAME  path,
				 const slxFILENAME  orgFileName,
				 const slxFILENAME  ext,
				 const int          hdrNo,
				 const int          scanHdrs,
				 oslxFITS           &fits,
				 ftuTRANSLATION     &translation);

    utilSTAT         GenFileNames(oslxFITS            &fits,
				  const slxFILENAME   fileName);

    utilSTAT         FindNextIndex(const slxFILENAME   path,
				   const slxFILENAME   baseFilename,
				   const slxFILENAME   ext,
				   int                 *index,
				   slxFILENAME         indexStr);

    utilSTAT SkipHeader_(mmap_file             *mmap,
			 const int             loops,
			 unsigned int          *mmapIdx);

    utilSTAT         GenerateNewFile(oslxFITS            &fits,
				     const slxFILENAME   fileName);

    utilSTAT         CreateTmpFile(oslxFITS             &fits,
				   const slxFILENAME    fileName);
    
    utilSTAT         AddHistory(const char     *action,
				oslxFITS       *fitsObj,
				const char     *keyValSrc,
				const char     *keyActionDescr);

    int                   noOfTransPerf_;

    oslxDICTIONARY        fttDic_;
    ftuTRANSLATION_HASH   fttTbl_;
    fttFILENAME_SET_HASH  fileNames_;
    fttFITS_HASH          fitsObjs_;
    ftuHDR_DUMP_TYPES     dumpHdr_;
    ftuTRANSLATION        rename_;
    ftuTRANSLATION        split_;
    int                   hduList_[64], splitSelHdus_;
    ftuTRANSLATION        path_;
    oslxDICTIONARY        fitsDicStat_;
    oslxDICTIONARY        fitsDicDyn_;
    slxVALUE              date_;
    ftuCONDITIONS         *conditions_;
    slxFILENAME           fttFileName_;
    utilCHRONOMETER       chronometer_;
    int                   fileIndex_;
};

#define oslxBLANK_BUF_2880_END \
"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                END                                                                             "

#endif /*!ftuFTT_H*/

// --- oOo ---
