/****************************************************************************
 * E.S.O. - Archive System
 *
 * "@(#) $Id: ftuFTT_File.C,v 1.3.2.1 2008/09/11 08:11:40 cguirao Exp $"
 *
 * Who       When        What
 * --------  ----------  ----------------------------------------------
 * jknudstr  20/01/2000  Created 
 */

/************************************************************************
 *
 *  NAME
 *  ftuFTT_File - Contains methods to handle file operations during the
 *                translation. 
 *
 *  SYNOPSIS
 *   
 *  PARENT CLASS
 * 
 *  DESCRIPTION
 *
 *  PUBLIC METHODS
 *
 *  PUBLIC DATA MEMBERS
 *
 *  PROTECTED METHODS
 *
 *  PROTECTED DATA MEMBERS
 *
 *  FILES
 *
 *  ENVIRONMENT
 *
 *  RETURN VALUES
 *
 *  CAUTIONS 
 *
 *  EXAMPLES
 *
 *  SEE ALSO
 *
 *  BUGS   
 * 
 *------------------------------------------------------------------------
 */

static const char *rcsId="@(#) $Id: ftuFTT_File.C,v 1.3.2.1 2008/09/11 08:11:40 cguirao Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

#define _ALL_SOURCE 1
#define __EXTENSIONS__ 1

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "utilPAF.h"

#include "ftuDefs.h"
#include "ftuFTT.h"
#include "ftuTRANSLATION.h"
#include "ftuSwap.h"


char *GenIndexStr_(const int  idx)
{
    static char  tmpIndex[16];

    if (idx < 10)
	sprintf(tmpIndex, "000%d", idx);
    else if (idx < 100)
	sprintf(tmpIndex, "00%d", idx);
    else if (idx < 1000)
	sprintf(tmpIndex, "0%d", idx);
    else 
	sprintf(tmpIndex, "%d", idx);

    return tmpIndex;
}


utilSTAT ftuFTT::FindNextIndex(const slxFILENAME   path,
			       const slxFILENAME   baseFilename,
			       const slxFILENAME   ext,
			       int                 *index,
			       slxFILENAME         indexStr)
{
    int           idx = *index;
    slxFILENAME   tmpFilename; 

    do
	{
	idx++;
	strcpy(indexStr, GenIndexStr_(idx));
	sprintf(tmpFilename, "%s/%s_%s.%s", path, baseFilename, indexStr, ext);
	}
    while (utilFILE_MGR::FileExists(tmpFilename));
    *index = idx;

    return SUCCESS;
}


void ftuFTT::TrimFileName(slxFILENAME  fileName)
{
    // Trim string + clean up special characters (convert to underscore).
    slxTrimString(fileName, " \"");
    int   idx = 0;
    while (*(fileName + idx) != '\0')
	{
	if (strchr(" !@#$%^&*()+=|~;<>,?\"\n", *(fileName + idx)) != NULL)
	    *(fileName + idx) = '_';
	idx++;
	}
}


char  *ftuFTT::TestFileTag()
{
    if (Conditions().TestFiles())
	return "_test";
    else
	return "";
}


int ftuFTT::SetFileIndex(const int   index)
{
    fileIndex_ = index;
    return GetFileIndex();
}


int ftuFTT::IncrFileIndex()
{
    SetFileIndex(GetFileIndex() + 1);
    return GetFileIndex();
}


void ftuFTT::ResetFileIndex()
{
    SetFileIndex(0);
}


int ftuFTT::GetFileIndex()
{
    return fileIndex_;
}


utilSTAT ftuFTT::GenFileName(const slxFILENAME   path,
			     const slxFILENAME   orgFileName,
			     const slxFILENAME   ext,
			     const int           hdrNo,
			     const int           scanHdrs,
			     oslxFITS            &fits,
			     ftuTRANSLATION      &translation)
{
    ftuVALUE_SRC   *valueSource;
    slxFILENAME    targetFileName, extLoc;
    slxVALUE       value;

    strcpy(extLoc, ext);


    // Check if we need to generate a new target directory.
    slxFILENAME  targetPath;
    *targetPath = '\0';
    if ((*(Conditions().TargetDir()) != '\0') || 
	(path_.GetAction() == ftuFILE_PATH))
	{
	slxFILENAME  tmpPath;

	// If a path is specified with the -targetDir command line parameter,
	// take this path, otherwise get the info about this from the FTT.
	if (*(Conditions().TargetDir()) != '\0')
	    strcpy(tmpPath, Conditions().TargetDir());
	else
	    {
	    // Get the base (can be empty).
	    if (path_.GetValue(0, &valueSource) == FAILURE)
		return FAILURE;
	    strcpy(tmpPath, valueSource->GetReference());
	    slxTrimString(tmpPath, " \n\t");
	    
	    // Append the additional values from the Value Sources specified.
	    for (int transNo = 1; transNo < path_.NoOfValues(); transNo++)
		{
		if (path_.GetValue(transNo, &valueSource) == FAILURE)
		    return FAILURE;
		if (GetValue(fits, hdrNo, scanHdrs, *valueSource, 
			     value, path_) == FAILURE)
		    return FAILURE;
		slxTrimString(value, " \n\t");
		strcat(tmpPath, value);
		}
	    }

	// If nothing is stored in the target path by now, we set the 
	// target directory to $PWD.
	slxTrimString(tmpPath, " \n\t");
	if (*tmpPath == '\0')
	    strcpy(tmpPath, "$PWD");
	
	// Resolve the path and clean it up.
	char   *completePath;
	if (slxResolvePath(tmpPath, &completePath, &ErrStack()) == FAILURE)
	    return FAILURE;
	strcpy(targetPath, completePath);
	slxTrimString(targetPath, " \n\t");
	TrimFileName(targetPath);
	if (slxGetLastChar(targetPath) == '/')
	    *(targetPath + strlen(targetPath) - 1) = '\0';
	}
    else
	{
	// If no specific target path is specfied, we simply use the path
	// of the source file.
	strcpy(targetPath, path);
	}
    INFO(2,"Target Path is: %s.",targetPath);

    // Generate the filename(s).
    int len;
    len = 0;
    *targetFileName = '\0';
    if (*(Conditions().TargetFile()) != '\0')
	{
	// Split the target filename specified and set the extension to the
	// extension of that file (if any). The path should not be set.
	slxFILENAME    tmpPath, fn;
	utilFILE_MGR::SplitFileName(Conditions().TargetFile(), tmpPath, fn,
				    extLoc);
	strcpy(targetFileName, fn);
	}
    else if (&translation != &path_)
	{
	if (translation.GetValue(0, &valueSource) == FAILURE)
	    return FAILURE;
	if (strcmp(valueSource->GetReference(), "$FILENAME") == 0)
	    strcat(targetFileName, orgFileName);
	else
	    strcat(targetFileName, valueSource->GetReference());
	for (int transNo = 1; transNo < translation.NoOfValues(); transNo++)
	    {
	    if (translation.GetValue(transNo, &valueSource) == FAILURE)
		return FAILURE;
	    if (GetValue(fits, hdrNo, scanHdrs, *valueSource, value,
			 translation) == FAILURE)
		{
		// We give it one more try:
		//
		// If the key wasn't found in the present header worked with,
		// we should try to find it in the merge of the present
		// header (if > primary header) with the primary header. 
		// Instead of merging we simply try to extract it from the 
		// primary header.
		int errorExit;
		errorExit = TRUE;
		if (hdrNo > 0)
		    {
		    if (GetValue(fits, 0, scanHdrs, *valueSource, value,
				 translation) == SUCCESS)
			{
			ClearStack();
			errorExit = FALSE;
			}
		    }   // end-if (hdrNo > 0)

		// In case we are running in tolerant mode we do not
		// bail out in case we fail to query the value of a 
		// keyword but we set the value to a string of 
		// length 0.
		if (errorExit && 
		    (Severity() <= utilERR_WARNING) && Conditions().Tolerant())
		    {
		    *value = '\0';
		    LogFlushError(Severity(), *(utilERROR *)this);
		    INFO(1,"Tolerant Mode used - ignoring WARNING(s).");
		    ClearStack();
		    }
		else if (errorExit)
		    return FAILURE;
		}   // end-if (GetValue(fits, hdrNo, *valueSource, value, ...

	    // Allow max. 128 characters in the filename. A 'cheap' solution
	    // to the problem described in DFS00728.
	    if ((len += strlen(value)) <= 128)
		strcat(targetFileName, value);
	    else
		{
		ErrAdd("ftu", ftuERR_GEN, utilERR_FATAL, "Length of "
		       "generated filename too long.");
		return FAILURE;
		}
	    }   // end-for (int transNo = 1; transNo < ...
	}   // end-else if (&translation != &path_)
    else
	strcpy(targetFileName, orgFileName);

    // Clean up filename + remove slashes from filenames.
    TrimFileName(targetFileName);
    SubstituteChr(targetFileName, '/', '_');

    // Now, generate the final target filename (include the path).
    slxFILENAME      completeFilenameNoExt, completeFilename;
    sprintf(completeFilenameNoExt, "%s/%s%s", targetPath, targetFileName,
	    TestFileTag());
    if (*extLoc != '\0')
	sprintf(completeFilename, "%s.%s", completeFilenameNoExt, extLoc);
    else
	sprintf(completeFilename, "%s", completeFilenameNoExt);

    // This loop iterates until it finds a unique name for the target files
    // and also unique names for the temporary target files. In case test
    // files should be generated, we don't care about overwriting files 
    // already stored on the disk.
    char   indexStr[16];
    int    checkLevel = 0;
    // -- if we are running in Overwrite Mode we do not care about this aspect.
    while (!Conditions().Overwrite() && (checkLevel < 2))
	{
	// Check that this name is not used for one of the other files 
	// already registered as target files. We keep on increasing the
	// index until a unique name is found.
	fttFILENAME_SET  *tmpFileSetPtr; 
	if (fileNames_.GetSeq(TRUE, (void **)&tmpFileSetPtr) == FAILURE)
	    return FAILURE;
	for (int fileNo = 1; fileNo <= fileNames_.NoOfEls(); fileNo++)
	    {
	    if (fileNames_.GetSeq(FALSE, (void **)&tmpFileSetPtr) == FAILURE)
		return FAILURE;
	    if (strcmp(tmpFileSetPtr->GetTargetFileName(),
		       completeFilename) == 0)
		{
		// Increase the index and start the loop again.
		sprintf(completeFilenameNoExt, "%s/%s%s_%s", targetPath, 
			targetFileName, TestFileTag(), 
			GenIndexStr_(IncrFileIndex()));
		sprintf(completeFilename, "%s.%s", completeFilenameNoExt,
			extLoc);
		if (fileNames_.GetSeq(TRUE,(void **)&tmpFileSetPtr)==FAILURE)
		    return FAILURE;
		fileNo = 0;
		}
	    }   // end-for (int fileNo = 0; fileNo < ...
	checkLevel++;
	
	// Check if a file with that name exists in the target directory.
	if (utilFILE_MGR::FileExists(completeFilename))
	    {
	    checkLevel--;
	    // We loop until we find an unused filename.
	    do
		{
		// Find the next free index and generate filename with the
		// index appended.
		int tmpIndex;
		tmpIndex = GetFileIndex();
		slxFILENAME  tmpTargetFileName;
		sprintf(tmpTargetFileName,"%s%s",targetFileName,TestFileTag());
		if (FindNextIndex(targetPath, tmpTargetFileName, extLoc, 
				  &tmpIndex, indexStr) == FAILURE)
		    return FAILURE;
		SetFileIndex(tmpIndex);
		sprintf(completeFilenameNoExt, "%s/%s%s_%s", targetPath, 
			targetFileName, TestFileTag(), indexStr);
		sprintf(completeFilename, "%s.%s",
			completeFilenameNoExt, extLoc);
		}  
	    while (utilFILE_MGR::FileExists(completeFilename));
	    }   // end-if (utilFILE_MGR::FileExists(completeFilename))
	else
	    checkLevel++;
	}   // end-while (checkLevel < 2)
    fttFILENAME_SET  *fileSetPtr = new(fttFILENAME_SET);
    fileSetPtr->SetTargetFileName(completeFilename);

    // - and the temporary target filename.
    slxFILENAME  completeTmpFilename;
    if (*extLoc != '\0')
	sprintf(completeTmpFilename, "%s_tmp.%s",completeFilenameNoExt,extLoc);
    else
	sprintf(completeTmpFilename, "%s_tmp", completeFilenameNoExt);
    fileSetPtr->SetTmpFileName(completeTmpFilename);

    INFO(3,"Generated final target filename: %s.",
	 fileSetPtr->GetTargetFileName());

    if (fileNames_.AddEl(hdrNo, (void **)&fileSetPtr) == FAILURE)
	return FAILURE;
 
    return SUCCESS;
}


utilSTAT ftuFTT::GenFileNames(oslxFITS               &fits,
			      const slxFILENAME      orgFileName)
{  
    slxFILENAME    path, fn, ext;
    utilFILE_MGR::SplitFileName(orgFileName, path, fn, ext);

    fileNames_.DeleteTable();

    INFO(2,"Generating filenames ...");
    chronometer_.Start("GenerateFileNames");

    // If Splitting is on we have to loop over all the extensions to generate
    // the filenames.
    if (Split(fits))
	{
	int    ext0DataSize;
	if (GetDataSize(fits, &ext0DataSize) == FAILURE)
	    return FAILURE;
	// If there is data in connection with the first extension (= 0)
	// we have to generate a filename for this as well.
	int  startHdr, selHdusCount;
	if (splitSelHdus_)
	    startHdr = 0;
	else if (ext0DataSize == 0)
	    startHdr = 1;
	else
	    startHdr = 0; 
	selHdusCount = 0;
	for (int hdrNo = startHdr; hdrNo < fits.NoOfHdrs(); hdrNo++)
	    {
	    // If we're splitting and if we're only considering a selected
	    // number of HDUs we take only these into account.
	    if (splitSelHdus_)
		{
		if (hdrNo != hduList_[selHdusCount])
		    continue;
		else
		    selHdusCount++;
		}
	    if (GenFileName(path, fn, ext, hdrNo, ftuNO_HDR_SCAN, fits, 
			    split_) == FAILURE)
		return FAILURE;
	    }   // end-for (int hdrNo = startHdr; hdrNo <= ... 
	}   // end-if (Split(fits))
    else if ((rename_.GetAction() == ftuFILE_RENAME) ||
	     (*(Conditions().TargetFile()) != '\0'))
	{
	if (GenFileName(path, fn, ext, 0, ftuNO_HDR_SCAN, fits, 
			rename_) == FAILURE)
	    return FAILURE;
	}
    else if ((path_.GetAction() == ftuFILE_PATH) ||
	     (*(Conditions().TargetDir()) != '\0'))
	{
	if (GenFileName(path, fn, ext, 0, ftuNO_HDR_SCAN, fits, 
			path_) == FAILURE)
	    return FAILURE;
	}
    else
	{
	// No File Splitting or Renaming is on. Or Splitting is on, but
	// the file is not using the extension.
	fttFILENAME_SET  *fileSetPtr = new(fttFILENAME_SET);
	slxFILENAME  fileName;
	if (!Conditions().TestFiles())
	    sprintf(fileName, "%s/%s.%s", path, fn, ext);
	else
	    sprintf(fileName, "%s/%s_test.%s", path, fn, ext);
	fileSetPtr->SetTargetFileName(fileName);
	sprintf(fileName, "%s/%s_tmp.%s", path, fn, ext);
	fileSetPtr->SetTmpFileName(fileName);
	if (fileNames_.AddEl(0, (void **)&fileSetPtr) == FAILURE)
	    return FAILURE;
	}

    INFO(2,"Generated filenames. Time: %.3fs.", 
	 chronometer_.Stop("GenerateFileNames"));

    return SUCCESS;
}


// Ensure that the file has extension ".fits" -> workaround 
// since OSLX enforces extension ".fits".
utilSTAT         ftuFTT::AddFitsExt(const slxFILENAME orgFileName,
				    slxFILENAME       tmpFileName)
{
    slxFILENAME  path, fn, ext;
    utilFILE_MGR::SplitFileName(orgFileName, path, fn, ext);
    strcpy(tmpFileName, orgFileName);
    if (strcmp(ext, "fits") != 0)
	{
	strcat(tmpFileName, ".fits");
	//if (rename(orgFileName, tmpFileName) != 0)
	if (RenameFile(orgFileName, tmpFileName, 0) == FAILURE)
	    {
	    sprintf(errBuf_, "Error renaming the file: %s to name: %s.",
		    orgFileName, tmpFileName);
	    ErrSysAdd("ftu", ftuERR_GEN, utilERR_FATAL, errBuf_);
	    return FAILURE;
	    }	 
	}

    return SUCCESS;
}


utilSTAT         ftuFTT::RenameFile(const slxFILENAME  srcFileName,
				    const slxFILENAME  targFileName,
				    const int          issueWarning)
{
    if (strcmp(srcFileName, targFileName) == 0)
	return SUCCESS;

    struct stat statBuf;
    if (stat(targFileName, &statBuf) == 0)
	{
	// Check if we can create the target file, i.e. if it already exists
	// and is writable.
	int canWrite = 0;
	// - World writable?
	if (statBuf.st_mode & S_IWOTH)
	    canWrite = 1;
	else 
	    {
	    uid_t ftuUid = geteuid();
	    // - Writable for user/group?
	    if (((ftuUid == statBuf.st_uid) && 
		 (statBuf.st_mode & S_IWUSR)) ||
		((ftuUid == statBuf.st_gid) && (statBuf.st_mode & S_IWGRP)))
		canWrite = 1;
	    }
	if (!canWrite)
	    {
	    sprintf(errBuf_, "Do not have permissions to overwrite existing "
		    "Target File: %s. Source File: %s.", 
		    targFileName, srcFileName);
	    ErrAdd("ftu", ftuERR_GEN, utilERR_WARNING, errBuf_);
	    return FAILURE;
	    }

	// Check if we should issue a warning if the target file already 
	// exists.
	if (issueWarning)
	    {
	    INFO(1,"WARNING: Target File %s already exists - overwriting "
		 "anyway.",targFileName);
	    }
	}   // end-if (stat(targFileName, &statBuf) == 0)

    // There is a problem with rename within the Beowulf Cluster.
    // if (rename(srcFileName, targFileName) != 0)
#ifdef __linux__
    char  shellCmd[512];
    sprintf(shellCmd, "mv -f %s %s", srcFileName, targFileName);
    if (system(shellCmd) == -1)
	{
	sprintf(errBuf_, "Error renaming the file: %s to name: %s.",
		srcFileName, targFileName);
	ErrSysAdd("ftu", ftuERR_GEN, utilERR_FATAL, errBuf_);
	return FAILURE;
	}	
#else
    if (rename(srcFileName, targFileName) == -1)
	{
	sprintf(errBuf_, "Error renaming the file: %s to name: %s.",
		srcFileName, targFileName);
	ErrSysAdd("ftu", ftuERR_GEN, utilERR_FATAL, errBuf_);
	return FAILURE;
	}	
#endif

    return SUCCESS;  
}


utilSTAT         ftuFTT::GenerateNewFile(oslxFITS             &fits,
					 const slxFILENAME    filename)
{
    if (GenFileNames(fits, filename) == FAILURE)
	return FAILURE;

    INFO(2,"Updating target file(s) with changes ...");

    // Delete the table in which we store the FITS objects.
    if (fitsObjs_.DeleteTable() == FAILURE)
	return FAILURE;

    // Create the temporary file. During this process we create the new
    // file, i.e., prepare the header block(s) and copy over the data.
    if (CreateTmpFile(fits, filename) == FAILURE)
	return FAILURE;

//     //////////////////////////////////////////////////////////////////////
//     // DFS02285 (see ftuFTT_Trans.C): Fix some issues in primary and
//     //                                extension headers concerning the keys
//     //                                SIMPLE and XTENSION.
//     //////////////////////////////////////////////////////////////////////
//     fits.GetKeyword("NAXIS");
//     int naxis = fits.CK().Integer();
//     char  tmpKey[64];
//     for (int n = 1; n <= 4; n++)
// 	{
// 	if (n > naxis)
// 	    {
// 	    sprintf(tmpKey, "NAXIS%d", n);
// 	    fits.RemoveKeyword(tmpKey);
// 	    }
// 	}

//     // Primary Header: Ensure SIMPLE=T and that there is no XTENSION 
//     // keyword.
//     if (fits.GetKeyword("SIMPLE") == FAILURE) return FAILURE;
//     if (!fits.CK().Logical()) fits.CK().Logical(TRUE);
//     if (fits.KeyInBuffer("XTENSION")) fits.RemoveKeyword("XTENSION");

//     // For extension headers, ensure that there is no SIMPLE key.
//     oslxFITS *tmpFitsPtr;
//     for (int hdu = 1; hdu <= fits.NoOfHdrs(); hdu++)
// 	{
// 	tmpFitsPtr = fits.GetHdr(hdu);
// 	if (tmpFitsPtr->KeyInBuffer("SIMPLE"))  
// 	    tmpFitsPtr->RemoveKeyword("SIMPLE");
// 	}
//     fits.DumpKeywords(); // >>>>
    //////////////////////////////////////////////////////////////////////

    // Loop over the temporary target files and finish them up.
    oslxFITS         *fitsObj;
    fttFILENAME_SET  *fileSetPtr;
    int              startHdr, stopHdr;
    int              hdrDataSize, selHdusCount;
    if (GetDataSize(fits, &hdrDataSize) == FAILURE)
	return FAILURE;

    // If we're splitting and a selected number of HDUs is specified we
    // should only consider this.
    if (splitSelHdus_)
	{
	startHdr = 0;
	stopHdr = fits.NoOfHdrs();
	}
    else if (Split(fits) && (hdrDataSize == 0))
	{
	startHdr = 1;
	stopHdr = fits.NoOfHdrs();
	}
    else if (Split(fits) && (hdrDataSize > 0))
	{
	startHdr = 0;
	stopHdr = fits.NoOfHdrs();
	}
    else
	{
	startHdr = 0;
	stopHdr = 1;
	}
    selHdusCount = 0;
    for (int fileNo = startHdr; fileNo < stopHdr; fileNo++)
	{
	// If we're splitting and if we're only considering a selected
	// number of HDUs we take only these into account.
	if (splitSelHdus_)
	    {
	    if (fileNo != hduList_[selHdusCount])
		continue;
	    else
		selHdusCount++;
	    }

	// Get reference to correct FITS object.
	if (fits.NoOfHdrs() == 1)
	    fitsObj = &fits;
	else if (Split(fits))
	    {
	    if (fitsObjs_.GetEl(fileNo, (void **)&fitsObj) == FAILURE)
		return FAILURE;
	    }
	else
	    fitsObj = &fits;

	// Get the next temporary target filename and target filename.
	if (fileNames_.GetEl(fileNo, (void **)&fileSetPtr) == FAILURE)
	    return FAILURE;
	INFO(2,"Doing final processing of temporary "
	     "target file: %s.", fileSetPtr->GetTmpFileName());

	// Store the COMMENT indicating that this file has been translated
	// with this FTT.
	if (Conditions().Comment())
	    {
	    slxVALUE comBuf, dayTime;
	    slxGenDayTim(dayTime);
	    sprintf(comBuf, "FTU-%s/%s/%s", Conditions().Version(), dayTime,
		    FttFileName());
	    INFO(2,"Adding FTU translation comment: %s.",comBuf);
	    if (fitsObj->AddComment(comBuf) == FAILURE)
		return FAILURE;
	    }

	// The CHECKSUM part removes the COMMENT, HISTORY (and ESO-LOG)
	// entries from the FITS header.
	oslxFITS  dumpHdr;
	if (GetDumpHdr())
	    {
	    // IMPL.: Due to that oslxFITS::Copy() does not copy all headers
	    // over, so for the moment we have to load in file again.
	    // --> Use internally provided ftuFTT::CopyFitsObj().
	    /*
	      dumpHdr.SetDictionary(&fitsDicDyn_);
	      dumpHdr.AutoAddKeysInDic(TRUE);	    
	      if (dumpHdr.Copy(*fitsObj) == FAILURE)
	      return FAILURE;
	    */
	    chronometer_.Start("LoadFitsHdrsInDumpHdrObj");
	    dumpHdr.SetDictionary(&fitsDicDyn_);
	    if (dumpHdr.LoadHeader(fits.FitsFileName()) == FAILURE)
		return FAILURE;
	    INFO(3,"Loaded header(s) of FITS file: %s, into Dump Header "
		 "Object. Time: %.3fs.", fits.FitsFileName(),
		 chronometer_.Stop("LoadFitsHdrsInDumpHdrObj"));
	    }

	// Store the CHECKSUM keyword in the header if checksum is requested.
	if (Conditions().Checksum())
	    {
	    if (fitsObj->CreateStoreFits("CHECKSUM", 
					 "0000000000000000") == FAILURE)
		return FAILURE;
	    }
	else
	    {
	    // If checksum should not be stored, we remove it if 
	    // contained in the header, since it otherwise will contain 
	    // an incorrect value.
	    if (fitsObj->KeyInBuffer("CHECKSUM"))
		{
		if (fitsObj->RemoveKeyword("CHECKSUM") == FAILURE)
		    return FAILURE;
		}
	    }

	// Now store the contents of the FITS header in the temporary file.

	// Ensure that the file has extension ".fits" -> workaround 
	// since OSLX enforces extension ".fits". 
	chronometer_.Start("SaveHeaders");
	slxFILENAME   tmpFileName;
	if (AddFitsExt(fileSetPtr->GetTmpFileName(), tmpFileName)==FAILURE)
	    return FAILURE;

	if (fitsObj->Save(tmpFileName) == FAILURE)
	    return FAILURE;
	if (strcmp(tmpFileName, fileSetPtr->GetTmpFileName()) != 0)
	    {
	    if (RenameFile(tmpFileName, fileSetPtr->GetTmpFileName(),
			   0) == FAILURE)
		return FAILURE;
	    }
	INFO(3,"Time for saving headers in target file: %.3fs.",
	     chronometer_.Stop("SaveHeaders"));
	
	// Calculate and write the CHECKSUM keyword if requested.
	if (Conditions().Checksum())
	    {	
	    chronometer_.Start("CalculateChecksum");
	    
	    mmap_file       *mmap;
	    if ((mmap = mmap_open(fileSetPtr->GetTmpFileName())) == NULL)
		{
		sprintf(errBuf_, "Could not memory map file: %s.", 
			fileSetPtr->GetTmpFileName());
		ErrAdd("ftu", ftuERR_GEN, utilERR_FATAL, errBuf_);
		return FAILURE;
		}
	    INFO(3,"Calculating CHECKSUM value ..."); 

	    /* Now compute the checksum incrementally, chunks must be 
	     * smaller than  2**17 = 131072 bytes.
	     */
	    int   chunk = 2880; // 129600  45 * 2880.
	    int   left, size;
	    char  *buf;
	    //unsigned short  sum16 = 0;
	    //unsigned int    sum32 = 0;
	    long unsigned int sum = 0;
	    for (buf = mmap->buf, left = mmap->size; left > 0; buf += chunk) 
		{
		size = (left > chunk)? chunk : left;
		//checksum(buf, size, &sum16, &sum32);
		ffcsum(buf, 1, &sum);
		left -= chunk;
		}
	    //sum32 = sum;
	    slxVALUE  checksumValue;
	    ffesum(sum, 1, checksumValue);
	    //char_encode(~sum32, checksumValue, 4, 1);
	    INFO(3,"CHECKSUM calculated (%ld/%s). Time: %.3fs.", ~sum,
		 checksumValue, chronometer_.Stop("CalculateChecksum"));
	    mmap_close(mmap);
		
	    // Update the CHECKSUM value in the header.
	    INFO(3,"Updating file with the CHECKSUM value ...");
	    chronometer_.Start("UpdateChecksum");
	    if (fitsObj->ClearAllAuxKeys() == FAILURE)
		return FAILURE;
	    if (fitsObj->CreateStoreFits("CHECKSUM", checksumValue) == FAILURE)
		return FAILURE;
	    
	    slxFILENAME   tmpFileName;
	    if (AddFitsExt(fileSetPtr->GetTmpFileName(), 
			   tmpFileName) == FAILURE)
		return FAILURE; 

	    if (fitsObj->Save(tmpFileName) == FAILURE)
		return FAILURE;
	    if (RenameFile(tmpFileName, 
			   fileSetPtr->GetTmpFileName(), 1) == FAILURE)
		return FAILURE;
	    INFO(2,"File updated with CHECKSUM value (%s). Time: %.3fs.",
		 checksumValue, chronometer_.Stop("UpdateChecksum"));
	    }

	// Rename the file to the final name.
	INFO(3,"Renaming temporary target file: %s to target "
	     "filename: %s.", fileSetPtr->GetTmpFileName(), 
	     fileSetPtr->GetTargetFileName());
	if (RenameFile(fileSetPtr->GetTmpFileName(), 
		       fileSetPtr->GetTargetFileName(), 1) == FAILURE)
	    {
	    sprintf(errBuf_, "Error renaming the file: %s to name: %s.",
		    fileSetPtr->GetTmpFileName(), 
		    fileSetPtr->GetTargetFileName());
	    ErrSysAdd("ftu", ftuERR_GEN, utilERR_FATAL, errBuf_);
	    return FAILURE;
	    }
	INFO(1,"Target file %s updated.",fileSetPtr->GetTargetFileName());

	// Dump the FITS headers if requested.
	if (GetDumpHdr())
	    {
	    if (DumpFitsHeader(dumpHdr,
			       fileSetPtr->GetTargetFileName())==FAILURE)
	    return FAILURE;
	    }

	} // end- for (int fileNo = startHdr; fileNo < stopHdr; fileNo++)

    return SUCCESS;
}


utilSTAT ftuFTT::DumpFitsHeader(oslxFITS           &fitsObj,
				const slxFILENAME  targetFileName)
{
    if (GetDumpHdr() == ftuHDR_DUMP_NONE)
	return SUCCESS;

    chronometer_.Start("DumpFitsHeader");
    slxFILENAME    path, fn, ext;
    utilFILE_MGR::SplitFileName(targetFileName, path, fn, ext);    
    slxFILENAME  tmpHdrFileName, targetHdrFileName;
    sprintf(tmpHdrFileName, "%s/%s_tmp_hdr.%s", path, fn, ext);
    sprintf(targetHdrFileName, "%s/%s.hdr", path, fn);
    if (utilFILE_MGR::FileExists(tmpHdrFileName))
	remove(tmpHdrFileName);
    if (GetDumpHdr() == ftuHDR_DUMP_NL)
	{
	if (fitsObj.SaveAsciiFile(tmpHdrFileName) == FAILURE)
	    return FAILURE;
	}
    else
	{
	// ftuHDR_DUMP_NO_NL.
	if (fitsObj.Save(tmpHdrFileName) == FAILURE)
	    return FAILURE;
	}

    //if (rename(tmpHdrFileName, targetHdrFileName) != 0)
    if (RenameFile(tmpHdrFileName, targetHdrFileName, 0) == FAILURE)
	{
	sprintf(errBuf_, "Error renaming the file: %s to name: %s.",
		tmpHdrFileName, targetHdrFileName);
	ErrSysAdd("ftu", ftuERR_GEN, utilERR_FATAL, errBuf_);
	return FAILURE;
	}

    INFO(2,"Dumped FITS header in file: %s. Time: %.3fs.",
	 targetHdrFileName, chronometer_.Stop("DumpFitsHeader"));

    return SUCCESS;
}


utilSTAT ftuFTT::SkipHeader_(mmap_file       *mmap,
			     const int       loops,
			     unsigned int    *mmapIdx)
{
    int         search;

    chronometer_.Start("SkipHeader");
    for (int loopNo = 0; loopNo < loops; loopNo++)
	{
	search = TRUE;
	while (search == TRUE)
	    {
	    if (memcmp((mmap->buf + *mmapIdx), "END ", 4) == 0)
		{
		*mmapIdx += ftuFITS_CARD;
		search = FALSE;
		// mmapIdx must be a multiple of 2880.
		int modulus;
		if ((modulus = (*mmapIdx % ftuHDU_BLOCK)) != 0)
		    *mmapIdx += (ftuHDU_BLOCK - modulus);
		}
	    else
		{
		*mmapIdx += ftuFITS_CARD;
		if (*mmapIdx >= mmap->size)
		    search = FALSE;
		}
	    }   // end-while(search)
	}   // end-for (int loopNo = 0; loopNo < loops; loops++)
    INFO(3,"Time for skipping next header in source file: %.3fs.",
	 chronometer_.Stop("SkipHeader"));

    return SUCCESS;
}


utilSTAT    ftuFTT::CreateTmpFile(oslxFITS             &fits,
				  const slxFILENAME    fileName)
{

    // Generate the temporary, target file. Create only the necessary 
    // number of blocks.
    FILE  *tmpFilePtr;
    tmpFilePtr = NULL;
    unsigned int  mmapIdx = 0;
    mmap_file     *mmap;
    mmap = NULL;

    if ((mmap = mmap_open((char *)fileName)) == NULL)
	{
	sprintf(errBuf_, "Error opening source FITS file: %s.",	fileName);
	ErrSysAdd("ftu", ftuERR_GEN, utilERR_FATAL, errBuf_);
	goto errorExit;
	}

    oslxFITS    *hdrPtr;
    int         reqBlocks;
    int         bytesRead;
    bytesRead = 0;
    utilLOGICAL  deferFileRead;
    deferFileRead = FALSE;
    oslxFITS     *mergeFits;
    INFO(2,"Generating temporary target file(s) ...");
    chronometer_.Start("GenTmpFile");
    int hdrNo, startHdr, selHdusCount;

    // If the file should be split and if the primary header (extension 0)
    // does not contain any data, we start from header number 1 (2 in OSLX).
    int ext0DataSize; 
    if (GetDataSize(fits, &ext0DataSize) == FAILURE)
	goto errorExit;

    // If the file should be split and if the FILE.SPLIT.HDUS key is
    // defined, we should only consider the headers specified in latter.
    if (Split(fits) && splitSelHdus_)
	startHdr = 0;
    else if (Split(fits) && (ext0DataSize == 0))
	startHdr = 1;
    else
	startHdr = 0;
    selHdusCount = 0;
    int     imageSize, totalImageSize, bitPix, naxis;
    double  bscale, bzero;
    for (hdrNo = startHdr; hdrNo < fits.NoOfHdrs(); hdrNo++)
	{
	// If only selected HDUs should be considered, we skip the iterations
	// concerning other HDUs.
	if (splitSelHdus_)
	    {
	    if (hdrNo != hduList_[selHdusCount])
		{
		// We should not forget to update the mmap index:
		// 1. Split header.
		// 2. Split data.
		if (SkipHeader_(mmap, 1, &mmapIdx) == FAILURE)
		    goto errorExit;
		if ((hdrPtr = fits.GetHdr((hdrNo + 1))) == NULL)
		    {
		    sprintf(errBuf_, "Could not get access to header: "
			    "%d in FITS file: %s.", hdrNo,fits.FitsFileName());
		    ErrAdd("ftu", ftuERR_GEN, utilERR_FATAL, errBuf_);
		    goto errorExit;
		    }
		if (ImageInfo(*hdrPtr, &imageSize, &totalImageSize, 
			      &bitPix, &naxis, &bscale, &bzero) == FAILURE)
		    goto errorExit;
		mmapIdx += totalImageSize;
		continue;
		}
	    else
		selHdusCount++;
	    }

	// Get the next temporary target fileName + create the file.
	fttFILENAME_SET *fileSetPtr;
	utilLOGICAL     openFile = FALSE;
	if (!Split(fits) && (hdrNo == 0))
	    {
	    if (fileNames_.GetEl(0, (void **)&fileSetPtr) == FAILURE)
		goto errorExit;
	    openFile = TRUE;
	    }
	else if (Split(fits))
	    {
	    if (fileNames_.GetEl(hdrNo, (void **)&fileSetPtr) == FAILURE)
		goto errorExit;
	    openFile = TRUE;
	    }
	if (openFile)
	    {
	    if ((tmpFilePtr = fopen(fileSetPtr->GetTmpFileName(), "w"))==NULL)
		{
		sprintf(errBuf_, "Error creating temporary target FITS "
			"file: %s.", fileSetPtr->GetTmpFileName());
		ErrSysAdd("ftu", ftuERR_GEN, utilERR_FATAL, errBuf_);
		goto errorExit;
		}
	    }

	// Get the reference to the next FITS object.
	if ((hdrPtr = fits.GetHdr((hdrNo + 1))) == NULL)
	    {
	    sprintf(errBuf_, "Could not get access to header: %d in FITS "
		    "file: %s.", hdrNo, fits.FitsFileName());
	    ErrAdd("ftu", ftuERR_GEN, utilERR_FATAL, errBuf_);
	    goto errorExit;
	    }
	
	// If we're splitting files and if the number of extensions is
	// larger than 1 in the oslxFITS object, we generate a merged
	// FITS header consisting of the keywords from the primary header
	// with the keywords from the extension header merged on top of it.
	if (Split(fits))
	    {
	    oslxFITS *fitsPtr, *tmpFitsPtr;
	    tmpFitsPtr = hdrPtr;
	    if ((mergeFits = new(oslxFITS)) == NULL)
		{
		ErrAdd("ftu", ftuERR_GEN, utilERR_FATAL, "Error creating "
		       "object of type oslxFITS.");
		goto errorExit;
		}
	    mergeFits->SetDictionary(&fitsDicDyn_);
	    mergeFits->AutoAddKeysInDic(TRUE);
	    hdrPtr = mergeFits;
	    fitsPtr = mergeFits;
	    if (fitsObjs_.AddEl(hdrNo, (void **)&fitsPtr) == FAILURE)
		goto errorExit;

	    // Get the keywords from the primary header.
	    fitsPtr = fits.GetHdr(1);
	    if (CopyFitsObj(*fitsPtr, *mergeFits) == FAILURE)
		goto errorExit;
	    if (CopyFitsObj(*tmpFitsPtr, *mergeFits) == FAILURE)
		goto errorExit;
	    // Remove EXTEND + XTENSION keywords.
	    if (mergeFits->KeyInBuffer("EXTEND"))
		{
		if (mergeFits->RemoveKeyword("EXTEND") == FAILURE)
		    goto errorExit;
		}
	    if (mergeFits->KeyInBuffer("XTENSION"))
		{
		if (mergeFits->RemoveKeyword("XTENSION") == FAILURE)
		    goto errorExit;
		}
	    }
	    
	// Get info about the next image (if any).
	//int     imageSize, totalImageSize, bitPix, naxis;
	//double  bscale, bzero;
	if (ImageInfo(*hdrPtr, &imageSize, &totalImageSize, 
		      &bitPix, &naxis, &bscale, &bzero) == FAILURE)
	    goto errorExit;
	
	// In the first block we have to write, BITPIX, NAXIS, NAXIS1, ...
	// We generate these lines already now. This is just for the 
	// temporary file so we do it as quickly as possible.
	slxKEYW_LINE  bitPixLine, naxisLine, naxisLines[32], tmpKey;
	sprintf(bitPixLine, "BITPIX  =   %d    / BITPIX  ", bitPix);
	sprintf(naxisLine,  "NAXIS   =   %d    / NAXIS   ", naxis);	
	for (int naxisNo = 1; naxisNo <= naxis; naxisNo++)
	    {
	    sprintf(tmpKey, "NAXIS%d", naxisNo);
	    hdrPtr->GetKeyword(tmpKey);
	    sprintf(naxisLines[naxisNo], "NAXIS%d  =   %d    / NAXISn  ", 
		    naxisNo, hdrPtr->CK().Integer());
	    }

	// If we need to add the CHECKSUM keyword (later) and if this key
	// is not already contained in the first header, we have to ensure
	// that there is space for this key in the target file. If statistics
	// is on and the keys are not already contained in the header we have
	// to ensure that there is enough space for these as well.
	chronometer_.Start("AllocHeader");
	vltINT32   extraKeys;
	extraKeys = 1;  // END keyword

	// Allocate space for the FTU comment?
	if (Conditions().Comment())
	    extraKeys++;

	// Allocate space for CHECKSUM?
	if (((hdrNo == 0) || Split(fits)) && 
	    (Conditions().Checksum() && !hdrPtr->KeyInBuffer("CHECKSUM")))
	    extraKeys++;   

	// If the CHECKSUM keyword is contained in the header but calculation
	// of CHECKSUM is not enabled, we will remove the CHECKSUM keyword.
	if (!Conditions().Checksum() && hdrPtr->KeyInBuffer("CHECKSUM"))
	    extraKeys--;

	ftuFITS_HDR_TYPES hdrType = HeaderType(*hdrPtr);

	// Allocate space for DATAMIN, DATAMAX, MEAN and MEDIAN?
	if ((totalImageSize > 0) && Conditions().Statistics() && 
	    ((hdrType == ftuPRIM_HDR) || (hdrType == ftuIMAGE_EXT)))
	    {
	    if (!hdrPtr->KeyInBuffer("DATAMIN"))
		{
		extraKeys++;
		if (Conditions().History())
		    extraKeys++;
		}
	    if (!hdrPtr->KeyInBuffer("DATAMAX"))
		{
		extraKeys++;
		if (Conditions().History())
		    extraKeys++;
		}
	    if (!hdrPtr->KeyInBuffer("DATAMEAN"))
		{
		extraKeys++;
		if (Conditions().History())
		    extraKeys++;
		}
	    if (!hdrPtr->KeyInBuffer("DATAMED"))
		{
		extraKeys++;
		if (Conditions().History())
		    extraKeys++;
		}
	    if (!hdrPtr->KeyInBuffer("DATARMS"))
		{
		extraKeys++;
		if (Conditions().History())
		    extraKeys++;
		}
	    }

	// Calculate the total size of the header.
	vltINT32  totalSize;
	totalSize = (slxMAX_FITS_LI_LEN * 
		     (hdrPtr->NoOfKeywords() + hdrPtr->NumberOfComments() + 
		      hdrPtr->NumberOfLogs() +
		      hdrPtr->NumberOfHistoryEntries() + extraKeys));
	reqBlocks = (vltINT32)ceil((double)totalSize / 
				   (double)slxFITS_HD_BLOCK_LEN);

	// Write the necessary number of header blocks in the temporary 
	// target file.
	char  tmpBlock[ftuHDU_BLOCK], *bufPtr;
	int   reqBlocksMin1 = (reqBlocks - 1);
	for (int blockNo = 0; blockNo < reqBlocks; blockNo++)
	    {	 
	    // If the first block we have to write either SIMPLE or EXTENSION.
	    if ((blockNo == 0) && ((hdrNo == 0) || Split(fits)))
		{
		if (reqBlocks == 1)
		    memcpy(tmpBlock, oslxBLANK_BUF_2880_END, ftuHDU_BLOCK);
		else
		    memcpy(tmpBlock, oslxBLANK_BUF_2880, ftuHDU_BLOCK);
		memcpy(tmpBlock, ftuSIMPLE_LINE, ftuFITS_CARD);
		bufPtr = tmpBlock;
		}
	    else if (blockNo == 0)
		{
		if (reqBlocks == 1)
		    memcpy(tmpBlock, oslxBLANK_BUF_2880_END, ftuHDU_BLOCK);
		else
		    memcpy(tmpBlock, oslxBLANK_BUF_2880, ftuHDU_BLOCK);
		memcpy(tmpBlock, ftuXTENSION_LINE, ftuFITS_CARD);
		bufPtr = tmpBlock;
		}
	    else if (blockNo == reqBlocksMin1)
		{
		memcpy(tmpBlock, oslxBLANK_BUF_2880_END, ftuHDU_BLOCK);
		bufPtr = tmpBlock;
		}
	    else
		bufPtr = oslxBLANK_BUF_2880;
	    
	    // In the first block we have to store BITPIX, NAXIS, ...
	    if (blockNo == 0)
		{
		int idx = ftuFITS_CARD;
		memcpy((bufPtr + idx), bitPixLine, (strlen(bitPixLine) - 1));
		idx += ftuFITS_CARD;

		memcpy((bufPtr + idx), naxisLine, (strlen(naxisLine) - 1));
		idx += ftuFITS_CARD;

		for (int naxisNo = 1; naxisNo <= naxis; naxisNo++)
		    {
		    memcpy((bufPtr + idx), naxisLines[naxisNo], 
			   (strlen(naxisLines[naxisNo]) - 1));
		    idx += ftuFITS_CARD;
		    }
		}   // end-if (blockNo == 0)
	    
	    if (fwrite(bufPtr, ftuHDU_BLOCK, 1, tmpFilePtr) != 1)
		{
		sprintf(errBuf_,"Could not write in file: %s.",	
			fileSetPtr->GetTmpFileName());
		ErrSysAdd("ftu", ftuERR_GEN, utilERR_FATAL, errBuf_);
		goto errorExit;
		}
		
	    }   // end-for (int blockNo = 0; blockNo < reqBlocks; blockNo++)
	    
	INFO(3,"Time for allocating next header in target file: %.3fs.",
	     chronometer_.Stop("AllocHeader"));

	// Copy over the data from the source file to the target file 
	// (if any).
	// Skip first the header in connection with that data in
	// the source file.
	// If data size for primary header is 0, and we're splitting files,
	// and we're working on the first extension header, we have to
	// skip first the primary header and afterwards the extension header.
	int loops;
	if (splitSelHdus_)
	    loops = 1;
	else if (Split(fits) && (hdrNo == 1))
	    {
	    int hdrDataSize;
	    if (GetDataSize(fits, &hdrDataSize) == FAILURE)
		goto errorExit;
	    if (hdrDataSize == 0)
		loops = 2;
	    else
		loops = 1;
	    }
	else 
	    loops = 1;
	if (SkipHeader_(mmap, loops, &mmapIdx) == FAILURE)
	    goto errorExit;

	// Copy over the data.
	char    *startData;
	startData = (mmap->buf + mmapIdx);
	if ((totalImageSize > 0) && (mmapIdx < mmap->size))
	    {	  
	    INFO(3,"Writing data in temporary target file %s ...",
		 fileSetPtr->GetTmpFileName());
	    chronometer_.Start("WriteData");
	    int writeSize;
	    if (mmap->size < (totalImageSize + mmapIdx))
		{
		writeSize = (mmap->size - mmapIdx);
		sprintf(errBuf_,"WARNING: File %s is corrupted (too short).",
			fits.FitsFileName());
		ErrSysAdd("ftu", ftuERR_GEN, utilERR_WARNING, errBuf_);
		goto errorExit;
		}
	    else
		writeSize = totalImageSize;
	    if (fwrite((mmap->buf + mmapIdx), writeSize, 1, tmpFilePtr) != 1)
		{
		sprintf(errBuf_,"Could not write in file: %s.",
			fileSetPtr->GetTmpFileName());
		ErrSysAdd("ftu", ftuERR_GEN, utilERR_FATAL, errBuf_);
		goto errorExit;
		}
	    INFO(3,"Time for writing data in temporary target file: "
		 "%.3fs.", chronometer_.Stop("WriteData"));
	    mmapIdx += totalImageSize;
	    }   // end-if ((totalImageSize > 0) && (mmapIdx < mmap->size))

	// Store the statistics keywords into the FITS object if requested.
	if ((totalImageSize > 0) && Conditions().Statistics() && 
	    ((hdrType == ftuPRIM_HDR) || (hdrType == ftuIMAGE_EXT)))
	    {
	    // Run 1: Calculate minimum, maximum, mean and RMS.
	    double   min, max, mean, rms, sum;
	    if (GenStatistics(bitPix, startData, imageSize, bscale, bzero, 
	    		      &min, &max, &mean, &rms, &sum) == FAILURE)
		goto errorExit;

	    // Run 2: Calculate median.
	    INFO(2,"Calculating median ...");
	    chronometer_.Start("CalcMedian");
	    double  median;
	    switch (bitPix)
		{
		case -32:
		    median = median_pixelvalue_FP32((ftuFP32 *)startData, 
						    (imageSize / 4));
		    break;
		case 32:
		    median = median_pixelvalue_INT32((ftuINT32 *)startData, 
						     (imageSize / 4));
		    break;
		case 16:
		    median = median_pixelvalue_INT16((ftuINT16 *)startData, 
						     (imageSize / 2));
		    break;
		case -64:
		    median = median_pixelvalue_FP64((ftuFP64 *)startData, 
						    (imageSize / 8));
		    break;
		default:
		    // case 8:
		    median = median_pixelvalue_INT8((ftuINT8 *)startData,
						    imageSize);
		    break;
		}
	    // Scale median according to BSCALE/BZERO.
	    median = ((bscale * median) + bzero);
	    INFO(2,"Median calculated to: %f. Time: %.3fs.", median,
		 chronometer_.Stop("CalcMedian"));

	    if (AddHistory(ftuADD_STR, hdrPtr, "DATAMIN","DATAMIN") == FAILURE)
		goto errorExit;
	    if (hdrPtr->CreateStoreFits("DATAMIN",(void *)&min) == FAILURE)
		goto errorExit;

	    if (AddHistory(ftuADD_STR, hdrPtr, "DATAMAX","DATAMAX") == FAILURE)
		goto errorExit;
	    if (hdrPtr->CreateStoreFits("DATAMAX",(void *)&max) == FAILURE)
		goto errorExit;

	    if (AddHistory(ftuADD_STR, hdrPtr, "DATAMEAN","DATAMEAN")==FAILURE)
		goto errorExit;
	    if (hdrPtr->CreateStoreFits("DATAMEAN",(void *)&mean)==FAILURE)
		goto errorExit;

	    if (AddHistory(ftuADD_STR, hdrPtr, "DATARMS","DATARMS") == FAILURE)
		goto errorExit;
	    if (hdrPtr->CreateStoreFits("DATARMS",(void *)&rms) == FAILURE)
		goto errorExit;
	    
	    if (AddHistory(ftuADD_STR, hdrPtr, "DATAMED","DATAMED") == FAILURE)
		goto errorExit;
	    if (hdrPtr->CreateStoreFits("DATAMED", (void *)&median)== FAILURE)
		goto errorExit;
	    }

	// CLose the temporary file pointer if we're splitting files, as we
	// will be creating the next split-file in the beginning of the loop.
	if (Split(fits))
	    {
	    fclose(tmpFilePtr);
	    tmpFilePtr = NULL;
	    }
	}   // end-for (hdrNo = startHdr; hdrNo < fits.NoOfHdrs(); hdrNo++)

    INFO(2,"Temporary target file generated. Time: %.3fs.", 
	 chronometer_.Stop("GenTmpFile"));

    // If splitting was not on, we have to close the file here, since it
    // wasn't closed in the loop.
    if (!Split(fits))
	fclose(tmpFilePtr);

    mmap_close(mmap);

    return SUCCESS;

  errorExit:
    if (tmpFilePtr != NULL)
	fclose(tmpFilePtr);
    if (mmap != NULL)
	mmap_close(mmap);
    return FAILURE;
}


// Replaces oslxFITS::Copy() until bug reported in SPR VLTSW20010135
// has been fixed (if it will be ...).
ccsCOMPL_STAT ftuFTT::CopyFitsObj(oslxFITS  &srcObj,
				  oslxFITS  &trgObj)
{
    vltINT32 noOfKeys;

    // IMPL.: Support copying of extension headers.

    srcObj.ResetFitsHeaderPtr();
    noOfKeys = srcObj.NoOfKeywords();
    for (int i = 0; i < noOfKeys; i++)
	{
	if (srcObj.GetNextKeyword() == FAILURE)
	    return FAILURE;
	// Cheap solution to solve problem with precision - should be
	// improved if this method stays within ftuFTT.
	if (fitsDicDyn_.KeyInDic(srcObj.CK().Keyword()))
	    {
	    if (fitsDicDyn_.LookUp(srcObj.CK().Keyword()) == FAILURE)
		return FAILURE;
	    if (fitsDicDyn_.DicKeywType() == slxDOUBLE)
		{
		slxKEYW_LINE  keywordLine;
		sprintf(keywordLine, "%s %.20f", srcObj.CK().Keyword(),
			srcObj.CK().Double());
		if (trgObj.StoreKeyword(keywordLine) == FAILURE)
		    return FAILURE;
		}
	    else
		if (trgObj.StoreKeyword(srcObj.CK()) == FAILURE)
		    return FAILURE;
	    }
	else
	    if (trgObj.StoreKeyword(srcObj.CK()) == FAILURE)
		return FAILURE;
	}   // end-for (int i = 0; i < noOfKeys; i++)
    
    // Copy over comments
    char  *chrPtr;
    srcObj.ResetCommentPtr();
    for (int com = 0; com < srcObj.NumberOfComments(); com++)
	{
	if ((chrPtr = srcObj.GetNextComment()) != NULL)
	    {
	    if (trgObj.AddComment(chrPtr) == FAILURE)
		return FAILURE;
	    }
	}   // end-for (com = 0; com < ...

    // Copy over logs
    srcObj.ResetLogPtr();
    for (int log = 0; log < srcObj.NumberOfLogs(); log++)
	{
	if ((chrPtr = srcObj.GetNextLog()) != NULL)
	    {
	    if (trgObj.AddLog(chrPtr) == FAILURE)
		return FAILURE;
	    }
	}   // end-for (log = 0; log < ...
    
    // Copy over history entries
    srcObj.ResetHistoryPtr();
    for (int hist = 0; hist < srcObj.NumberOfHistoryEntries(); hist++)
	{
	if ((chrPtr = srcObj.GetNextHistory()) != NULL)
	    {
	    if (trgObj.AddHistory(chrPtr) == FAILURE)
		return FAILURE;
	    }
	}   // end-for (int hist = 0; hist < ...
    
    return SUCCESS;
}


/*___oOo___*/
