/****************************************************************************
 * E.S.O. - Archive System
 *
 * "@(#) $Id: ftuFTT_Trans.C,v 1.11.2.1 2008/09/11 08:11:40 cguirao Exp $"
 *
 * Who       When        What
 * --------  ----------  ----------------------------------------------
 * jknudstr  20/01/2000  Created 
 */

/************************************************************************
 *
 *  NAME
 *  ftuFTT_Trans - Contains methods to handle the translations.
 *
 *  SYNOPSIS
 *   
 *  PARENT CLASS
 * 
 *  DESCRIPTION
 *
 *  PUBLIC METHODS
 *
 *  PUBLIC DATA MEMBERS
 *
 *  PROTECTED METHODS
 *
 *  PROTECTED DATA MEMBERS
 *
 *  FILES
 *
 *  ENVIRONMENT
 *
 *  RETURN VALUES
 *
 *  CAUTIONS 
 *
 *  EXAMPLES
 *
 *  SEE ALSO
 *
 *  BUGS   
 * 
 *------------------------------------------------------------------------
 */

static const char *rcsId="@(#) $Id: ftuFTT_Trans.C,v 1.11.2.1 2008/09/11 08:11:40 cguirao Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

#define _ALL_SOURCE 1
#define __EXTENSIONS__ 1

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "utilPAF.h"

#include "ftuDefs.h"
#include "ftuFTT.h"
#include "ftuTRANSLATION.h"
#include "ftuSwap.h"


utilSTAT    ftuFTT::Translate(const slxFILENAME    fileName)
{
    oslxFITS   fits;

    ResetNoOfTransCount();
    ResetFileIndex();

    // Always reset first the Dynamic FITS Dictionary in order to get rid
    // of possible wrong, previous keyword definitions when processing
    // several files in one go.
    fitsDicDyn_.ResetDic();
    if (fitsDicDyn_.Merge(&fitsDicStat_) == FAILURE)
	return FAILURE;

    INFO(1,"Starting translation of file: %s.",fileName);
    chronometer_.Start("TranslateFile");

//     //////////////////////////////////////////////////////////////////////
//     // Ensure the proper sequence of keywords in the primary header.
//     // Patch for ticket DFS02285.
//     //////////////////////////////////////////////////////////////////////
//     oslxDICTIONARY  *dicP = &fitsDicDyn_;
//     oslxFITS        *fitsP = &fits;
//     char  *tmpKeyList[] = {"SIMPLE", "logical", "%c", 
// 			   "Standard FITS format (NOST-100-2.0)",
// 			   "XTENSION", "string", "%s",
// 			   "FITS Extension first keyword", 
// 			   "BITPIX", "integer", "%d", 
// 			   "# bits storing pix values", 
// 			   "NAXIS", "integer", "%d", 
// 			   "# of axes in frame",
// 			   "NAXISi", "integer", "%d",
// 			   "# pixels/axis",
// 			   "EXTEND", "logical", "%c", 
// 			   "FITS Extensions may be present",
// 			   NULL};
//     int idx = 0;
//     while (tmpKeyList[idx] != NULL)
// 	{
// 	if (dicP->StoreKeyword(tmpKeyList[idx], "header", "FITS",
// 			       tmpKeyList[idx + 1], tmpKeyList[idx + 2], "", 
// 			       tmpKeyList[idx + 3], NULL, FALSE) == FAILURE)
// 	    return FAILURE;
// 	idx += 4;
// 	}
//     fitsP->SetDictionary(dicP);
    
//     // Define keyword sequence.
//     fitsP->SetKeySequence("SIMPLE,BITPIX,NAXIS,NAXIS1,NAXIS2,NAXIS3,NAXIS4,"
// 			  "EXTEND");
//     //////////////////////////////////////////////////////////////////////

    fits.SetDictionary(&fitsDicDyn_); 
    fits.AutoAddKeysInDic(TRUE);
    INFO(2,"Loading header(s) of FITS file: %s ...",fileName);
    chronometer_.Start("LoadHeaders");
    if (fits.LoadHeader(fileName) == FAILURE)
	{
	if (errErrInStack("slxERR_LOCATE_FILE_GEN", 
			  &fits.ErrStack()) == FAILURE)
	    {
	    errResetStack(&fits.ErrStack());
	    sprintf(errBuf_, "Could not locate FITS file: %s.", fileName);
	    INFO(1,errBuf_);
	    ErrAdd("ftu", ftuERR_GEN, utilERR_FATAL, errBuf_);
	    }
	return FAILURE;
	}
    INFO(2,"Loaded header(s) of FITS file: %s. Time: %.3fs.",fileName,
	  chronometer_.Stop("LoadHeaders"));

    // Check if the file has already been translated with this FTT.
    chronometer_.Start("CheckFtuComment");
    if (!Conditions().Force() && (*FttFileName() != '\0'))
	{
	INFO(2,"Checking if file already translated with this FTT ...");
	fits.ResetCommentPtr();
	int         comCount = 0, noOfComs = fits.NumberOfComments();
	char        *comPtr;
	while (comCount < noOfComs)
	    {
	    comPtr = fits.GetNextComment();
	    if (strstr(comPtr, "FTU-") != NULL)
		{
		// Format of the FTU comment is:
		// COMMENT  FTU-v.v/2000-01-26/<ftt name>
		slxKEYW_LINE     subStrings[16];
		vltINT32         noOfSubStr;
		if (slxSplit("/", comPtr, 16, subStrings, &noOfSubStr,
			     &ErrStack()) == FAILURE)
		    return FAILURE;
		slxTrimString(subStrings[2], " \n\t");
		if (strcmp(FttFileName(), subStrings[2]) == 0)
		    {
		    sprintf(errBuf_, "FITS file: %s has already been "
			    "translated with the FTT: %s. Use -force to "
			    "force translation.", fileName, FttFileName());
		    ErrAdd("ftu", ftuERR_GEN, utilERR_WARNING, errBuf_);
		    if (Conditions().Tolerant())
			{
			CloseStack();
			break;
			}
		    else
			return FAILURE;
		    }
		}
	    comCount++;
	    }   // end-while (!found && (comCount < noOfComs))	
	INFO(3,"Time for checking for FTU translation comment: %.3fs.",
	     chronometer_.Stop("CheckFtuComment"));
	}   // end-if (!Conditions().Force())

    // We have to loop over the various translation registered in the
    // FTT Translation Hash Table and carry out each translation on the
    // FITS file. 
    INFO(2,"Translating file: %s.",fileName);
    ftuTRANSLATION  *transPtr;
    chronometer_.Start("ApplyTT");
    for (int transNo = 0; transNo < fttTbl_.NoOfEls(); transNo++)
	{
	if ((transPtr = GetTranslation(transNo)) == NULL)
	    {  
	    sprintf(errBuf_, "Error occurred during translation. Could not "
		    "retrieve Translation Action number %d", transNo);
	    ErrAdd("ftu", ftuERR_GEN, utilERR_FATAL, errBuf_);
	    return FAILURE;
	    }
	switch (transPtr->GetAction())
	    {
	    case ftuADD:
		if (HandleActionAdd(fits, *transPtr) == FAILURE)
		    return FAILURE;
		break;
	    case ftuCHANGE:
		if (HandleActionChange(fits, *transPtr) == FAILURE)
		    return FAILURE;
		break;  
	    case ftuRENAME:
		if (HandleActionRename(fits, *transPtr) == FAILURE)
		    return FAILURE;
		break;
	    case ftuMOVE:
		if (HandleActionMove(fits, *transPtr) == FAILURE)
		    return FAILURE;
		break;
	    case ftuCOPY:
		if (HandleActionCopy(fits, *transPtr) == FAILURE)
		    return FAILURE;
		break;
	    default:
		// case ftuREMOVE:
		if (HandleActionRemove(fits, *transPtr) == FAILURE)
		    return FAILURE;
		break;
	    }   // end-switch (transP->GetAction())
	}   // end-for (int transNo = 0; transNo < fttTbl_.NoOfEls(); ...
    INFO(2,"Translated keywords in file: %s. Time: %.3fs.",fileName,
	 chronometer_.Stop("ApplyTT"));

    // Update the file. First create a temporary file whereby we copy over
    // the contents of the old file, remove blank 2800 blocks, allocate
    // more blocks if needed.
    if (GenerateNewFile(fits, fileName) == FAILURE)
	return FAILURE;

    INFO(1,"Translation carried out successfully for file: %s. Time: %.3fs.",
	 fileName, chronometer_.Stop("TranslateFile"));

    if (GetNoOfTransCount())
	{
	INFO(1,"Number of Translation Actions carried out successfully on "
	     "file %s: %d.",fileName, GetNoOfTransCount());
	}
    else
	{
	INFO(1,"No Translation Actions carried out on file %s.", fileName);
	}

    return SUCCESS;
}


void ftuFTT::SubstituteChr(char   *str,
			   char   srcChr,
			   char   trgChr)
{
    int   idx = 0;
    while (*(str + idx) != '\0')
	{
	if (*(str + idx) == srcChr)
	    *(str + idx) = trgChr;
	idx++;
	}
}


utilSTAT  ftuFTT::ExecPlugIn(const char  *cmd,
			     slxVALUE    result)
{
    int bytesRead = 0;

    // popen()/pclose() not available on Linux.
    INFO(3,"Invoking External Plug-In Conversion Function: %s",cmd);
    chronometer_.Start("InvokePlugIn");
    FILE  *filePtr = NULL;

    // IMPL: The following conditional can be removed.
#ifdef __linux__DISABLED__
    // Build a command like this on Linux:
    // "rm -f /tmp/.ftu_tmp; <plug-in> <arg1> <arg2> ... > /tmp/.ftu_tmp".
    char  cmdBuf[1024];

    char      *completePath;
    slxFILENAME  ftuTmpFile;
    if (slxResolvePath("/tmp/.ftu_tmp", &completePath, &ErrStack()) == FAILURE)
	return FAILURE;
    strcpy(ftuTmpFile, completePath);
    sprintf(cmdBuf, "rm -f %s; %s > %s", ftuTmpFile, cmd, ftuTmpFile);
    if (system(cmdBuf) == -1)
	{
	sprintf(errBuf_,"Problem occurred executing shell command: %s.",
		cmdBuf);
	goto errExit;
	}
    if ((filePtr = fopen(ftuTmpFile, "r")) == NULL)
	{
	sprintf(errBuf_, "Problem occurred opening temporary file: "
		"/tmp/.ftu_tmp.");
	goto errExit;
	}
    bytesRead = fread(result, 1, sizeof(slxVALUE), filePtr);
    fclose(filePtr);
    *(result + bytesRead) = '\0';
#else	
    if ((filePtr = popen((char *)cmd, "r")) == NULL)
	{
	sprintf(errBuf_, "Problem occurred invoking External "
		"Plug-In Conversion Function: %s",cmd);
	goto errExit;
	}
    if ((bytesRead = fread(result, 1, sizeof(slxVALUE), filePtr)) < 1)
	{
	sprintf(errBuf_, "Problem occurred invoking External "
		"Plug-In Conversion "
		"Function: %s - Problem reading from pipe.",cmd);
	goto errExit;
	}
    *(result + bytesRead) = '\0';
    int exitCode;
    if ((exitCode = pclose(filePtr)) != 0)
        {
	sprintf(errBuf_, "Problem occurred invoking External Plug-In "
		"Conversion Function: %s - Plug-In Exit Code different "
		"from 0: %d.", cmd, exitCode);
	goto errExit;
        }
#endif

    slxTrimString(result, "\t\n ");
    // Sometimes a string like "Hello\nle" can be returned from the shell
    // command which is subsequently not properly trimmed. We trim it 
    // explicitly.
    char *chrPtr;
    if ((chrPtr = strchr(result, '\n')) != NULL)
	*chrPtr = '\0';
    INFO(3,"Result returned by External Plug-In Conversion Function: |%s|. "
	 "Execution Time: %.3fs.",result, chronometer_.Stop("InvokePlugIn"));

    return SUCCESS;

 errExit:
    if (filePtr != NULL) pclose(filePtr);
    INFO(1,errBuf_);
    ErrAdd("ftu", ftuERR_GEN, utilERR_WARNING, errBuf_);
    return FAILURE;
}


utilSTAT ftuFTT::HandleActionSkip(const char  *errMsg)
{
    if ((Severity() <= utilERR_WARNING) && Conditions().Tolerant())
	{
	ErrAdd("ftu", ftuERR_GEN, utilERR_WARNING, errMsg);
	LogFlushError(Severity(), *(utilERROR *)this);
	INFO(1,"Tolerant Mode used - ignoring WARNING(s).");
	ClearStack();
	return SUCCESS;
	}
    else
	{
	ErrAdd("ftu", ftuERR_GEN, utilERR_FATAL, errMsg);
	return FAILURE;
	}
}

#define ftuCHECK_X_PI_CMD(retVal, fitsHdrP, key, hdrNo) \
{ \
    if (strstr(retVal, "#FTU-CMD:") == retVal) \
        { \
	if (strstr(retVal, "REMKEY") != 0) \
	    { \
	    if (fitsHdrP->KeyInBuffer(key)) \
	        { \
		INFO(3,"External Plug-In returned: \"#FTU-CMD: REMKEY\". " \
		     "Removing keyword: %s from header: %d.", key, hdrNo); \
		if (AddHistory(ftuREMOVE_STR, fitsHdrP, key, key) == FAILURE) \
		    goto skip; \
		if (fitsHdrP->RemoveKeyword(key) == FAILURE) \
		    goto skip; \
		} \
	    return SUCCESS; \
	    } \
	else \
	    { \
	    static slxKEYW_LINE errMsg; \
	    sprintf(errMsg, "Illegal FTU External Plug-In command line: %s.", \
		    retVal); \
	    return HandleActionSkip(errMsg); \
	    } \
	} \
}

#define ftuCHECK_X_PI_ERR(retVal) \
{ \
    if (strstr(retVal, "#FTU-ERR:") == retVal) \
        { \
	char *errPtr; \
	if ((errPtr = strchr(retVal, ' ')) != NULL) \
	    { \
	    slxKEYW_LINE errMsg; \
	    sprintf(errMsg, "FTU External Plug-In returned error message: " \
                    "%s", ++errPtr); \
	    ErrAdd("ftu", ftuERR_GEN, utilERR_WARNING, errMsg); \
	    } \
        } \
}


utilSTAT         ftuFTT::HandleActionAddChange(oslxFITS        &fits, 
					       ftuTRANSLATION  &translation,
					       ftuACTION       action)
{ 
    utilFITS_VAL  value;
    slxVALUE      rawValue;

    // Get target key.
    slxKEYW_LINE    targKey;
    strcpy(targKey, translation.GetSrcKey());

    // Get access to the specified header.
    oslxFITS  *fitsHdr;
    ftuGET_SRC_HDR(fitsHdr);

    // Generate the value.
    if (GenValue(fits, targKey, translation, value, rawValue) == FAILURE)
	goto skip;
    ftuCHECK_X_PI_CMD(rawValue, fitsHdr, targKey, translation.GetSrcHdr());

    if (CheckKeyAndValTypes(targKey, rawValue) == FAILURE)
	goto skip;

    // Special handling if keyword added is a COMMENT.
    if (strcmp(targKey, "COMMENT") != 0)
	{
	if (fitsHdr->CreateStoreFits(targKey,(void *)value.Value()) == FAILURE)
	    goto skip;
	}
    else
	{
	if (fitsHdr->AddComment(rawValue) == FAILURE)
	    goto skip;
	}

    if (action == ftuCHANGE)
	{
	if (AddHistory(ftuCHANGE_STR, fitsHdr, targKey, targKey) == FAILURE)
	    goto skip;
	}
    else
	{
	if (AddHistory(ftuADD_STR, fitsHdr, targKey, targKey) == FAILURE)
	    goto skip;
	}

    return SUCCESS;

    
  skip:
    ftuCHECK_X_PI_ERR(rawValue);
    char *errMsg;
    if (action == ftuCHANGE)
	errMsg = "Problem occurred performing action: Change.";
    else
	errMsg = "Problem occurred performing action: Add.";
    return HandleActionSkip(errMsg);
}


utilSTAT         ftuFTT::HandleActionAdd(oslxFITS        &fits, 
					 ftuTRANSLATION  &translation)
{ 
    INFO(3,"Executing action: Add.");
    INFO(3,"Adding keyword: %s in header: %d.",translation.GetSrcKey(),
	 translation.GetSrcHdr());
    if (HandleActionAddChange(fits, translation, ftuADD) == FAILURE)
	return FAILURE;
    INFO(3,"Successfully Executed action: Add.");
    IncNoOfTransCount();

    return SUCCESS;
}


utilSTAT         ftuFTT::HandleActionChange(oslxFITS        &fits, 
					   ftuTRANSLATION  &translation)
{
    INFO(3,"Executing action: Change."); 
    INFO(3,"Changing value of keyword: %s in header: %d.",
	 translation.GetSrcKey(), translation.GetSrcHdr());
    if (HandleActionAddChange(fits, translation, ftuCHANGE) == FAILURE)
	return FAILURE;
    INFO(3,"Successfully Executed action: Change.");
    IncNoOfTransCount();

    return SUCCESS;
}


utilSTAT         ftuFTT::HandleActionRename(oslxFITS        &fits, 
					    ftuTRANSLATION  &translation)
{
    utilFITS_VAL   value;

    INFO(3,"Executing action: Rename."); 

    if (!AllowKeyRemove(translation.GetSrcKey()))
	goto skip;

    // Get the value of the old keyword, store the new keyword with that
    // value, remove the old keyword.
    INFO(3,"Renaming key: %s to: %s in header: %d.",translation.GetSrcKey(),
	 translation.GetTargKey(), translation.GetSrcHdr());

    // If the target is defined in the Dictionary, this will be used for
    // generating this key. Otherwise the comment and format from the
    // source keyword will be taken.
    if (!fitsDicDyn_.KeyInDic(translation.GetTargKey()))
	{
	if (fitsDicDyn_.LookUp(translation.GetSrcKey()) == FAILURE)
	    goto skip;
	if (fitsDicDyn_.StoreKeyword(translation.GetTargKey(),
				     fitsDicDyn_.KeywClass(),
				     fitsDicDyn_.KeywContext(),
				     utilSlxType2Str(fitsDicDyn_.
						     DicKeywType()),
				     fitsDicDyn_.ValFormat(),
				     fitsDicDyn_.KeywUnit(),
				     fitsDicDyn_.ComField(),
				     "") == FAILURE)
	    goto skip;
	}

    // Get the value from the source keywowrd and store it as the target
    // keyword name. Afterwards we remove the source keyword.
    oslxFITS  *fitsHdr;
    ftuGET_SRC_HDR(fitsHdr);
    slxVALUE  rawValue;
 
    if (GenValue(fits, translation.GetTargKey(), translation, value,
		 rawValue) == FAILURE)
	goto skip;
    ftuCHECK_X_PI_CMD(rawValue, fitsHdr, translation.GetTargKey(), 
		      translation.GetSrcHdr());
   
    if (CheckKeyTypes(translation.GetSrcKey(), 
		      translation.GetTargKey()) == FAILURE)
	goto skip;
    if (CheckKeyAndValTypes(translation.GetTargKey(), rawValue) == FAILURE)
	goto skip;

    if (fitsHdr->CreateStoreFits(translation.GetTargKey(), 
				 (void *)value.Value()) == FAILURE)
	goto skip;

    slxKEYW_LINE renameAction;
    sprintf(renameAction, "%s->%s", translation.GetSrcKey(),
	    translation.GetTargKey());
    if (AddHistory(ftuRENAME_STR, fitsHdr, translation.GetSrcKey(),
		   renameAction) == FAILURE)
	goto skip;   

    if (fitsHdr->RemoveKeyword(translation.GetSrcKey()) == FAILURE)
	goto skip;

    INFO(3,"Successfully Executed action: Rename.");
    IncNoOfTransCount();

    return SUCCESS;
    
  skip:
    ftuCHECK_X_PI_ERR(rawValue);
    return HandleActionSkip("Problem occurred performing action: Rename.");
}


utilSTAT         ftuFTT::HandleActionMove(oslxFITS        &fits, 
					  ftuTRANSLATION  &translation)
{
    utilFITS_VAL  value; 

    INFO(3,"Executing action: Move."); 
    INFO(3,"Moving keyword: %s from header: %d to header: %d.",
	 translation.GetSrcKey(), translation.GetSrcHdr(),
	 translation.GetTargHdr());

    if (!AllowKeyRemove(translation.GetSrcKey()))
	goto skip;

    // Get the keyword from the source header, apply possibly a function
    // to the value, remove it from the source header and create it in the 
    // target header.
    oslxFITS      *fitsSrcHdr, *fitsTargHdr;
    ftuGET_SRC_HDR(fitsSrcHdr);
    ftuGET_TARG_HDR(fitsTargHdr);
    slxVALUE      rawValue;
    if (GenValue(fits, translation.GetSrcKey(), translation, value,
		 rawValue) == FAILURE)
	goto skip;
    ftuCHECK_X_PI_CMD(rawValue, fitsSrcHdr, translation.GetSrcKey(), 
		      translation.GetSrcHdr());

    if (CheckKeyAndValTypes(translation.GetSrcKey(), rawValue) == FAILURE)
	goto skip;

    if (fitsTargHdr->CreateStoreFits(translation.GetSrcKey(), 
				     (void *)value.Value()) == FAILURE)
	goto skip;

    if (AddHistory(ftuMOVE_STR, fitsSrcHdr, translation.GetSrcKey(),
		   translation.GetSrcKey()) == FAILURE)
	goto skip;

    // Remove only the keyword if source and target headers are different.
    if (translation.GetSrcHdr() != translation.GetTargHdr())
	{
	if (fitsSrcHdr->RemoveKeyword(translation.GetSrcKey()) == FAILURE)
	    goto skip;
	}

    INFO(3,"Successfully Executed action: Move.");
    IncNoOfTransCount();

    return SUCCESS;

  skip:
    ftuCHECK_X_PI_ERR(rawValue);
    return HandleActionSkip("Problem occurred performing action: Move.");
}


utilSTAT         ftuFTT::HandleActionCopy(oslxFITS        &fits, 
					  ftuTRANSLATION  &translation)
{
    INFO(3,"Executing action: Copy."); 
    INFO(3,"Copying the keyword: %s from header: %d, to keyword: %s in "
	 "header: %d.", translation.GetSrcKey(), translation.GetSrcHdr(),
	 translation.GetTargKey(), translation.GetTargHdr());

    // Get source key from source header, store it as the target key in
    // the target header, apply possibly a function to the value.
    oslxFITS       *fitsSrcHdr, *fitsTargHdr;
    utilFITS_VAL   value;
    slxVALUE       rawValue;
    ftuGET_SRC_HDR(fitsSrcHdr);
    ftuGET_TARG_HDR(fitsTargHdr);
 
    if (GenValue(fits, translation.GetTargKey(), translation, value,
		 rawValue) == FAILURE)
	goto skip;
    ftuCHECK_X_PI_CMD(rawValue, fitsSrcHdr, translation.GetSrcKey(), 
		      translation.GetSrcHdr());
    
    if (CheckKeyTypes(translation.GetSrcKey(), 
		      translation.GetTargKey()) == FAILURE)
	goto skip;

    if (CheckKeyAndValTypes(translation.GetTargKey(), rawValue) == FAILURE)
	goto skip;

    if (fitsTargHdr->CreateStoreFits(translation.GetTargKey(), 
				     (void *)value.Value()) == FAILURE)
	goto skip;

    slxKEYW_LINE copyAction;
    sprintf(copyAction, "%s->%s", translation.GetSrcKey(),
	    translation.GetTargKey());
    if (AddHistory(ftuCOPY_STR, fitsSrcHdr, translation.GetSrcKey(),
		   copyAction) == FAILURE)
	goto skip;
    INFO(3,"Successfully Executed action: Copy.");
    IncNoOfTransCount();

    return SUCCESS;

  skip:
    ftuCHECK_X_PI_ERR(rawValue);
    return HandleActionSkip("Problem occurred performing action: Copy.");
}


utilSTAT         ftuFTT::HandleActionRemove(oslxFITS        &fits, 
					    ftuTRANSLATION  &translation)
{
    oslxKEYW_FILTER  filter;

    INFO(3,"Executing action: Remove.");
    INFO(3,"Removing keyword(s) with pattern: %s from header: %d.",
	 translation.GetSrcKey(), translation.GetSrcHdr());

    // Apply a filter on the header referenced with the pattern given.
    oslxFITS  *fitsHdr;
    ftuGET_SRC_HDR(fitsHdr);
    if (filter.AddFilter(translation.GetSrcKey()) == FAILURE)
	goto skip;
    if (fitsHdr->ExtractKeywords(FALSE, &filter) == FAILURE)
	goto skip;
    if (fitsHdr->NoOfExtractedKeywords() > 0)
	{
	slxKEYW_LINE     keywords[6000];
	vltINT32         noOfSubStr;
	if (slxSplit(",", fitsHdr->GetExtractBuffer(), 6000, keywords,
		     &noOfSubStr, &ErrStack()) == FAILURE)
	    goto skip;
	slxKEYW_LINE  key;
	char          *chrPtr;
	for (int keyNo = 1; keyNo < noOfSubStr; keyNo++)
	    {
	    if (slxGetNextField(keywords[keyNo], sizeof(slxKEYW_LINE),
				key, &chrPtr, &ErrStack()) == FAILURE)
		goto skip;
	    slxTrimString(keywords[keyNo], " \t\n");

	    if (AllowKeyRemove(key))
		{
		INFO(4,"Removing keyword: %s.",key); 
		if (AddHistory(ftuREMOVE_STR, fitsHdr, key, key) == FAILURE)
		    goto skip;
		if (fitsHdr->RemoveKeyword(key) == FAILURE)
		    goto skip;
		}
	    }   // end-for (int keyNo = 1; ...
	}   // end-if (fitsHdr->NoOfExtractedKeywords() > 0)
    INFO(3,"Successfully Executed action: Remove.");
    IncNoOfTransCount();

    return SUCCESS;
    
  skip: 
    return HandleActionSkip("Problem occurred performing action: Remove.");
}


utilSTAT        ftuFTT::Date(const char  *date)
{
    strcpy(date_, date);
    return SUCCESS;
}


char            *ftuFTT::Date()
{
    return date_;
}


utilSTAT  ftuFTT::AddHistory(const char     *action,
			     oslxFITS       *fitsObj,
			     const char     *keyValSrc,
			     const char     *keyActionDescr)
{
    if (Conditions().History()) 
        {
	slxKEYW_LINE  historyEntry;
	if (strcmp(action, ftuADD_STR) == 0)
	    {
	    sprintf(historyEntry, "FTU-%s/%s/%s: %s",
		    Conditions().Version(), Date(), action, keyActionDescr);
	    }
	else
	    {
	    slxKEYW_LINE  value;
	    if (fitsObj->GetKeyword(keyValSrc) == FAILURE)
		return FAILURE;
	    strcpy(value, fitsObj->CK().ValueString());
	    slxTrimString(value, " \n\t");
	    if (fitsObj->CK().Type() == slxLOGICAL)
		slxTrimString(value, "\"");
	    sprintf(historyEntry, "FTU-%s/%s/%s: %s, %s",
		    Conditions().Version(),
		    Date(), action, keyActionDescr, value);
	    }
	if (fitsObj->AddHistory(historyEntry) == FAILURE)
	    return FAILURE;
        } 

    return SUCCESS;
} 


utilLOGICAL ftuFTT::AllowKeyRemove(const char *key)
{
    // We do not allow to rename, move or remove keywords: BITPIX.
    if (strcmp(key, "BITPIX") == 0)
	{
	INFO(1,"WARNING: Request to rename/move/remove keyword %s ignored.",
	     key);
	return FALSE;
	}
    else
	return TRUE;
}


ftuFITS_HDR_TYPES ftuFTT::HeaderType(oslxFITS   &fitsObj)
{
    if (fitsObj.KeyInBuffer("SIMPLE"))
	return ftuPRIM_HDR;
    else if (fitsObj.KeyInBuffer("XTENSION"))
	{
	fitsObj.GetKeyword("XTENSION");
	if (strstr(fitsObj.CK().String(), ftuIMAGE_EXT_STR) != NULL)
	    return ftuIMAGE_EXT;
	else if (strstr(fitsObj.CK().String(), ftuBIN_TABLE_EXT_STR) != NULL)
	    return ftuBIN_TABLE_EXT;
	else if (strstr(fitsObj.CK().String(), ftuASCII_TABLE_EXT_STR) != NULL)
	    return ftuASCII_TABLE_EXT;
	else
	    return ftuUNKNOWN_HDR_TYPE;
	}
    else
	return ftuUNKNOWN_HDR_TYPE;
}


void               ftuFTT::ResetNoOfTransCount()
{
    noOfTransPerf_ = 0;
}


void               ftuFTT::IncNoOfTransCount()
{
    noOfTransPerf_++;
}


int                ftuFTT::GetNoOfTransCount()
{
    return noOfTransPerf_;
}


utilSTAT         ftuFTT::GetValue(oslxFITS        &fits, 
				  const int       hdrNo,
				  const int       scanHdrs,
				  ftuVALUE_SRC    &valueSource,
				  slxVALUE        value,
				  ftuTRANSLATION  &translation)
{
    *value = '\0';
    switch (valueSource.GetSource())
	{
	case ftuSRC_KEY:
	    INFO(3,"Getting value for keyword: %s.",
		 valueSource.GetReference());

	    // Determine from which header to extract the keyword:
	    //
	    // 1. If a header is specified in the value source, take that.
	    // 2. If Header Scanning is specified, do that.
	    // 3. If a Header is specified take that.
	    //
	    // IMPL.: This approach can be discussed.
	    int startHdr, stopHdr;
	    if (valueSource.GetHeader() != ftuUNDEFINED)
		{
		INFO(2, "Getting value from header %d specified in Value "
		     "Source.", valueSource.GetHeader());
		startHdr = valueSource.GetHeader();
		stopHdr = (valueSource.GetHeader() + 1);
		}
	    else if (scanHdrs == ftuSCAN_HDRS)
		{
		INFO(2, "Scanning through headers ...");
		startHdr = 0;
		stopHdr = fits.NoOfHdrs();
		}
	    else
		{
		INFO(2, "Getting value from header %d.", hdrNo);
		startHdr = hdrNo;
		stopHdr = (hdrNo + 1);
		}

	    oslxFITS *hdrPtr;
	    hdrPtr = NULL;
	    int hdNo, found;
	    found = FALSE;
	    for (hdNo = startHdr; (hdNo < stopHdr) && !found; hdNo++)
		{
		if ((hdrPtr = fits.GetHdr(hdNo + 1)) == NULL)
		    {
		    sprintf(errBuf_, "Could not get header number: %d "
			    "from FITS file: %s.", hdNo, fits.FitsFileName());
		    ErrAdd("ftu", ftuERR_GEN, utilERR_WARNING, errBuf_);
		    return FAILURE;
		    }

		// We mark this only as a WARNING condition, since if Tolerant
		// Mode is used we ignore this.
		if (hdrPtr->GetKeyword(valueSource.GetReference()) == FAILURE)
		    {
		    errResetStack(&ErrStack());
		    if (hdNo >= (stopHdr - 1))
			{
			slxKEYW_LINE  tmpKey;
			strcpy(tmpKey, valueSource.GetReference());
			slxTrimString(tmpKey, " \t");
			if (*tmpKey == '\0')
			    {
			    slxVALUE action;
			    strcpy(action, 
				   ftuAction2Str(translation.GetAction()));
			    sprintf(errBuf_, "Keyword missing for a value "
				    "source for action %s.", action);
			    }
			else
			    sprintf(errBuf_, "Error retrieving keyword %s "
				    "from FITS header %d.", tmpKey, hdrNo);
			INFO(1,errBuf_);
			ErrAdd("ftu", ftuERR_GEN, utilERR_WARNING, errBuf_);
			return FAILURE;
			}   // end-if (hdNo < stopHdr)
		    }   // end-if (hdrPtr->GetKeyword( ...
		else
		    found = TRUE;
		}   // end-for (int hdNo = startHdr; (hdNo <= stopHdr) ...
	    // Should not be possible to reach this point with hdrPtr == NULL,
	    // but just in case ...
	    if (hdrPtr != NULL)
		strcpy(value, hdrPtr->CK().ValueString());
	    else
		*value = '\0';
	    break;
	case ftuSRC_DB:
	    INFO(3,"Querying value from DB, DB query: %s.",
		 valueSource.GetReference());
	    *value = '\0';
	    break;
	case ftuSRC_FCT:
	    INFO(3,"Getting value from a function: %s.",
		 valueSource.GetReference());
	    if (GetFctValue(valueSource.GetReference(), value) == FAILURE)
		return FAILURE;
	    break;
	default:
	    // case ftuSRC_CONST:
	    INFO(3,"Getting constant value: %s.", valueSource.GetReference());
	    strcpy(value, valueSource.GetReference());
	    break;
	}   // end-switch (valueSource.GetSource())
    slxTrimString(value, " \"\t\n");
    
    return SUCCESS;
}


utilSTAT         ftuFTT::GenValue(oslxFITS         &fits, 
				  slxKEYW_LINE     targKey,
				  ftuTRANSLATION   &translation,
				  utilFITS_VAL     &fitsVal,
				  slxVALUE         rawValue)
{
    // For the moment we can handle up to 16 input parameters. All values
    // are considered as strings, and are converted in the function to the
    // appropriate type.
    slxVALUE       values[16];
    ftuVALUE_SRC   *value;
    for (int valNo = 0; valNo < translation.NoOfValues(); valNo++)
	{
	if (translation.GetValue(valNo, &value) == FAILURE)
	    return FAILURE;
	// Get reference to the source FITS header.
	if (GetValue(fits, translation.GetSrcHdr(), ftuNO_HDR_SCAN, *value, 
		     values[valNo], translation) == FAILURE)
	    return FAILURE;
	}   // end-for (int valNo = 0; ...

    // Generate the final value. If a function is registered we apply this
    // on the values queried. Otherwise, we simply convert the one value
    // to the appropriate type for that keyword.
    if (fitsDicDyn_.LookUp(targKey) == FAILURE)
	return FAILURE;
    if (*translation.GetFct() == '\0')
	{
	fitsVal.SetFromString(fitsDicDyn_.DicKeywType(), values[0]);
	strcpy(rawValue, values[0]);
	}
    else
	{
	if (ComputeValue(translation.GetFct(), translation.NoOfValues(),
			 values, rawValue) == FAILURE)
	    return FAILURE;
	fitsVal.SetFromString(fitsDicDyn_.DicKeywType(), rawValue);
	//strcpy(rawValue, rawValue);
	}

    return SUCCESS;
}


/*___oOo___*/
