/******************************************************************************
 * E.S.O. - Archive System
 *
 * "@(#) $Id: ftuSESSION.C,v 1.2.2.1 2008/09/11 08:11:40 cguirao Exp $"
 *
 * Who       When        What
 * --------  ----------  ----------------------------------------------
 * jknudstr  20/01/2000  Created 
 */

/************************************************************************
 *
 *   NAME
 *   ftuSESSION - Implementing the managing for a translation session
 * 
 *   SYNOPSIS
 *   
 *   PARENT CLASS
 * 
 *   DESCRIPTION
 *   The class provides two fundamental methods which are used to carry
 *   out a translation session.
 * 
 *   The class can be used from e.g. application to implement the 
 *   translation facility.
 * 
 *   PUBLIC METHODS
 *
 *   utilSTAT     ParseInputPars(int argc, char *argv[])
 *   Parse the input parameters. See fitsTranslate.C for the complete
 *   list of parameters.
 *
 *   utilSTAT     Execute()
 *   Execute the translation according to the input parameters parsed.
 *
 *   PUBLIC DATA MEMBERS
 *
 *   PROTECTED METHODS
 *
 *   PROTECTED DATA MEMBERS
 *
 *   FILES
 *
 *   ENVIRONMENT
 *
 *   RETURN VALUES
 *
 *   CAUTIONS 
 *
 *   EXAMPLES
 *
 *   SEE ALSO
 *
 *   BUGS   
 * 
 *------------------------------------------------------------------------
 */

static const char *rcsId="@(#) $Id: ftuSESSION.C,v 1.2.2.1 2008/09/11 08:11:40 cguirao Exp $"; 
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

#define _ALL_SOURCE 1
#define __EXTENSIONS__ 1

#include "ftuSESSION.h"

ftuSESSION::ftuSESSION()
{
    *logFile_ = '\0';
    *fttFile_ = '\0';
}

ftuSESSION::~ftuSESSION()
{
}

void ftuSESSION::SetFttFile(const slxFILENAME fttFile)
{
    strcpy(fttFile_, fttFile);
}

char *ftuSESSION::GetFttFile() 
{
    return(fttFile_);
}

utilSTAT ftuSESSION::ParseInputPars(int argc, char *argv[])
{
    slxFILENAME  tmpBuf;

    // Add a log indicating the version number of the FTU + the FTT used.
    slxVALUE comBuf;
    sprintf(comBuf, "FTU Version: %s", conditions_.Version());
    SetLogLevel(1);INFO(1,comBuf);SetLogLevel(0);

    fileTbl_.DeleteTable();

    for (int i = 1; i < argc; i++)
        {
        strcpy(tmpBuf, argv[i]);
        slxStrToUpper(tmpBuf);
        if (*argv[i] != '-')
            {
            char  *filenamePtr;
            do
                {
                if (*argv[i] != '-')
                    {
                    INFO(1,"Registering file to be translated: %s.",argv[i]);
                    filenamePtr = slxStrDup(argv[i]);
                    if (fileTbl_.AddEl(fileTbl_.NoOfEls(),
				       (void **)&filenamePtr) == FAILURE)
                        return FAILURE;
                    }
                i++;
                }
            while ((i < argc) && (*argv[i] != '-'));
            if (i < argc)
                if (*argv[i] == '-')
                    i--;
            }
        else if (strcmp(tmpBuf, "-V") == 0)
            {
            if ((i + 1) >= argc)
                goto correctUsage;
            i++;
            SetVerboseLevel(atoi(argv[i]));
            INFO(2,"Setting verbose output level to: %d.",atoi(argv[i]));
            }
        else if (strcmp(tmpBuf, "-NOSTAT") == 0)
            {
            INFO(1,"Switching off generation of statistics.");
	    conditions_.Statistics(FALSE);
            }
        else if (strcmp(tmpBuf, "-NOCHECKSUM") == 0)
            {
            INFO(1,"Switching off generation of checksum.");
	    conditions_.Checksum(FALSE);
            }
        else if (strcmp(tmpBuf, "-NOHISTORY") == 0)
            {
            INFO(1,"Switching off generation of HISTORY entries for changed "
		 " keys in FITS file.");
	    conditions_.History(FALSE);
            }
        else if (strcmp(tmpBuf, "-NOCOMMENT") == 0)
            {
            INFO(1,"Do not generate a COMMENT entry indicating that this "
		 "file was translated.");
	    conditions_.Comment(FALSE);
            }
        else if (strcmp(tmpBuf, "-SKIPFILEONERROR") == 0)
            {
            INFO(1,"File will be skipped if it cannot be translated.");
	    conditions_.SkipFileOnError(TRUE);
	    }
	else if (strcmp(tmpBuf, "-FORCE") == 0)
	    {
	    INFO(1,"Forcing translation.");
	    conditions_.Force(TRUE);
	    }
	else if (strcmp(tmpBuf, "-LOGFILE") == 0)
	    {
            if ((i + 1) >= argc)
                goto correctUsage;
            i++;
	    INFO(2,"Logging into file: %s.",argv[i]);
	    SetLogFile(argv[i]);
	    SetLogCaching(TRUE);
	    }
        else if (strcmp(tmpBuf, "-LOGLEVEL") == 0)
            {
            if ((i + 1) >= argc)
                goto correctUsage;
            i++;
            SetLogLevel(atoi(argv[i]));
            INFO(2,"Setting Logging Level to: %d.",atoi(argv[i]));
            }
	else if (strcmp(tmpBuf, "-TEST") == 0)
	    {
	    INFO(2,"Switching creating of <filename>_test files on.");
	    conditions_.TestFiles(TRUE);
	    }
	else if (strcmp(tmpBuf, "-FTT") == 0)
	    {
            if ((i + 1) >= argc)
                goto correctUsage;
            i++;
	    INFO(1,"Using FTT File: %s.",argv[i]);
	    SetFttFile(argv[i]);
	    }
	else if (strcmp(tmpBuf, "-TOLERANT") == 0)
	    {
	    INFO(1,"Running in Tolerant error handling mode.");
	    conditions_.Tolerant(TRUE);
	    }
	else if (strcmp(tmpBuf, "-OVERWRITE") == 0)
	    {
	    INFO(1,"Running in File Overwrite Mode.");
	    conditions_.Overwrite(TRUE);
	    }
	else if (strcmp(tmpBuf, "-TARGETDIR") == 0)
	    {
	    if ((i + 1) >= argc)
                goto correctUsage;
            i++;
	    INFO(1,"Target Directory Specified to: %s.",argv[i]);
	    conditions_.TargetDir(argv[i]);
	    }
	else if (strcmp(tmpBuf, "-TARGETFILE") == 0)
	    {
	    if ((i + 1) >= argc)
                goto correctUsage;
            i++;
	    INFO(1,"Target File Specified to: %s.",argv[i]);
	    conditions_.TargetFile(argv[i]);
	    }
	else if (strcmp(tmpBuf, "-REPEAT") == 0)
	    {
	    if ((i + 1) >= argc) goto correctUsage;
            i++;
	    INFO(1,"Repeat factor set to: %d.",atoi(argv[i]));
	    conditions_.Repeat(atoi(argv[i]));
	    }
	else
	    goto correctUsage;
	}
    
    // At least one FITS filename and a name of an FTT must be given in.
    if (fileTbl_.NoOfEls() == 0)
	goto correctUsage;

    /*
      #ifdef __linux__
      if (conditions_.Checksum())
      {
      INFO(1,"Calculation of checksum not supported on Linux for the "
      "time being. Switching off generation of checksum.");
      conditions_.Checksum(FALSE);
      }
      #endif
    */

  return SUCCESS;

  correctUsage:
    printf("\nCorrect usage is:\n\n");
    printf("%%> %s [-v <level=1..5>] [-force] [-test] "
	   "[-noStat] [-noChecksum]\n"
	   "          [-noHistory] [-noComment] [-skipFileOnError]\n"
	   "          [-logFile <filename>] [-logLevel <level>] [-tolerant]\n"
	   "          [-overwrite] [-ftt <ftt name>] [-targetDir <dir>]\n"
	   "          [-targetFile <name>] <file (list)>\n"
	   "\n"
	   "Parameters:\n"
	   "\n"
	   "-v <level=1..5>:     Verbose mode. It is possible to specify the\n"
	   "                     the verbose level, i.e. how detailed the\n"
	   "                     information should be.\n"
	   "\n"
	   "-force:              Force update of a file which has already\n"
	   "                     been updated by the tool.\n"
	   "\n"
	   "-test:               Don't update the FITS files directly, but\n"
	   "                     create target files with the name\n"
	   "                     \"<filename>_test.fits\". This feature is\n"
	   "                     used while implementing/testing an FTT.\n"
	   "\n"
	   "-noStat:             Do not generate statistics for the data\n"
	   "                     (minimum, maximum, mean, median).\n"
	   "\n"
	   "-noChecksum:         Do not calculate checksum. If the CHECKSUM\n"
	   "                     is encountered in the header, it is\n"
	   "                     removed. Default, checksum is always\n"
	   "                     calculated.\n"
	   "\n"
	   "-noHistory:          Disable reporting of original key values\n"
	   "                     changed in the FITS files (HISTORY key).\n"
	   "\n"
	   "-noComment:          Do not write the COMMENT entry indicating\n"
	   "                     that this file was translated.\n"
	   "\n"
	   "-skipFileOnError:    Indicates if the tool should continue even\n"
	   "                     if the translation of one of the files from\n"
	   "                     a list fails.\n"
	   "\n"
	   "-logFile <filename>: Log information into the specified file.\n"
	   "                     If the file exists, the information is\n"
	   "                     appended.\n"
	   "\n"
	   "-logLevel <level>:   Specify the level at which log information\n"
	   "                     should be produced ([1; 5]). Default is 1.\n"
	   "\n"
	   "-tolerant:           Try to continue translate a FITS file even\n"
	   "                     though a WARNING condition is encountered.\n"
	   "\n"
	   "-overwrite:          Overwrite target FITS files existing. I.e.,\n"
	   "                     don't force unique, indexed names.\n"
	   "\n"
	   "-targetDir <dir>:    Specify a Target Directory in which to put\n"
	   "                     the FITS files generated. Overwrites a\n"
	   "                     possible Target Directory specified in the\n"
	   "                     FTT (FILE.PATH.*).\n"
	   "\n"	   
	   "-targetFile <name>:  Specify a Target Filename to use to build\n"
	   "                     the names of the FITS files generated.\n"
	   "                     Overwrites possible settings in this\n"
	   "                     connection in the FTT (FILE.RENAME.*,\n"
	   "                     FILE.SPLIT.*).\n"
	   "\n"
	   "-ftt <ftt name>:     Name of the FITS Translation Table to use.\n"
	   "\n"
	   "<file (list)>:       The list of files to translate.\n"
	   "\n"
	   "\n"
	   "(c) ESO/DMD 2000 - FTU-%s - %s.\n"
	   "\n", 
	   argv[0], conditions_.Version(), conditions_.VersionDate());
    return FAILURE;
}

utilSTAT ftuSESSION::Execute()
{
    utilFILE_MGR  fileMgr;
    ftuFTT        ftt(conditions_);

    // Load the FTT(s).
    if (ftt.Load(GetFttFile(), FALSE) == FAILURE)
	goto errorExit;

    // Loop over the FITS files to translate. Note, references to files
    // could be using wildcards. Therefore we have to resolve and possibly
    // loop over several filenames for each 'raw' filename given in as
    // input parameter.
    char         *filenamePtr;
    for (int it = 0; it < conditions_.Repeat(); it++)
	{
	if (conditions_.Repeat() > 1)
	    INFO(3,"Repeating time number: %d", (it + 1));

	for (int fileNo = 0; fileNo < fileTbl_.NoOfEls(); fileNo++)
	    {
	    if (fileTbl_.GetEl(fileNo, (void **)&filenamePtr) == FAILURE)
		goto errorExit;
	    INFO(3,"Pattern for next file to translate is: %s.",filenamePtr);
	    fileMgr.Reset();
	    if (fileMgr.GlobMatch(filenamePtr) == FAILURE)
		goto errorExit;
	    if (fileMgr.NoOfFilesFound() == 0)
		{
		ErrStackReset();
		sprintf(errBuf_,"Error locating file: %s.",filenamePtr);
		INFO(1,errBuf_);
		ErrAdd("ftu", ftuERR_GEN, utilERR_FATAL, errBuf_);
		if (!conditions_.SkipFileOnError())
		    {
		    LogError(utilERR_FATAL, *(utilERROR *)this); 
		    ClearStack();
		    goto errorExit;
		    }
		else
		    {
		    LogError(utilERR_WARNING, *(utilERROR *)this); 
		    ClearStack();
		    INFO(1,"-- Skip-File-On-Error is on - ignoring error.");
		    }
		}

	    slxFILENAME  filename;
	    for (int fileFoundNo = 0; fileFoundNo < fileMgr.NoOfFilesFound();
		 fileFoundNo++)
		{
		if (fileMgr.GetFileComplete(fileFoundNo, filename) == FAILURE)
		    goto errorExit;
		if (ftt.Translate(filename) == FAILURE)
		    {
		    INFO(1,"Problem ocurred while translating file: %s ...",
			 filename);
		    if (Severity() == utilERR_FATAL)
			{
			INFO(1,"Cannot continue translation - aborting!!");
			goto errorExit;
			}
		    if (conditions_.SkipFileOnError())
			{
			// Log the error and continue.
			INFO(1,"-- Skip-File-On-Error is on - ignoring "
			     "error.");
			CloseStack();
			}
		    else
			{
			// Bail out.
			INFO(1,"-- Skip-File-On-Error is NOT on - "
			     "bailing out.");
			goto errorExit;
			}
		    } 
		}
	    }   // end-for (int fileNo = 0; fileNo < fileTbl_.NoOfEls(); ...
	}   // end-for (int it = 0; it < conditions_.Repeat(); it++)

    return SUCCESS;

  errorExit:
    //LogError(Severity(), *(utilERROR *)this);
    LogFlushError(Severity(), *(utilERROR *)this);
    //LogFlush();
    return FAILURE;
}

/*___oOo___*/
