/******************************************************************************
 * E.S.O. - Archive System
 *
 * "@(#) $Id: ftuTRANSLATION.C,v 1.1.2.1 2008/09/11 08:11:40 cguirao Exp $"
 *
 * Who       When        What
 * --------  ----------  ----------------------------------------------
 * jknudstr  20/01/2000  Created 
 */

/************************************************************************
 *
 *   NAME
 *   ftuTRANSLATION - Contains information for one translation.
 * 
 *   SYNOPSIS
 *   
 *   PARENT CLASS
 * 
 *   DESCRIPTION
 *   The class is used as a place holder for the information in connection
 *   with one translation.
 *
 *   PUBLIC METHODS
 *   utilSTAT   SetAction(const ftuACTION  action)
 *   ftuACTION  GetAction()
 *   Set/get the action related to this translation, e.g. Copy, Change, ... 
 *
 *   utilSTAT   SetSrcKey(const slxKEYW_LINE  key)
 *   char       *GetSrcKey()
 *   Set/get the source keyword of the object. If there is only one key
 *   related to a translation, the source key should be used.
 *
 *   utilSTAT   SetTargKey(const slxKEYW_LINE  key)
 *   char       *GetTargKey()
 *   Set/get thre target key for the translation.
 *
 *   utilSTAT   SetSrcHdr(const slxKEYW_LINE  hdr)
 *   int        GetSrcHdr()
 *   Set/get the source header for the translation. If only one header is
 *   involved in the translation, only this key should be used.
 *
 *   utilSTAT   SetTargHdr(const slxKEYW_LINE  hdr)
 *   int        GetTargHdr()
 *   Set/get the target header for the translation.
 *
 *   utilSTAT   AddValueSrcAdr(const char    *valueSrc,
 *	 		       const char    *srcAdr)
 *   Add a value source reference from which a value will be extracted
 *   in the translation.
 *
 *   utilSTAT   SetFct(const slxKEYW_LINE  fct)
 *   char       *GetFct()
 *   Set the Conversion Function to process the values specified for the
 *   keyword.
 *
 *   int       NoOfValues()
 *   Return the number of value sources specified for the translation.
 *
 *   utilSTAT  GetValue(const int      valueNo,
 *	                ftuVALUE_SRC   **value)
 *   Get a specific value from the set of values sources. The first value
 *   source has number 0.
 *
 *
 *   PUBLIC DATA MEMBERS
 *
 *   PROTECTED METHODS
 *
 *   PROTECTED DATA MEMBERS
 *
 *   FILES
 *
 *   ENVIRONMENT
 *
 *   RETURN VALUES
 *
 *   CAUTIONS 
 *
 *   EXAMPLES
 *
 *   SEE ALSO
 *
 *   BUGS   
 * 
 *------------------------------------------------------------------------
 */

static const char *rcsId="@(#) $Id: ftuTRANSLATION.C,v 1.1.2.1 2008/09/11 08:11:40 cguirao Exp $"; 
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

#include <string.h>

#include "ftuTRANSLATION.h"
 
// Callback to handle deletion of the ftuTRANSLATION class.
void fttTranslationDeleteEl(void *elPtr) {
    delete (ftuTRANSLATION *)elPtr;
}


ftuTRANSLATION::ftuTRANSLATION(const ftuACTION  action)
{
    Clear(); 
    SetAction(action); 
}


ftuTRANSLATION::~ftuTRANSLATION()
{
}


utilSTAT ftuTRANSLATION::Clear() 
{
    SetAction(ftuUNKNOWN);
    *srcKey_  = '\0';
    *targKey_ = '\0';
    srcHdr_   = -1;
    targHdr_  = -1;
    *fct_     = '\0';
    if (valueTbl_.DeleteTable() == FAILURE)
	return FAILURE;
     
    return SUCCESS;
}


utilSTAT ftuTRANSLATION::SetAction(const ftuACTION  action)
{
    action_ = action;
    return SUCCESS;
}


ftuACTION ftuTRANSLATION::GetAction()
{
    return action_;
}


utilSTAT  ftuTRANSLATION::AddValueSrcAdr(const char    *valueSrc,
					 const char    *srcAdr,
					 const int     header)
{
    ftuVALUE_SRC *valSrc = new(ftuVALUE_SRC);
    
    if (valSrc->SetSource(valueSrc) == FAILURE)
	goto cleanExit;
    if (valSrc->SetReference(srcAdr) == FAILURE)
	goto cleanExit;
    if (valSrc->SetHeader(header) == FAILURE)
	goto cleanExit;
    if (valueTbl_.AddEl(valueTbl_.NoOfEls(), (void **)&valSrc) == FAILURE)
	goto cleanExit;

    return SUCCESS;
    
  cleanExit:
    delete valSrc;
    return FAILURE;
}


utilSTAT  ftuTRANSLATION::SetSrcKey(const slxKEYW_LINE  key)
{
    strcpy(srcKey_, key);
    slxTrimString(srcKey_, " \t\n");
    return SUCCESS;
}


char  *ftuTRANSLATION::GetSrcKey()
{
    return srcKey_;
}


utilSTAT  ftuTRANSLATION::SetTargKey(const slxKEYW_LINE  key)
{
    strcpy(targKey_, key);
    slxTrimString(targKey_, " \t\n");
    return SUCCESS;
}


char     *ftuTRANSLATION::GetTargKey()
{
    return targKey_;
}


utilSTAT   ftuTRANSLATION::SetSrcHdr(const slxKEYW_LINE  hdr)
{
    srcHdr_ = atoi(hdr);
    return SUCCESS;
}


int  ftuTRANSLATION::GetSrcHdr()
{
    return srcHdr_; 
}


utilSTAT  ftuTRANSLATION::SetTargHdr(const slxKEYW_LINE  hdr)
{
    targHdr_ = atoi(hdr);
    return SUCCESS;
}


int  ftuTRANSLATION::GetTargHdr()
{
    return targHdr_;
}


utilSTAT  ftuTRANSLATION::SetFct(const slxKEYW_LINE  fct)
{
    strcpy(fct_, fct);
    slxTrimString(fct_, " \t\n");
    return SUCCESS;
}


char  *ftuTRANSLATION::GetFct()
{
    return fct_;
}


int      ftuTRANSLATION::NoOfValues()
{
    return valueTbl_.NoOfEls();
}


utilSTAT ftuTRANSLATION::GetValue(const int      valueNo,
				  ftuVALUE_SRC   **value)
{
    if (valueTbl_.GetEl(valueNo, (void **)value) == FAILURE)
	return FAILURE;
    return SUCCESS;
}

/*
// Checks if the translation object has a Value Source of the given
// name defined. In case yes the index of the Value Source Object is
// returned, otherwise -1 is returned.
int   ftuTRANSLATION::HasValSrc(const char    *valueSrc)
{
    ftuVALUE_SRC   *value;
    for (int srcNo = 0; srcNo < NoOfValues(); srcNo++)
	{
	if (valueTbl_.GetEl(srcNo, (void **)&value) == FAILURE)
	    return -1;
	if (strcmp(value->GetReference(), valueSrc) == 0)
	    return srcNo;
	}
    return -1;
}
*/

/*___oOo___*/
