

#include "median.h"

/*#define median_WIRTH(a,n) kth_smallest(a,n,(((n)&1)?((n)/2):(((n)/2)-1)))*/



/*---------------------------------------------------------------------------
   Function :   kth_smallest()
   In       :   array of elements, # of elements in the array, rank k
   Out      :   one element
   Job      :   find the kth smallest element in the array
   Notice   :  	use the median_WIRTH() macro to get the median. 

   				Reference:

		          Author: Wirth, Niklaus 
				   Title: Algorithms + data structures = programs 
			   Publisher: Englewood Cliffs: Prentice-Hall, 1976 
	Physical description: 366 p. 
				  Series: Prentice-Hall Series in Automatic Computation 

 ---------------------------------------------------------------------------*/


#define PIX_SWAP(a,b) { register pixelvalue t=(a);(a)=(b);(b)=t; }

pixelvalue kth_smallest(pixelvalue a[], int n, int k)
{
    register int i, j, l, m;
    register pixelvalue x;

    l = 0; m = n - 1;
    while (l < m) {
        x = a[k];
        i = l;
        j = m;
        do {
            while (a[i] < x) i++;
            while (x < a[j]) j--;
            if (i <= j) {
                PIX_SWAP(a[i], a[j]);
                i++; j--;
            }
        } while (i <= j);
        if (j < k) l = i;
        if (k < i) m = j;
    }
    return a[k];
}

#undef PIX_SWAP



/*---------------------------------------------------------------------------
   Function :   kth_smallest_double()
   In       :   array of doubles, # of elements in the array, rank k
   Out      :   one element
   Job      :   find the kth smallest element in the array
   Notice   :  	use the median_double() macro to get the median. 
 ---------------------------------------------------------------------------*/


#define DBL_SWAP(a,b) { register double t=(a);(a)=(b);(b)=t; }

double kth_smallest_double(double a[], int n, int k)
{
    register int i,j,l,m ;
    register double x ;

    l=0 ; m=n-1 ;
    while (l<m) {
        x=a[k] ;
        i=l ;
        j=m ;
        do {
            while (a[i]<x) i++ ;
            while (x<a[j]) j-- ;
            if (i<=j) {
                DBL_SWAP(a[i],a[j]) ;
                i++ ; j-- ;
            }
        } while (i<=j) ;
        if (j<k) l=i ;
        if (k<i) m=j ;
    }
    return a[k] ;
}

#undef DBL_SWAP




#define PIX_SORT(a,b) { if ((a)>(b)) PIX_SWAP((a),(b)); }
#define PIX_SWAP(a,b) { pixelvalue temp=(a);(a)=(b);(b)=temp; }

/*----------------------------------------------------------------------------
 * Function :   opt_med3()
 * In       :   pointer to array of 3 pixel values
 * Out      :   a pixelvalue
 * Job      :   optimized search of the median of 3 pixel values
 * Notice   :   found on sci.image.processing
 *              cannot go faster unless assumptions are made
 *              on the nature of the input signal.
 *--------------------------------------------------------------------------*/

pixelvalue
opt_med3(
    pixelvalue  *   p
)
{
    PIX_SORT(p[0],p[1]) ; PIX_SORT(p[1],p[2]) ; PIX_SORT(p[0],p[1]) ;
    return(p[1]) ;
}

/*----------------------------------------------------------------------------
 * Function :   opt_med5()
 * In       :   pointer to array of 5 pixel values
 * Out      :   a pixelvalue
 * Job      :   optimized search of the median of 5 pixel values
 * Notice   :   found on sci.image.processing
 *              cannot go faster unless assumptions are made
 *              on the nature of the input signal.
 *--------------------------------------------------------------------------*/

pixelvalue
opt_med5(
    pixelvalue  *   p
)
{
    PIX_SORT(p[0],p[1]) ; PIX_SORT(p[3],p[4]) ; PIX_SORT(p[0],p[3]) ;
    PIX_SORT(p[1],p[4]) ; PIX_SORT(p[1],p[2]) ; PIX_SORT(p[2],p[3]) ;
    PIX_SORT(p[1],p[2]) ; return(p[2]) ;
}

/*----------------------------------------------------------------------------
 * Function :   opt_med7()
 * In       :   pointer to array of 7 pixel values
 * Out      :   a pixelvalue
 * Job      :   optimized search of the median of 7 pixel values
 * Notice   :   found on sci.image.processing
 *              cannot go faster unless assumptions are made
 *              on the nature of the input signal.
 *--------------------------------------------------------------------------*/

pixelvalue
opt_med7(
    pixelvalue  *   p
)
{
    PIX_SORT(p[0], p[5]) ; PIX_SORT(p[0], p[3]) ; PIX_SORT(p[1], p[6]) ;
    PIX_SORT(p[2], p[4]) ; PIX_SORT(p[0], p[1]) ; PIX_SORT(p[3], p[5]) ;
    PIX_SORT(p[2], p[6]) ; PIX_SORT(p[2], p[3]) ; PIX_SORT(p[3], p[6]) ;
    PIX_SORT(p[4], p[5]) ; PIX_SORT(p[1], p[4]) ; PIX_SORT(p[1], p[3]) ;
    PIX_SORT(p[3], p[4]) ; return (p[3]) ;
}

/*----------------------------------------------------------------------------
 * Function :   opt_med9()
 * In       :   pointer to an array of 9 pixelvalues
 * Out      :   a pixelvalue
 * Job      :   optimized search of the median of 9 pixelvalues
 * Notice   :   in theory, cannot go faster without assumptions on the
 *              signal.
 *              Formula from:
 *              XILINX XCELL magazine, vol. 23 by John L. Smith
 *
 *              The input array is modified in the process
 *              The result array is guaranteed to contain the median
 *              value
 *              in middle position, but other elements are NOT sorted.
 *--------------------------------------------------------------------------*/

pixelvalue
opt_med9(
    pixelvalue  *   p
)
{
    PIX_SORT(p[1], p[2]) ; PIX_SORT(p[4], p[5]) ; PIX_SORT(p[7], p[8]) ;
    PIX_SORT(p[0], p[1]) ; PIX_SORT(p[3], p[4]) ; PIX_SORT(p[6], p[7]) ;
    PIX_SORT(p[1], p[2]) ; PIX_SORT(p[4], p[5]) ; PIX_SORT(p[7], p[8]) ;
    PIX_SORT(p[0], p[3]) ; PIX_SORT(p[5], p[8]) ; PIX_SORT(p[4], p[7]) ;
    PIX_SORT(p[3], p[6]) ; PIX_SORT(p[1], p[4]) ; PIX_SORT(p[2], p[5]) ;
    PIX_SORT(p[4], p[7]) ; PIX_SORT(p[4], p[2]) ; PIX_SORT(p[6], p[4]) ;
    PIX_SORT(p[4], p[2]) ; return(p[4]) ;
}


#undef PIX_SORT
#undef PIX_SWAP


/*---------------------------------------------------------------------------
   Function	:	median_pixelvalue()
   In 		:	allocated array of pixelvalues, # of pixels in the array
   Out 		:	1 pixel value
   Job		:	compute the median pixel value out of an array
   Notice	:	calls the fastest method depending on the number of
				elements in input.
 ---------------------------------------------------------------------------*/



pixelvalue median_pixelvalue(pixelvalue * a, int n)
{
	pixelvalue	median;
	median = 0;

	switch(n) {
		case 3:
		median = opt_med3(a);
		break ;

		case 5:
		median = opt_med5(a);
		break ;

		case 7:
		median = opt_med7(a);
		break ;

		case 9:
		median = opt_med9(a);
		break ;

		default:
/*		median = median_WIRTH(a,n);*/
		break ;
	}
	return median;
}

