
#undef  BITPIX_INT8
#undef  BITPIX_INT32
#undef  BITPIX_FP32
#undef  BITPIX_FP64
#define BITPIX_INT16

#include <stdio.h>
#include <netinet/in.h>

#include "median.h"

#ifndef htons
    #define	htons(x)	(x)
#endif

#define median_WIRTH_INT16(a,n) kth_smallest_INT16(a,n,(((n)&1)?((n)/2):(((n)/2)-1)))



/*---------------------------------------------------------------------------
   Function :   kth_smallest_INT16
   In       :   array of elements, # of elements in the array, rank k
   Out      :   one element
   Job      :   find the kth smallest element in the array
   Notice   :  	use the median_WIRTH() macro to get the median. 

   Reference:

   Author: Wirth, Niklaus 
   Title: Algorithms + data structures = programs 
   Publisher: Englewood Cliffs: Prentice-Hall, 1976 
   Physical description: 366 p. 
   Series: Prentice-Hall Series in Automatic Computation 

 ---------------------------------------------------------------------------*/


#define PIX_SWAP(a,b) { register pixelvalue t=(a);(a)=(b);(b)=t; }

pixelvalue kth_smallest_INT16(pixelvalue a[], int n, int k)
{
    register int i, j, l, m;
    register pixelvalue x;

    l = 0; m = n - 1;
    while (l < m) {
        x = (ftuINT16)htons(a[k]);
        i = l;
        j = m;
        do {
            while ((ftuINT16)htons(a[i]) < x) i++;
            while (x < (ftuINT16)htons(a[j])) j--;
            if (i <= j) {
                PIX_SWAP(a[i], a[j]);
                i++; j--;
            }
        } while (i <= j);
        if (j < k) l = i;
        if (k < i) m = j;
    }
    return (ftuINT16)htons(a[k]);
}

#undef PIX_SWAP



/*---------------------------------------------------------------------------
   Function :   kth_smallest_double_INT16()
   In       :   array of doubles, # of elements in the array, rank k
   Out      :   one element
   Job      :   find the kth smallest element in the array
   Notice   :  	use the median_double() macro to get the median. 
 ---------------------------------------------------------------------------*/


#define DBL_SWAP(a,b) { register double t=(a);(a)=(b);(b)=t; }

double kth_smallest_double_INT16(double a[], int n, int k)
{
    register int i,j,l,m ;
    register double x ;

    l=0 ; m=n-1 ;
    while (l<m) {
        x=a[k] ;
        i=l ;
        j=m ;
        do {
            while (a[i]<x) i++ ;
            while (x<a[j]) j-- ;
            if (i<=j) {
                DBL_SWAP(a[i],a[j]) ;
                i++ ; j-- ;
            }
        } while (i<=j) ;
        if (j<k) l=i ;
        if (k<i) m=j ;
    }
    return a[k] ;
}

#undef DBL_SWAP




#define PIX_SORT(a,b) { if ((a)>(b)) PIX_SWAP((a),(b)); }
#define PIX_SWAP(a,b) { pixelvalue temp=(a);(a)=(b);(b)=temp; }

/*----------------------------------------------------------------------------
 * Function :   opt_med3_INT16()
 * In       :   pointer to array of 3 pixel values
 * Out      :   a pixelvalue
 * Job      :   optimized search of the median of 3 pixel values
 * Notice   :   found on sci.image.processing
 *              cannot go faster unless assumptions are made
 *              on the nature of the input signal.
 *--------------------------------------------------------------------------*/

pixelvalue
opt_med3_INT16(
    pixelvalue  *   p
)
{
    PIX_SORT(p[0],p[1]) ; PIX_SORT(p[1],p[2]) ; PIX_SORT(p[0],p[1]) ;
    return(p[1]) ;
}

/*----------------------------------------------------------------------------
 * Function :   opt_med5_INT16()
 * In       :   pointer to array of 5 pixel values
 * Out      :   a pixelvalue
 * Job      :   optimized search of the median of 5 pixel values
 * Notice   :   found on sci.image.processing
 *              cannot go faster unless assumptions are made
 *              on the nature of the input signal.
 *--------------------------------------------------------------------------*/

pixelvalue
opt_med5_INT16(
    pixelvalue  *   p
)
{
    PIX_SORT(p[0],p[1]) ; PIX_SORT(p[3],p[4]) ; PIX_SORT(p[0],p[3]) ;
    PIX_SORT(p[1],p[4]) ; PIX_SORT(p[1],p[2]) ; PIX_SORT(p[2],p[3]) ;
    PIX_SORT(p[1],p[2]) ; return(p[2]) ;
}

/*----------------------------------------------------------------------------
 * Function :   opt_med7_INT16()
 * In       :   pointer to array of 7 pixel values
 * Out      :   a pixelvalue
 * Job      :   optimized search of the median of 7 pixel values
 * Notice   :   found on sci.image.processing
 *              cannot go faster unless assumptions are made
 *              on the nature of the input signal.
 *--------------------------------------------------------------------------*/

pixelvalue
opt_med7_INT16(
    pixelvalue  *   p
)
{
    PIX_SORT(p[0], p[5]) ; PIX_SORT(p[0], p[3]) ; PIX_SORT(p[1], p[6]) ;
    PIX_SORT(p[2], p[4]) ; PIX_SORT(p[0], p[1]) ; PIX_SORT(p[3], p[5]) ;
    PIX_SORT(p[2], p[6]) ; PIX_SORT(p[2], p[3]) ; PIX_SORT(p[3], p[6]) ;
    PIX_SORT(p[4], p[5]) ; PIX_SORT(p[1], p[4]) ; PIX_SORT(p[1], p[3]) ;
    PIX_SORT(p[3], p[4]) ; return (p[3]) ;
}

/*----------------------------------------------------------------------------
 * Function :   opt_med9_INT16()
 * In       :   pointer to an array of 9 pixelvalues
 * Out      :   a pixelvalue
 * Job      :   optimized search of the median of 9 pixelvalues
 * Notice   :   in theory, cannot go faster without assumptions on the
 *              signal.
 *              Formula from:
 *              XILINX XCELL magazine, vol. 23 by John L. Smith
 *
 *              The input array is modified in the process
 *              The result array is guaranteed to contain the median
 *              value
 *              in middle position, but other elements are NOT sorted.
 *--------------------------------------------------------------------------*/

pixelvalue
opt_med9_INT16(
    pixelvalue  *   p
)
{
    PIX_SORT(p[1], p[2]) ; PIX_SORT(p[4], p[5]) ; PIX_SORT(p[7], p[8]) ;
    PIX_SORT(p[0], p[1]) ; PIX_SORT(p[3], p[4]) ; PIX_SORT(p[6], p[7]) ;
    PIX_SORT(p[1], p[2]) ; PIX_SORT(p[4], p[5]) ; PIX_SORT(p[7], p[8]) ;
    PIX_SORT(p[0], p[3]) ; PIX_SORT(p[5], p[8]) ; PIX_SORT(p[4], p[7]) ;
    PIX_SORT(p[3], p[6]) ; PIX_SORT(p[1], p[4]) ; PIX_SORT(p[2], p[5]) ;
    PIX_SORT(p[4], p[7]) ; PIX_SORT(p[4], p[2]) ; PIX_SORT(p[6], p[4]) ;
    PIX_SORT(p[4], p[2]) ; return(p[4]) ;
}


#undef PIX_SORT
#undef PIX_SWAP


/*---------------------------------------------------------------------------
   Function	:	median_pixelvalue_INT16()
   In 		:	allocated array of pixelvalues, # of pixels in the 
                        array
   Out 		:	1 pixel value
   Job		:	compute the median pixel value out of an array
   Notice	:	calls the fastest method depending on the number of
				elements in input.
 ---------------------------------------------------------------------------*/



pixelvalue median_pixelvalue_INT16(pixelvalue *a, int n)
{
    return median_WIRTH_INT16(a, n);
}

/* --- oOo --- */

