/******************************************************************************
 * E.S.O. - Archive System
 *
 * "@(#) $Id: utilFILE_MGR.C,v 1.2 2008/09/04 15:47:41 cguirao Exp $"
 *
 * Who       When      What
 * --------  --------  ----------------------------------------------
 * jknudstr  20/08/99  Created 
 */

/************************************************************************
 *
 *   NAME
 *   
 * 
 *   SYNOPSIS
 *   
 *   PARENT CLASS
 * 
 *   DESCRIPTION
 *
 *   PUBLIC METHODS
 *
 *   PUBLIC DATA MEMBERS
 *
 *   PROTECTED METHODS
 *
 *   PROTECTED DATA MEMBERS
 *
 *   FILES
 *
 *   ENVIRONMENT
 *
 *   RETURN VALUES
 *
 *   CAUTIONS 
 *
 *   EXAMPLE
 *
 *   SEE ALSO
 *
 *   BUGS   
 * 
 *------------------------------------------------------------------------
 */

static const char *rcsId="@(#) $Id: utilFILE_MGR.C,v 1.2 2008/09/04 15:47:41 cguirao Exp $"; 
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <strings.h>
#include <string.h>

#include "utilErrorsNoCcs.h"
#include "utilUtils.h"
#include "utilFILE_MGR.h"

// Callback to handle deletion of the utilFILE_INFO class.
void fttFileInfoDeleteEl(void *elPtr) {
    delete (utilFILE_INFO *)elPtr;
}

utilFILE_MGR::utilFILE_MGR()
{
    Reset();
}

utilFILE_MGR::~utilFILE_MGR()
{
    Reset();
}

utilSTAT utilFILE_MGR::Reset()
{
    fileTable_.DeleteTable();
    noOfFilesFound_ = 0;
    return SUCCESS;
}

utilSTAT utilFILE_MGR::GetExt(const slxFILENAME  filename,
			      slxFILENAME        ext)
{
    char  *extPtr;
    extPtr = slxGetExtension(filename);
    if (extPtr == NULL)
	*ext = '\0';
    else
	{
	slxCOPY_MAX_N_BYTES(ext, extPtr, sizeof(slxFILENAME), &ErrStack());
	}

    return SUCCESS;
}

utilSTAT utilFILE_MGR::GlobMatch(const slxFILENAME  fileNamePattern)
{
    char             *completePath;
    DIR              *dirPtr;
    struct dirent    *dirEnt;
    slxFILENAME      dirNameLoc, fileNameLoc, extension;

    // If no wildcards are specified we only try to complete the filename
    // (in case e.g. environment variables were used in the name).
    if (strchr(fileNamePattern, '*') == NULL)
	{
	if (slxResolvePath(fileNamePattern, &completePath, 
			   &ErrStack()) == FAILURE)
	    return FAILURE;
	if (AddFile(completePath) == FAILURE)
	    return FAILURE;
	return SUCCESS;
	}

    // The name contains wildcards, we apply regular expression to the
    // files found in the directory referenced.

    // If the is an extension this should be taken into account.
    if (GetExt(fileNamePattern, extension) == FAILURE)
	return FAILURE;

    if (Reset() == FAILURE)
	return FAILURE;

    if (slxGetFileName(fileNamePattern, fileNameLoc, &ErrStack()) == FAILURE)
	return FAILURE;

    if (slxGetPath(fileNamePattern, dirNameLoc, &ErrStack()) == FAILURE)
	return FAILURE;
    if (*dirNameLoc == '\0')
	strcpy(dirNameLoc, "$PWD");
    if (slxResolvePath(dirNameLoc, &completePath, &ErrStack()) == FAILURE)
	return FAILURE;
    slxFILENAME  tmpPath;
    strcpy(tmpPath, completePath);
    if (slxGetLastChar(tmpPath) != '/')
	strcat(tmpPath, "/");
    slxCOPY_MAX_N_BYTES(dirNameLoc, tmpPath, sizeof(slxFILENAME), &ErrStack());
    if ((dirPtr = opendir(dirNameLoc)) == NULL)
	{
	char  errBuf[1024];
	sprintf(errBuf, "Error in opening directory %s (opendir())",
		dirNameLoc);
	ErrSysAdd("util", utilERR_GEN, __FILE_LINE__, modUNIX, errBuf); 
	goto errorCond;
	}
    
    while ((dirEnt = readdir(dirPtr)) != NULL)
	{
	if (*(dirEnt->d_name) != '.')
	    {
	    if (utilRegExp(fileNameLoc, dirEnt->d_name))
		{
		// If the extension is given, we check it against the 
		// filename.
		utilLOGICAL takeFile;
		if (*extension != '\0')
		    {
		    slxFILENAME  tmpExt;
		    if (GetExt(dirEnt->d_name, tmpExt) == FAILURE)
			return FAILURE;
		    if (strcmp(tmpExt, extension) != 0)
			takeFile = FALSE;
		    else
			takeFile = TRUE;
		    }
		else
		    takeFile = TRUE;
		if (takeFile)
		    {
		    if (AddFile(dirNameLoc, dirEnt->d_name) == FAILURE)
			return FAILURE;
		    }
		}
	    }    // end-if (*(dirEnt->d_name) != '.')
	}    // end-while ((dirEnt = readdir(dirPtr)) != NULL)

    closedir(dirPtr);
    
    return SUCCESS;
    
 errorCond:
    closedir(dirPtr);
    return FAILURE;
}

utilSTAT utilFILE_MGR::AddFile(const  char   *fileName)
{
    utilFILE_INFO  *fileInfoPtr = new(utilFILE_INFO);
    fileInfoPtr->FileName(fileName);
    if (fileTable_.AddEl(noOfFilesFound_, (void **)&fileInfoPtr) == FAILURE)
	goto errorCond;
    noOfFilesFound_++;

    return SUCCESS;

  errorCond:
    if (fileInfoPtr != NULL)
	free(fileInfoPtr);
    return FAILURE;
}

utilSTAT utilFILE_MGR::AddFile(const  char   *dir,
			       const  char   *fileName)
{
    slxFILENAME  tmpName;
    if (slxGetLastChar((char *)dir) == '/')
	sprintf(tmpName, "%s%s", dir, fileName);
    else
	sprintf(tmpName, "%s/%s", dir, fileName);
    if (AddFile(tmpName) == FAILURE)
	return FAILURE;

    return SUCCESS;
}

int     utilFILE_MGR::NoOfFilesFound()
{
    return noOfFilesFound_;
}
    
utilSTAT utilFILE_MGR::GetFile(const int    fileNo,
			       slxFILENAME  fileName)
{
    char           key[32]; 
    utilFILE_INFO   *fileInfoPtr;

    sprintf(key, "%d", fileNo);
    if (fileTable_.GetEl(key, (void **)&fileInfoPtr) == FAILURE)
	return FAILURE;
    strcpy(fileName, fileInfoPtr->FileName());

    return SUCCESS;
}

// Try to complete the filename.
utilSTAT utilFILE_MGR::GetFileComplete(const int    fileNo,
				       slxFILENAME  fileName)
{
    slxFILENAME  tmpFile;
    if (GetFile(fileNo, tmpFile) == FAILURE)
	return FAILURE;
    if ((*tmpFile != '/') && ((*tmpFile == '~') || (*tmpFile == '$') ||
			      ((*tmpFile == '.') && (*(tmpFile + 1) == '/'))))
	{
	char  *namePtr;
	if (slxResolvePath(tmpFile, &namePtr, &ErrStack()) == FAILURE)
	    return FAILURE;
	strcpy(fileName, namePtr);
	}
    else if (*tmpFile != '/')
	{
	// Try to locate the file as in the current working directory.
	char  *envPtr;
	if (slxGetValueEnvVar("PWD", &envPtr, &ErrStack()) == FAILURE)
	    return FAILURE;
	slxFILENAME  completeName;
	strcpy(completeName, envPtr);
	if (slxGetLastChar(completeName) != '/')
	    strcat(completeName, "/");
	strcat(completeName, tmpFile);
	struct stat   statBuf;
	if (stat(completeName, &statBuf) == 0)
	    strcpy(fileName, completeName);
	else
	    strcpy(fileName, tmpFile);
	}
    else
	strcpy(fileName, tmpFile);

    return SUCCESS;
}

// Following cases tested:
// "/path/path2/path3/file.name.ext"
// "file.name.ext"
// "filename"
// "/path/path2/path3/filename"
// "/path/path.2/path3/filename"
// "/path/path.2/path3/"
void utilFILE_MGR::SplitFileName(const slxFILENAME completeName,
				 slxFILENAME       path,
				 slxFILENAME       fileName,
				 slxFILENAME       extension)
{
    char  *extPtr, *fileNamePtr;

    // Get extension.
    extPtr = (char *)(completeName + (strlen(completeName) - 1));
    while ((extPtr != completeName) && ((*extPtr != '.'))) 
	extPtr--;
    if (*extPtr == '.')
	strcpy(extension, (extPtr + 1));
    else
	*extension = '\0';

    // Get filename.
    if (*extension == '\0')
	extPtr = (char *)(completeName + (strlen(completeName) - 1));
    fileNamePtr = (char *)(completeName + (strlen(completeName) - 1));
    if (*(completeName + strlen(completeName) - 1) == '/')
	*fileName = '\0';
    else
	{
	while ((fileNamePtr != completeName) && ((*fileNamePtr != '/'))) 
	    fileNamePtr--;
	if ((fileNamePtr == completeName) && (*extension == '\0'))
	    strcpy(fileName, completeName);
	else if (*extension == '\0')
	    strcpy(fileName, (fileNamePtr + 1));
	else if (fileNamePtr == completeName)
	    {
	    strncpy(fileName, fileNamePtr, (extPtr - fileNamePtr));
	    *(fileName + (extPtr - fileNamePtr)) = '\0';
	    }
	else
	    {
	    strncpy(fileName, (fileNamePtr + 1), (extPtr - fileNamePtr - 1));
	    *(fileName + (extPtr - fileNamePtr  - 1)) = '\0';
	    }
	}

    // Cover up for the case where there is no extension, but where
    // the is a dot in the path.
    if ((*extension != '\0') && (*fileName != '\0'))
	{
	if (extPtr < fileNamePtr)
	    *extension = '\0';
	}

    // Cover up for the case where these is no filename -> no filename 
    // -> no extension.
    if (*fileName == '\0')
	*extension = '\0';

    // Get path if there.
    if (fileNamePtr != completeName)
	{
	strncpy(path, completeName, (fileNamePtr - completeName));
	*(path + (fileNamePtr - completeName)) = '\0';
	}
    else
	*path = '\0';
}

utilLOGICAL utilFILE_MGR::FileExists(const slxFILENAME    fileName)
{
    struct stat   statBuf;
    if (stat(fileName, &statBuf) == 0)
	return TRUE;
    else
	return FALSE;
}


/*___oOo___*/
