/******************************************************************************
 * E.S.O. - Archive System
 *
 * "@(#) $Id: utilFITS_VAL.C,v 1.2 2008/09/04 15:47:41 cguirao Exp $"
 *
 * Who       When        What
 * --------  ----------  ----------------------------------------------
 * jknudstr  21/01/2000  Created 
 */

/************************************************************************
 *
 *   NAME
 *   utilFITS_VAL - Implements the FITS value
 * 
 *   SYNOPSIS
 *   
 *   PARENT CLASS
 * 
 *   DESCRIPTION
 *
 *   PUBLIC METHODS
 *
 *   PUBLIC DATA MEMBERS
 *
 *   PROTECTED METHODS
 *
 *   PROTECTED DATA MEMBERS
 *
 *   FILES
 *
 *   ENVIRONMENT
 *
 *   RETURN VALUES
 *
 *   CAUTIONS 
 *
 *   EXAMPLES
 *
 *   SEE ALSO
 *
 *   BUGS   
 * 
 *------------------------------------------------------------------------
 */

static const char *rcsId="@(#) $Id: utilFITS_VAL.C,v 1.2 2008/09/04 15:47:41 cguirao Exp $"; 
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

#include <stdlib.h>
#include <string.h>


#include "utilUtils.h"
#include "utilFITS_VAL.h"

utilFITS_VAL::utilFITS_VAL()
{
    type_    = slxUNKNOWN;
    string_  = NULL;
    logical_ = FALSE;
    integer_ = 0;
    double_  = 0;
}

utilFITS_VAL::~utilFITS_VAL()
{
    if (string_ != NULL)
	free(string_);
}

void         utilFITS_VAL::String(const char *string)
{
    if (string_ != NULL)
	free(string_);
    string_ = slxStrDup(string);
    slxTrimString(string_, " \"\'");
    type_ = slxSTRING;
}

char         *utilFITS_VAL::String()
{
    return string_;
}

void         utilFITS_VAL::Logical(const utilLOGICAL logical)
{
    logical_ = logical;
    type_ = slxLOGICAL;
}

utilLOGICAL   utilFITS_VAL::Logical()
{
    return logical_;
}

void         utilFITS_VAL::Integer(const int integer)
{
    integer_ = integer;
    type_ = slxINTEGER;
}

int          utilFITS_VAL::Integer()
{
    return integer_;
}

void         utilFITS_VAL::Double(const double dbl)
{
    double_ = dbl;
    type_ = slxDOUBLE;
}

double       utilFITS_VAL::Double()
{
    return double_;
}
	
slxPAR_TYPE  utilFITS_VAL::Type()
{
    return type_;
}

void         utilFITS_VAL::SetFromString(const slxPAR_TYPE  type,
					 const char         *valueAsString)
{
    switch (type)
	{
	case slxSTRING:
	    String(valueAsString);
	    break;
	case slxLOGICAL:
	    Logical(utilStr2Logical(valueAsString));
	    break;
	case slxINTEGER:
	    if ((*valueAsString == 'T') && (*(valueAsString + 1) == '\0'))
		Integer(1);
	    else if ((*valueAsString == 'F') && (*(valueAsString + 1) == '\0'))
		Integer(0);
	    else
		{
		double tmpDbl = atof(valueAsString);
		if (tmpDbl < 0)
		    Integer((int)(tmpDbl - 0.5));
		else
		    Integer((int)(tmpDbl + 0.5));
		}
	    break;
	default:
	    // case slxDOUBLE:
	    if ((*valueAsString == 'T') && (*(valueAsString + 1) == '\0'))
		Double(1);
	    else if ((*valueAsString == 'F') && (*(valueAsString + 1) == '\0'))
		Double(0);
	    else
		Double(atof(valueAsString));
	    break;
	}
}

void        utilFITS_VAL::GetAsString(slxVALUE  value)
{
    switch (type_)
	{
	case slxSTRING:
	    sprintf(value, "%s", string_);
	    break;
	case slxLOGICAL:
	    if (logical_)
		strcpy(value, "T");
	    else
		strcpy(value, "F");
	    break;
	case slxINTEGER:
	    sprintf(value, "%d", integer_);
	    break;
	default:
	    // case slxDOUBLE:
	    sprintf(value, "%.32E", double_);
	    break;
	}
}

void         *utilFITS_VAL::Value()
{
    void  *retPtr;

    switch (type_)
	{
	case slxSTRING:
	    retPtr = string_;
	    break;
	case slxLOGICAL:
	    retPtr = &logical_;
	    break;
	case slxINTEGER:
	    retPtr = &integer_;
	    break;
	default:
	    // case slxDOUBLE:
	    retPtr = &double_;
	    break;
	}
    return retPtr;
}

/*___oOo___*/
