/******************************************************************************
 * E.S.O. - Archive System
 *
 * "@(#) $Id: utilLOG.C,v 1.2 2008/09/04 15:47:41 cguirao Exp $"
 *
 * Who       When      What
 * --------  --------  ----------------------------------------------
 * jknudstr  05/08/99  Created 
 */

/************************************************************************
 *
 *   NAME
 *   
 * 
 *   SYNOPSIS
 *   
 *   PARENT CLASS
 * 
 *   DESCRIPTION
 *
 *   PUBLIC METHODS
 *
 *   PUBLIC DATA MEMBERS
 *
 *   PROTECTED METHODS
 *
 *   PROTECTED DATA MEMBERS
 *
 *   FILES
 *
 *   ENVIRONMENT
 *
 *   RETURN VALUES
 *
 *   CAUTIONS 
 *
 *   EXAMPLES
 *
 *   SEE ALSO
 *
 *   BUGS   
 * 
 *------------------------------------------------------------------------
 */

static const char *rcsId="@(#) $Id: utilLOG.C,v 1.2 2008/09/04 15:47:41 cguirao Exp $"; 
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>

#include "miscNoCcs.h"
#include "slxBasics.h"
#include "slxUtils.h"

#include "utilTYPES.h"
#include "utilHASH.h"
#include "utilLOG.h"

// Static variables.
utilLOGICAL   utilLOG::initialized_;
int           utilLOG::verboseLevel_;
int           utilLOG::logLevel_;
utilLOGICAL   utilLOG::logCaching_;
slxFILENAME   utilLOG::logFile_;
utilHASH      utilLOG::logCache_;
ccsERROR      utilLOG::tmpErr_;


utilLOG::utilLOG()
{
    if (!initialized_)
	{
	verboseLevel_ = 0;
	logLevel_ = 0;
	logCaching_ = FALSE;
	*logFile_ = '\0';
	initialized_ = TRUE;  
	SetLogCaching(TRUE);
	errResetStack(&tmpErr_);
	}
}

utilLOG::~utilLOG()
{
    LogFlush();
}

void utilLOG::SetLogFile(const char  *logFile)
{
    strcpy((char *)logFile_, logFile);
}

char *utilLOG::GetLogFile()
{
    return (char *)logFile_;
}

void utilLOG::SetVerboseLevel(const int level)
{
    verboseLevel_ = level;
}

int utilLOG::GetVerboseLevel()
{
    return verboseLevel_;
}

void utilLOG::SetLogLevel(const int level)
{
    logLevel_ = level;
}

int utilLOG::GetLogLevel()
{
    return logLevel_;
}

void utilLOG::SetLogCaching(const utilLOGICAL caching)
{
    logCaching_ = caching;  
}

utilLOGICAL utilLOG::GetLogCaching()
{
    return logCaching_;
}

void utilLOG::Log(int         level,
		  const char  *fileLine,
		  const char  *logFormat, ...)
{ 
    static int      initialized;
    static ccsERROR tmpErr;
    va_list         argPtr;
    
    if (!initialized)
	{
	errResetStack(&tmpErr);
	char  *varPtr;
	if ((varPtr = getenv("UTIL_DEBUG")) != NULL)
	    SetVerboseLevel(atoi(varPtr));
	}
    
    slxKEYW_LINE    dayTime;
    if (slxGenDayTimPrec(4, dayTime, &tmpErr) == FAILURE)
	errCloseStack(&tmpErr);

    va_start(argPtr, logFormat);

    // If the specified level is less than or equal to the logging level.
    if (level <= GetVerboseLevel())
	{
	// Print out the log information on the stdout device.

	// Generate a format like:
	// <file>:<line no>:<time stamp>> 
	fprintf(stdout, "%s:%s> ", fileLine, dayTime);
	vfprintf(stdout, logFormat, argPtr);
	fprintf(stdout,"\n"); 
	fflush(stdout);
	}
 
    if (level <= GetLogLevel())
	{
	// If caching is on we cach the logs into the log hash table.
	char   logLine[512];
	if (GetLogCaching() || (*GetLogFile() != '\0'))
	    {
	    char   tmpLogLine[256];   
	    vsprintf(tmpLogLine, logFormat, argPtr);
	    sprintf(logLine, "%s [INFO] %s\n", dayTime, tmpLogLine);
	    }
	va_end(argPtr);
	
	// Handle logs for the log file.
	if (GetLogCaching())
	    {
	    char  *elPtr;
	    elPtr = slxStrDup(logLine);
	    if (logCache_.AddEl(logCache_.NoOfEls(), 
				(void **)&elPtr) == FAILURE)
		errCloseStack(&tmpErr);
	    }
	else if (*GetLogFile() != '\0')
	    WriteLog(logLine);

	// Test if we should auto-flush logs.
	if (logCache_.NoOfEls() >= utilLOG_AUTO_FLUSH_SZ)
	    LogFlush();
	}
}

utilSTAT utilLOG::LogFlush()
{
    int           noOfLogs;
    FILE          *fp;
    fp = NULL;

    if (((noOfLogs = logCache_.NoOfEls()) > 0) && (*GetLogFile() != '\0'))
	{
	if ((fp = fopen(GetLogFile(), "a+")) == NULL)
	    {
	    fprintf(stderr, "Error writing in log file: %s.\n", GetLogFile());
	    goto errorExit;
	    }
	
	char  *logPtr;
	for (int logNo = 0; logNo < noOfLogs; logNo++)
	    {
	    if (logCache_.GetEl(logNo, (void **)&logPtr) == FAILURE)
		{
		fprintf(stderr, "Error accessing log cache (No. of logs: %d, "
			"Req. Log No.: %d).\n", noOfLogs, logNo);
		goto errorExit;
		}
	    if (slxGetLastChar((char *)logPtr) != '\n')
		fprintf(fp, "%s\n", logPtr);
	    else
		fprintf(fp, "%s", logPtr);
	    } 
	fclose(fp);
	}

    logCache_.DeleteTable();

    return SUCCESS;

  errorExit:
    if (fp != NULL)
	fclose(fp);
    return FAILURE;
}

utilSTAT utilLOG::LogFlushError(const utilERR_SEVERITY  severity,
				utilERROR               &error)
{
    LogError(severity, error);
    LogFlush();
    return SUCCESS;
}

utilSTAT utilLOG::LogError(const utilERR_SEVERITY  severity,
			   utilERROR               &error)
{
    for (int errNo = 0; errNo < error.NoOfErrors(); errNo++)
	{
	if (LogError(severity, error.GetError(errNo)) == FAILURE)
	    return FAILURE;
	}   
    return SUCCESS;
}

utilSTAT utilLOG::LogError(const utilERR_SEVERITY  severity,
			   ccsSTACK_ELEM           *error)
{
    slxKEYW_LINE       logLine;

    LogFlush();

    // Write the error in the log file if defined, otherwise write 
    // it to stdout. Format of the log is: "<time> [<type>] <text>".
    char  *ts;
    slxVALUE  timeStamp;
    if (strstr((char *)error->timeStamp, "<not set>") != NULL)
	{
	if (slxGenDayTimPrec(3, timeStamp, &tmpErr_) == FAILURE)
	    {
	    errResetStack(&tmpErr_);
	    return FAILURE;
	    }
	ts = timeStamp;
	}
    else
	ts = (char *)error->timeStamp;

    if (severity == utilERR_WARNING)
	sprintf(logLine, "%s [%s] %s%s", ts, utilERR_WARNING_STR,
		error->runTimePar, "");
    else if (severity == utilERR_SEVERE)
	sprintf(logLine, "%s [%s] %s%s", ts, utilERR_SEVERE_STR,
		error->runTimePar, "");
    else
	sprintf(logLine, "%s [%s] %s%s", ts, utilERR_FATAL_STR,
		error->runTimePar, "");
    
    if (WriteLog(logLine) == FAILURE)
	return FAILURE;
    
    return SUCCESS;
}

utilSTAT utilLOG::WriteLog(const char *log)
{
    slxKEYW_LINE logLoc;
    strcpy(logLoc, log);
    //if (slxGetLastChar((char *)log) != '\n')
    //strcat(logLoc, "\n");
    slxTrimString(logLoc, " \n");

    utilLOGICAL logStdErr = TRUE;
    if (*GetLogFile() != '\0')
	{ 
	FILE          *fp;
	if ((fp = fopen(GetLogFile(), "a+")) == NULL)
	    {
	    fprintf(stderr, "Error opening log file: %s.\n", GetLogFile());
	    return FAILURE;
	    }
	fprintf(fp, "%s\n", logLoc);
	/*
	  if (slxGetLastChar((char *)logLoc) != '\n')
	  fprintf(fp, "%s\n", logLoc);
	  else
	  fprintf(fp, "%s", logLoc);
	*/
	fclose(fp);
	}
    
    if (logStdErr)
	fprintf(stderr, "%s\n", logLoc);

    return SUCCESS;
}

/*___oOo___*/
