/*******************************************************************************
* E.S.O. - VLT project
*
* slxFindSetupFile.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  20/09/94  Created 1. version
*/

/************************************************************************
*   NAME
*   slxFindSetupFile - finds a set-up file 
* 
*   SYNOPSIS
*   slxFindSetupFile <filename>
*
* 
*   DESCRIPTION
*   The program uses the function "slxLocateSetUp()" to locate a set-up file.
*   This is done in accordance with the search criterias described in 
*   INS Common SW/VLT-SPE-ESO-17240-0385.
*
*   The program writes to standard output one of the two strings:
*
*   1. File located as "<complete path>/<filename>"
*
*   2. Didn't find the file "<filename>"
* 
*   
*   ENVIRONMENT
*   INS_SETUPPATH   A colon separated list of paths where the set-up files can
*                   be stored.
*
*
*   RETURN VALUES
*   0   In case the operation was successfull. 
*   1   In case the it was not possible to obtain the value of INS_SETUPPATH.
*         
*
*   CAUTIONS 
*   The function returns the first occurrence of the file which is found.
*   If there is duplicate files, this might not be the desired file.
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxFindSetupFile.c,v 1.145 2005/07/27 09:12:47 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

#include <string.h>
/* Local Headers 
 */
#include "slx.h"

/* Main
 */
int main (int argc, char *argv[])
{
    ccsERROR     error;
    ccsPROCNAME  procName;
    char         completeFileName[256];


    if (argc < 2)
        {
        printf("\nCorrect syntax is: \n");
        printf("\nslxFindSetUpFile   <filename>\n\n");
        return(1);
        }

    strcpy(procName, "slxFindSUPFile");
    if (ccsInit(procName, 0, NULL, NULL, &error) == FAILURE)
        {
        printf("\nError in intializing on RTAP!!\n\n");
        return(1);
        }

    if (slxLocateSetupFile(argv[1], completeFileName, &error) != SUCCESS)
        {
        printf("Didn't find the file \"%s\"\n", argv[1]);
        ccsExit(&error);
        return(1);
        }

    if (*completeFileName != '\0')
        {
        printf("File located as \"%s\"\n", completeFileName);
        ccsExit(&error);
        return(0);
        }
    else
        {
        printf("Didn't find the file \"%s\"\n", argv[1]);
        ccsExit(&error);
        return(1);
        }
}






