/*******************************************************************************
* E.S.O. - VLT project
*
* slxGetNextKeyBuf.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* alongino  23/08/95  created
* alongino  07/09/95  modify to take out trailing ";"
*/

/************************************************************************
*   NAME
*   slxGetNextKeyBuf - Get from slx Dynamic buffer next keyword and value
* 
*   SYNOPSIS
*   #include "slx.h"
*   ccsCOMPL_STAT slxGetNextKeyBuf(slxDYN_BUF           *setupHandle,
*                                  const slxDYN_BUF_OP  initDynBuf,
*                                  char                 *keyword,
*                                  char                 *value,
*                                  ccsERROR             *error)
* 
*   DESCRIPTION
*   The function returns the sequentially next setup keyword and associated
*   value from the "setupHandle" buffer.
*
*   Before starting out using the function, it should be called once with
*   the "initDynBuf" set to slxINIT_DYN_BUF; no keyword is returned in this
*   case. Afterwards "initDynBuf" must be slxDO_NOT_INIT.
*
*   When the last keyword has been read from the buffer, the length of the
*   the "keyword" string will be 0.
*
*   setupHandle  IN  SLX Dynamic Buffer containing setup keyword information.
*
*   initDynBuf   IN  Indicates if the SLX Dynamic Buffer internal pointer
*                    shall be reset. Must be either slxINIT_DYN_BUF, or
*                    slxDO_NOT_INIT.
*
*   keyword      OUT String containing keyword name (length slxKEYW_LINE)
*
*   value        OUT String containing keyword value (length slxKEYW_LINE)
*                    If quoted, quotes are stripped. 
*
*   error        OUT CCS error structure indicating a possible error
*                    occurring.
*
*   RETURN VALUES 
*   SUCCESS   In case the operation was successfull.
*   FAILURE   In case the operation failed.
*
*   CAUTIONS 
*   Remember to initialize the Dynamic Buffer before using the function the
*   1st time, when starting a new readout of keywords from the SLX Dynamic
*   Buffer.
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS   
*
*------------------------------------------------------------------------
*/

#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxGetNextKeyBuf.c,v 1.145 2005/07/27 09:16:32 vltsccm Exp $"; 
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

#include <string.h>
/* Local Headers
 */
#include "slx.h"

/* Extract from a slx dynamic buffer the next keyword and its value 
 */
ccsCOMPL_STAT slxGetNextKeyBuf(slxDYN_BUF           *setupHandle,
                               const slxDYN_BUF_OP  initDynBuf,
                               char                 *keyword,
                               char                 *value,
                               ccsERROR             *error)

    {
    slxKEYW_LINE keyLine;           /* setup keyword line                    */
    char *newPos;                   /* points to start of new field in line  */
    int leng;                       /* string length                         */

   /*
    *# Get with slxGetNextLineBuf the next line in the slx dynamic buffer
    */
    if (slxGetNextLineBuf(setupHandle, initDynBuf, keyLine, error) != SUCCESS)
        return(FAILURE);
    if (initDynBuf != slxINIT_DYN_BUF)
        {
       /*
        *# If line not empty, extract keyword name and value
        */
        if (strlen(keyLine) > 0)
            {
            newPos = keyLine;
            if (slxGetNextField(newPos, slxKEYW_LINE_LEN, keyword, &newPos,
                                error) != SUCCESS) return(FAILURE);
            if (slxGetNextField(newPos, slxKEYW_LINE_LEN, value, &newPos,
                                error) != SUCCESS) return(FAILURE);
            if (*value == '"') slxStripQuotes(value);
            leng = strlen(value);
            if (value[leng-1] == ';') value[leng-1]='\0'; /* no trailing ';' */
            }
       /*
        *# else (last line), just copy keyLine in keyword name and value
        */
        else
            {
            strcpy(keyword, keyLine);
            strcpy(value, keyLine);
            }
        } 

    return(SUCCESS);
    }

/*___oOo___*/
