/******************************************************************************
* E.S.O. - VLT project
*
* slxLookUpKeywordHash.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  05/09/95  created
* - - - - - - - - - - APR2004 - - - - - - - - - - - - - - - - - - - 
* mnylund   02/02/04  Problems with APR2004, static variable initialized 
*                     is initialized to ccsFALSE, VLTSW20040034
*/

/************************************************************************
*   NAME
*   slxLookUpKeywordHash - look up a keyword in an SLX Hash Table
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxLookUpKeywordHash(const slxDICTIONARY dictionary,
*                                      const slxKEYW_LINE  keyword,
*                                      slxDIC_HANDLE       **dicInfo,
*                                      ccsERROR            *error)
* 
*   DESCRIPTION
*   The function is used to perform a fast search of a setup keyword in an
*   SLX Hash Table. The parameter "dictionary" indicates the Dictionary to use.
*
*
*   dictionary   IN   Name of the Dictionary to use. Should only be the
*                     extension of the name, e.g. "ISAAC", for the Dictionary
*                     "ESO-VLT-DIC.ISAAC".
*
*   keyword      IN   Name of the keyword to look up; must be in the Short-FITS
*                     format.
*
*   dicInfo      OUT  Pointer-pointer to SLX Dictionary Structure that 
*                     contains the information of the Dictionary Record 
*                     searched. The element points to an element in the SLX
*                     Hash Table (don't modify it).
*
*   error        OUT  CCS error structure indicating a possible error 
*                     occurring.
*
*
*   FILES
*   Dictionaries   The Dictionaries used must have the format as defined for
*                  VLT Data Interface Dictionaries.
*
*   ENVIRONMENT
*   INS_ROOT       Defines the root directory of the Instrument Directory 
*                  structure.
*
*   RETURN VALUES 
*   SUCCESS   In case the operation was successfull.
*   FAILURE   In case the operation failed.
*
*
*   CAUTIONS 
*
*   EXAMPLES
*
*   SEE ALSO
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxLookUpKeywordHash.c,v 1.145 2005/07/27 09:21:44 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <string.h>
#include <stdlib.h>

/* Local Headers
 */
#include "slx.h"

/* Function definition
 */
ccsCOMPL_STAT slxLookUpKeywordHash(const slxDICTIONARY  dictionary,
                                   const slxKEYW_LINE   keyword,
                                   slxDIC_HANDLE        **dicInfo,
                                   ccsERROR             *error)
{ 
/* 
   MNY: Static variable is initialzed to ccsFALSE
   Updated due to problem with APR2004 release, VLTSW20040034
 */
    static vltLOGICAL      initialized=ccsFALSE;
    static slxDICTIONARY   dicName;
    static miscHASH_TABLE  hashTable;
    
    if (!initialized)
	{
	miscHashInit(hashTable);
	*dicName = '\0';
	initialized = TRUE;
	}
    
    if (*dictionary != '\0')
	{ 
	if (strcmp(dicName, dictionary) != 0)
	    {
	    if (slxLoadDicHash(hashTable, dictionary, error) == FAILURE)
		return FAILURE;
	    slxCOPY_MAX_N_BYTES(dicName, dictionary, sizeof(slxDICTIONARY), 
				error);
	    }
	}

    if (slxLookUpKeywordHashX(hashTable, keyword, dicInfo, error) == FAILURE)
	return FAILURE;

    return SUCCESS;
}


/*
 *****************************************************************************
 */


/*___oOo___*/

