/******************************************************************************
* E.S.O. - VLT project
*
* slxLookUpKeywordHashX.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  05/09/95  created
* epozna    07/12/00  VLTSW20000517- slxLoadDicHashList (lost error logs)
* epozna    31/11/01  VLTSW20010618- indexed keywords (tmp)
* epozna    31/11/01  VLTSW20010618- indexed keywords update
*                                     (orig workarounds removed)  
* - - - - - - - - - - APR2004 - - - - - - - - - - - - - - - - - - - 
* mnylund   02/02/04  Problems with APR2004, static variable initialized
*                     is initialized to ccsFALSE, VLTSW20040034
*/

/************************************************************************
*   NAME
*   slxLookUpKeywordHashX, slxLoadDicHash, slxInstallDicBuf,
*   slxGetDicRevision, slxGetDicVersion - load a Dictionary into a 
*   Hash Tablelook/look up a keyword in an SLX Hash Table
* 
*   SYNOPSIS
*   #include "slx.h"
* 
*   ccsCOMPL_STAT slxInstallDicBuf(slxDYN_BUF          *dicBuf,
*	 		           miscHASH_TABLE      dicHashTable,
*                                  slxDIC_HANDLE       *dicList,
*			           ccsERROR            *error)
*
*   ccsCOMPL_STAT slxLoadDicHash(miscHASH_TABLE        dicHashTable,
*                                const slxDICTIONARY   dictionary,
*                                ccsERROR              *error)
*
*   ccsCOMPL_STAT slxLoadDicHashList(miscHASH_TABLE        dicHashTable,
*                                    const char            *dictionary,
*                                    slxDIC_HANDLE         *dicList,
*                                    ccsERROR              *error)
*
*   ccsCOMPL_STAT slxLookUpKeywordHashX(const miscHASH_TABLE  dicHashTable,
*                                       const slxKEYW_LINE    keyword,
*                                       slxDIC_HANDLE         **dicInfo,
*                                       ccsERROR              *error)
*
*
*   ccsCOMPL_STAT slxGetDicRevision(const slxDICTIONARY     dictionary,
*                                   const vltINT32          entryNo,
*                                   slxDIC_RECORD_ENTRY     revision,
*                                   ccsERROR                *error)
*
*   ccsCOMPL_STAT slxGetDicVersion(const slxDICTIONARY     dictionary,
*                                  slxDIC_RECORD_ENTRY     version,
*                                  ccsERROR                *error)
*
*   ccsCOMPL_STAT slxConvMethNo2Id(const slxUNIT_CONVERSIONS number,
*			           slxKEYW_LINE              id,
* 			           ccsERROR                  error)
*
*   DESCRIPTION
*   These functions makes it possible to use several Dictionaries at the
*   same time.
*
*   slxLoadDicHash() is used to load a Data Interface Dictionary into an
*   SLX Hash Table. It is assumed that no Hash Table is already loaded into
*   the Hash Table. If this is the case the function slxHashDelete() must be
*   called first. 
*
*   The name of the Dictionary/ies to load must follow the rules:
*
*   o dictionary = "ALL|all|All":
*   All Dictionaries found in the "$INS_ROOT/SYSTEM/Dictionaries" are loaded 
*   and merged together. Only names of the structure 
*   "<dictionary ID>-DIC.<name>" are taken.
*
*   o dictionary = "EMMI_OS":
*   The Dictionary is loaded as 
*   "$INS_ROOT/SYSTEM/Dictionary/ESO-VLT-DIC.EMMI_OS".
*
*   o dictionary = "<path>/ALL":
*   E.g. "~/vlt/myDictionaries/all". All Dictionaries found in the 
*   "<path>" directory are loaded.
*
*   o dictionary = "ESO-NTT-DIC.SOFI_DCS":
*   The Dictionary will be loaded as 
*   "$INS_ROOT/SYSTEM/Dictionary/ESO-NTT-DIC.SOFI_DCS".
*
*   o dictionary = "<path>/ESO-VLT-DIC.ISAAC_ICS":
*   The Dictionary is loaded as "<path>/ESO-VLT-DIC.ISAAC_ICS".
*   
*
*   It is possibly to specify to load several Dictionries in one go by 
*   given a list of Dictionaries as Dictionary, each element separated by
*   "|":
*  
*   "<name1>|<name2>|<name3>"
* 
*   The names in the list must follow the naming convention given above.
*
*   If a Dictionary was not found in the "$INS_ROOT/SYSTEM/Dictionary"
*   it will also be tried in "$INTROOT/config" or "$VLTROOT/config".
*
*   A basic Dictionary "ESO-VLT-DIC.PAF" must be available in the 
*   "$INS_ROOT/SYSTEM/Dictionary", "$INTROOT/include" or "$VLTROOT/include"
*   directories, or directory given with the Dictionary to load.
*
*   If it is tried to load several Dictionaries into one miscHASH_TABLE without
*   calling the function slxFreeDictionary() on the table, the Dictionaries
*   are merged together. I.e., if the a keyword appears several times, the 
*   last occurrence is the one stored in the Dictionary.
*
*   The slxLookUpKeywordHashX() function is used to perform a fast look-up of 
*   a keyword in the SLX Hash Table.
*
*   slxInstallDicBuf() is used to install a Dictionary in an SLX Hash Table
*   from an SLX Dynamic Buffer.
*  
*   The function slxGetDicRevision() returns the requested "Revision:" entry
*   from the Dictionary Header. If 0 (zero) is given, the last "Revision:" 
*   entry found in the Dictionary Header is returned in the "revision"
*   parameter. The first "Revision:" entry has number 1.
*
*
*   dictionary   IN      Name of the dictionary to load.
*
*   dicHashTable IN      The SLX Hash Table containing the Dictionary. The 
*                        Dictionary must be loaded by slxLoadDicHash().
*
*   keyword      IN      Name of the keyword to look up; must be in the 
*                        Short-FITS format.
*
*   dicInfo      OUT     Pointer-pointer to SLX Dictionary Structure that 
*                        contains the information of the Dictionary Record 
*                        searched. The element points to an element in the SLX
*                        Hash Table (don't modify it).
*
*   dicList      IN/OUT  Pointer to header of list of SLX Dictionary Handle
*                        structure. This list must have a header element.
*                        When calling the function the first time,
*                        it must be ensured that the element is properly
*                        initialized (dicHandle->next = dicHandle,
*                        dicHandle->prev = dicHandle).
*
*                        The keywords in the SLX Dictionary Handle List
*                        will be sorted alphabetically (order: decreasing).
*
*                        If it is not desirable to have the list, NULL
*                        can be given in for "dicList".
*
*   error        OUT     CCS error structure indicating a possible error 
*                        occurring.
*
*
*   FILES
*   Dictionaries   The Dictionaries used must have the format as defined for
*                  VLT Data Interface Dictionaries.
*
*   ENVIRONMENT
*   INS_ROOT       Defines the root directory of the Instrument Directory 
*                  structure.
*
*   RETURN VALUES 
*   SUCCESS   In case the operation was successfull.
*   FAILURE   In case the operation failed. If the keyword was not found
*             the error->errorNumber will be slxERR_KEYWORD_NOT_FOUND.
*
*
*   CAUTIONS 
*   It is the reponsability of the calling application to make a call to 
*   miscHashInit(3) before calling slxLoadDicHash(3).
*
*   EXAMPLES
*
*   SEE ALSO
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxLookUpKeywordHashX.c,v 1.145 2005/07/27 09:22:24 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

/* Local Headers
 */
#include "slx.h"
#ifndef NOCCS
#include "miscErrors.h"
#else
#include "miscErrorsNoCcs.h"
#endif

ccsCOMPL_STAT slxInstallDicBuf(slxDYN_BUF          *dicBuf,
			       miscHASH_TABLE      dicHashTable,
			       slxDIC_HANDLE       *dicList,
			       ccsERROR            *error)
{
    static ccsERROR   tmpError;
/* 
   MNY: Static variable is initialzed to ccsFALSE
   Updated due to problem with APR2004 release, VLTSW20040034
 */
    static vltLOGICAL initialized=ccsFALSE;
    slxDIC_HANDLE     *dicInfoTmp, *dicInfoTmp2, *dicInfoNew, *insertPt;
    vltLOGICAL        gotRecord, sort;
    ccsSTACK_ELEM     *errDescription;
    vltLOGICAL        errFound;


    if (!initialized)
	{
	errResetStack(&tmpError);
	initialized = TRUE;	
	}

    if (slxGetDicRecSeq(TRUE, &gotRecord, dicBuf, NULL, error) == FAILURE)
	{
	errAdd(error, slxMODULE_NAME, slxERR_READ_DIC, __FILE__, NULL, 
	       "From SLX Dynamic Buffer");
	free(dicInfoTmp);
	return FAILURE;
	}
    sort = FALSE;
    do
	{  
	if ((dicInfoTmp = malloc(sizeof(slxDIC_HANDLE))) == NULL)
	    {
	    errAdd(error, slxMODULE_NAME, slxERR_ALLOC_MEM, __FILE__,NULL);
	    return FAILURE;
	    }
	memset(dicInfoTmp, 0, sizeof(slxDIC_HANDLE));
	if (slxGetDicRecSeq(FALSE, &gotRecord, dicBuf, dicInfoTmp, 
			    error) == FAILURE)
	    {
	    slxResetDicRecord(dicInfoTmp, error);
	    free(dicInfoTmp);
	    return FAILURE;
	    }
	if (gotRecord)
	    {
	    dicInfoNew = insertPt = NULL;
	    
	    /* Check if the keyword is already loaded (used when merging).
	     * If loaded, delete it first.
	     */
	    /* VLTSW20010618: change  miscHashGetEl to new
	     * so that always the exact match is searchd here as originally.
	     */
	    if (miscHashGetElExact(dicHashTable, dicInfoTmp->parName, 
	       (void *)&dicInfoTmp2, &tmpError) == FAILURE)
		{
		if (errIsReason(&tmpError, "misc", miscERR_HASH_KEY_NOT_FOUND,
				&errFound, &errDescription) == FAILURE)
		    {
		    errMergeStack(error, &tmpError, errBOTTOM);
		    errResetStack(&tmpError);
		    return FAILURE;
		    }
		if (errFound)
		    {
		    errResetStack(&tmpError);
		    if (dicList != NULL)
			{
			sort = TRUE;
			dicInfoNew = dicInfoTmp;
			}
		    }
		else
		    {
		    errMergeStack(error, &tmpError, errBOTTOM);
		    errResetStack(&tmpError);
		    return FAILURE;
		    }
		}
	    else
		{
		/* Delete the element already stored
		 */
		if (dicList != NULL)
		    {
		    /* Take the element out of the list
		     */
		    dicInfoNew = dicInfoTmp;
		    insertPt = dicInfoTmp2->prev;
		    dicInfoTmp2->next->prev = dicInfoTmp2->prev;
		    dicInfoTmp2->prev->next = dicInfoTmp2->next;
		    }
		if (slxResetDicRecord(dicInfoTmp2, error) == FAILURE)
		    return FAILURE;
		
		if (miscHashDeleteEl(dicHashTable, dicInfoTmp->parName, 
				     error) == FAILURE)
		    return FAILURE;
		}
	    if (miscHashAddElement(dicHashTable, dicInfoTmp->parName, 
				   (void *)&dicInfoTmp, error) == FAILURE)
		return FAILURE;
	    if (dicList != NULL)
		{
		if (insertPt != NULL)
		    {
		    /* The element already existed so we know the insertion
		     * point
		     */
		   
		    dicInfoNew->next = insertPt->next;
		    dicInfoNew->prev = insertPt;
/*QQ*/
		    insertPt->next->prev = dicInfoNew;
		    insertPt->next = dicInfoNew;
  
		    }
		else
		    {
		    /* The element is new so we insert it last in the list
		     */  
		   
		    dicInfoNew->next = dicList;
		    dicInfoNew->prev = dicList->prev;
		    dicList->prev->next = dicInfoNew;
		    dicList->prev = dicInfoNew;
		    
		    }
		}
	    }
	else
	    free(dicInfoTmp);
	}
    while (gotRecord);

    /* Sort the Dictionary Handle List if required
     */
    if ((dicList != NULL) && sort)
	slxSortDicList(dicList);

    return SUCCESS;
}

void slxSortDicList(slxDIC_HANDLE *dicList)
{
   
    slxDIC_HANDLE  *dicInfoTmp;
    vltINT32       count, count2, i, j;

    /*VLTSW20010618 added caution check
     */
    if(dicList==NULL)
	return;

    /* IMPL.: Should use Quick Sort - for now Bubble Sort
     */
    dicInfoTmp = dicList->next;
    count = 0;
    while (dicInfoTmp != dicList)
	{
	/*printf("sorting:%d %s\n",count,dicInfoTmp->parName);*/
	count++;
	if((dicInfoTmp->next)==NULL)
	    {
	    /*printf("ERROR:THE next info is NULL\n");*/
	    }
	dicInfoTmp = dicInfoTmp->next;
	}
    count2 = count;
    for (i = 0; i < count; i++)
	{
	
	dicInfoTmp = dicList->next;
	for (j = 0; j < (count2 - 1); j++)
	    {
	    if (strcmp(dicInfoTmp->parName, dicInfoTmp->next->parName) > 0)
		{
		dicInfoTmp->next->next->prev = dicInfoTmp;
		dicInfoTmp->next->prev = dicInfoTmp->prev;
		dicInfoTmp->prev = dicInfoTmp->next;
		dicInfoTmp->next = dicInfoTmp->next->next;
		dicInfoTmp->prev->next = dicInfoTmp;
		dicInfoTmp->prev->prev->next = dicInfoTmp->prev;
		}
	    else
		dicInfoTmp = dicInfoTmp->next;
	    }
	count2--;
	}
    
    /* Number the elements
     */
    dicInfoTmp = dicList->next;
    count = 0;
    while (dicInfoTmp != dicList)
	{
	count++;
	dicInfoTmp->no = count;
	dicInfoTmp = dicInfoTmp->next;
	}	
}

/* Function to load a Dictionary into a Hash Table
 */
ccsCOMPL_STAT slxLoadDicHash(miscHASH_TABLE        dicHashTable,
			     const char            *dictionary,
			     ccsERROR              *error)
{
    if (slxLoadDicHashList(dicHashTable, dictionary, NULL, error) == FAILURE)
	return FAILURE;

    return SUCCESS;
}

/* Function to load a Dictionary into a Hash Table and maintains at the
 * same time a sorted list of slxDIC_HANDLE element
 */
ccsCOMPL_STAT slxLoadDicHashList(miscHASH_TABLE        dicHashTable,
				 const char            *dictionary,
				 slxDIC_HANDLE         *dicList,
				 ccsERROR              *error)
{
/* 
   MNY: Static variable is initialzed to ccsFALSE
   Updated due to problem with APR2004 release, VLTSW20040034
 */
    static vltLOGICAL       initialized=ccsFALSE;
    static ccsERROR         tmpError;
    static slxSTRING_LIST   *dicNameList;
    char                    *chrPtr, *dicNameListPtr;  
    slxDIC_HANDLE           *testDicInfo;
    slxDYN_BUF              dicBuf;
    slxFILENAME             dicPath, dicPrefix, dicName;
    slxFILENAME             dicFileName;
    slxFILENAME             curDic;
    slxFILENAME             tmpBuf;
    slxFILENAME             nextDic;
    slxFILENAME             dicNameTmp;
    slxKEYW_LINE            dicNameSubEls[16]; 
    slxSTRING_LIST          *dicPtr, *dicPtrDel;
    vltINT32                i, noDicsToLoad;
    vltINT32                noOfSubEls;
    vltINT32                idx, dicEl;
    vltLOGICAL              pafLoaded;


    if (!initialized)
	{
	if (slxInitStrList(&dicNameList, "HEADER", error) == FAILURE)
	    return FAILURE;
	errResetStack(&tmpError);
	initialized = TRUE;
	}
    if (slxInitDynBuf(&dicBuf, error) == FAILURE)
	return FAILURE;

    /* Check if the PAF dictionary is already loaded by probing for PAF.TYPE.
     *
     * EPO VLTSW20000517 (use local error stack)
     */
    if (slxLookUpKeywordHashX(dicHashTable, "PAF.TYPE", &testDicInfo,
			      &tmpError) == FAILURE)
	{
	errResetStack(&tmpError);
	pafLoaded = FALSE;
	}
    else
	pafLoaded = TRUE;
     
    dicNameListPtr = (char *)dictionary;
    if (slxGetNextField2('|', dicNameListPtr, sizeof(slxFILENAME),
			 nextDic, &dicNameListPtr, error) == FAILURE)
	return FAILURE;
    while (*nextDic != '\0')
	{
	/* Split up the name of the Dictionary
	 */ 
	if (slxGetPath(nextDic, dicPath, error) == FAILURE)
	    return FAILURE;
	if (slxGetFileName(nextDic, dicNameTmp, error) == FAILURE)
	    return FAILURE;
	if (slxSplit(".", dicNameTmp, 16, dicNameSubEls, &noOfSubEls,
		     error) == FAILURE)
	    return FAILURE;

	/* Assume that if the first element contains the string "-DIC", it
	 * is the Dictionary prefix; the rest is then the name
	 */
	idx = 0;
	if (strstr(dicNameSubEls[idx], "-DIC") != NULL)
	    strcpy(dicPrefix, dicNameSubEls[idx++]);
	else
	    *dicPrefix = '\0';
	*dicName = '\0';
	for (dicEl = idx; dicEl < noOfSubEls; dicEl++)
	    {
	    if (*dicName != '\0')
		strcat(dicName, ".");
	    strcat(dicName, dicNameSubEls[dicEl]);
	    }

	/* Check if Dictionary name is given as all|ALL|All
	 */
	strcpy(tmpBuf, dicName);
	slxStrToUpper(tmpBuf);
	if (strcmp(tmpBuf, "ALL") == 0)
	  {
	  slxCOPY_MAX_N_BYTES(dicName, "ALL", sizeof(slxFILENAME), error);
	  }
	  
	/* Find out how many Dictionaries to load
	 */
	noDicsToLoad = 0;
	if (strcmp(dicName, "ALL") != 0)
	    {
	    /* Load only PAF Dictionary and Dictionary specified as input 
	     * parameter
	     */
	    if (slxStrListMerge("PAF", dicNameList, &noDicsToLoad, 
				error) == FAILURE)
		return FAILURE;
	    if (slxStrListMerge((char *)dicName, dicNameList, &noDicsToLoad, 
				error) == FAILURE)
		return FAILURE;
	    }
	else
	    {
	    /* Generate a list of Dictionaries to load
	     */
	    if (*dicPath != '\0')
		chrPtr = dicPath;
	    else	    
		{
		slxCOPY_MAX_N_BYTES(tmpBuf, "$INS_ROOT/SYSTEM/Dictionary",
				    sizeof(slxFILENAME), error);
		if (slxResolvePath(tmpBuf, &chrPtr, error) == FAILURE)
		    {
		    return FAILURE;
		    }
		}	
	    if (slxGetDirInfo(chrPtr, dicNameList, &noDicsToLoad, 
			      error) == FAILURE)
		{
		/*printf("slxGetDirInfo failed\n");*/
		return FAILURE;
		}
	    
	    /* Check that all Dictionary names are correct 
	     * (contains "-DIC.")
	     * - remove the ones that doesn't comply
	     */
	    dicPtr = dicNameList->next;
	    while (dicPtr != dicNameList)
		{
		/*printf("loading dictionary .. %s \n",dicPtr->string);*/
		if ((strstr(dicPtr->string, "-DIC.") == NULL) &&
		    (strcmp(dicPtr->string, "PAF") != 0))
		    {
		    dicPtrDel = dicPtr;
		    dicPtr = dicPtr->next;
		    if (slxDelStrEl(dicPtrDel, error) == FAILURE)
			return FAILURE; 
		    noDicsToLoad--;
		    }
		else
		    dicPtr = dicPtr->next;
		}
	    
	    if (slxStrListMerge("PAF", dicNameList, &noDicsToLoad, 
				error) == FAILURE)
		return FAILURE;
	    }
	
	/* Load the Dictionaries
	 */
	dicPtr = dicNameList->next;
	for (i = 0; i < noDicsToLoad; i++)
	    {
	    if (slxResetDynBuf(&dicBuf, error) == FAILURE)
		return FAILURE;
	    slxCOPY_MAX_N_BYTES(curDic, dicPtr->string, sizeof(slxFILENAME),
				error);
	    if ((!pafLoaded && (strcmp(curDic, "PAF") == 0)) || 
		((strcmp(curDic, "PAF") != 0)))
		{
		if ((*dicPath == '/') && (*dicPrefix == '\0') && 
		    (strcmp(curDic, "PAF") != 0))
		    sprintf(dicFileName, "%s%s", dicPath, curDic);
		else if ((*dicPath == '/') && (*dicPrefix != '\0') && 
			 (strcmp(curDic, "PAF") != 0))
		    sprintf(dicFileName, "%s%s.%s", dicPath, dicPrefix,curDic);
		else 
		    {	
		    if (slxSearchDictionary((char *)curDic, dicFileName,
					    error) == FAILURE)
			{
			/* If the Dictionary is the PAF and it couldn't be
			 * found neither in $INS_ROOT/SYSTEM/Dictionary,
			 * $INTROOT/include|config or $VLTROOT/include|config,
			 * we try to find it in the path given (if given)
			 */
			if ((strcmp(curDic, "PAF") != 0) && (*dicPath != '\0'))
			    {
			    sprintf(dicFileName, "%s%s.%s", 
				    dicPath, slxDICO_PREFIX, curDic);
			    }
			else
			    return FAILURE;	    
			}
		    }
		if (slxLoadFile(dicFileName, "", &dicBuf,&tmpError) == FAILURE)
		    {
		    /* Last chance -- let's see if we can find it in one 
		     * of the paths of the INS_SETUPPATH - if it wasn't 
		     * given in with the full path.
		     */
		    if (*curDic != '/')
			{
			sprintf(dicFileName, "%s%s", slxDICO_PREFIX, curDic);
			if (slxLoadFile(dicFileName, slxINS_SETUPPATH_NAME,
					&dicBuf, &tmpError) == FAILURE)	    
			    {
			    errMergeStack(error, &tmpError, errBOTTOM);
			    errResetStack(&tmpError);
			    return FAILURE;
			    }
			else
			    errResetStack(&tmpError);
			}
		    else
			{
			errMergeStack(error, &tmpError, errBOTTOM);
			errResetStack(&tmpError);
			return FAILURE;
			}
		    }
		if (slxDynBufAddEOS(&dicBuf, error) == FAILURE)
		    return FAILURE;
		if (slxInstallDicBuf(&dicBuf, dicHashTable, dicList, 
				     error) == FAILURE)
		    {
		    errAdd(error, slxMODULE_NAME, slxERR_READ_DIC, __FILE__, 
			   NULL, dicFileName);
		    goto errorCond;
		    }
		if (!pafLoaded && (strcmp(curDic, "PAF") == 0))
		    pafLoaded = TRUE;
		}
	    dicPtr = dicPtr->next;	
	   
	    }
	if (slxGetNextField2('|', dicNameListPtr, sizeof(slxFILENAME),
			     nextDic, &dicNameListPtr, error) == FAILURE)
	    return FAILURE;
	}

    slxResetDynBuf(&dicBuf, error);
           
    return SUCCESS;

errorCond:
    slxResetDynBuf(&dicBuf, error);
    return FAILURE;
}


/* Function to look up a keyword in a Dictionary in the Hash Table
 */
#define _slxMAX_KEY_ELS 6
ccsCOMPL_STAT slxLookUpKeywordHashX(const miscHASH_TABLE  dicHashTable,
                                    const slxKEYW_LINE    keyword,
                                    slxDIC_HANDLE         **dicInfo,
                                    ccsERROR              *error)
{
    static ccsERROR   tmpError;
/* 
   MNY: Static variable is initialzed to ccsFALSE
   Updated due to problem with APR2004 release, VLTSW20040034
 */
    static vltLOGICAL initialized=ccsFALSE;
    slxKEYW_LINE      keyword2, lookUpKey;
    slxKEYW_LINE      keyElementsOrg[_slxMAX_KEY_ELS];
    slxKEYW_LINE      keyElements[_slxMAX_KEY_ELS];
    slxVALUE          value;
    slxCOMMENT        comment;
    vltINT32          i, j, n, m, l, noOfKeyEls;
    vltLOGICAL        found;
    ccsSTACK_ELEM     *errDescription;
    vltLOGICAL        errFound;


    if (!initialized)
	{
	errResetStack(&tmpError);
	initialized = TRUE;
	}
    memset(lookUpKey, 0, sizeof(slxKEYW_LINE));

    /* Look up the keyword.
     * JKN: If the keyword is not found as it is, 
     *      it is tested if there is an index, if yes this is replaced with "i"     
     * EPO: VLTSW20010618: this workaround is now removed (misc 1.27+)
     *      when dictionary (ie special flag) is used
     */
    if (miscHashGetEl(dicHashTable, keyword, (void **)dicInfo, 
		      &tmpError) == FAILURE)
	{
	if (errIsReason(&tmpError, "misc", miscERR_HASH_KEY_NOT_FOUND,
			&errFound, &errDescription) == FAILURE)
	    {
	    errMergeStack(error, &tmpError, errBOTTOM);
	    errResetStack(&tmpError);
	    return FAILURE;
	    }
	if (!errFound)
	    {
	    errMergeStack(error, &tmpError, errBOTTOM);
	    errResetStack(&tmpError);
	    return FAILURE;
	    }
	else
	    {
	    errResetStack(&tmpError);
	    
	    /*VLTSW20010618:  WORK AROUND REMOVED for DICTIONARIES
	     *                keywords should be found at the first time
	     */
	    if(miscIsDictionary(dicHashTable)==ccsTRUE)
		{
		goto notFound;
		}
	    memset(keyword2, 0, sizeof(slxKEYW_LINE));
	    if (slxProbeReplaceIndeces(keyword, keyword2, error)== FAILURE)
		return FAILURE;
	    if (miscHashGetElExact(dicHashTable, keyword2, 
			      (void **)dicInfo, &tmpError) == FAILURE)
		{
		errResetStack(&tmpError);

		/* Have to try to add index "i" to all keyword components
		 * not yet having this since it must be possible to look up 
		 * "DET.WIN.STARTX" also when the keyword is defined as 
		 * "DETi.WIN.STARTX" in the Dictionary.
		 * Note, in this way not all maximum 16 combinations are
		 * tried out.
		 */
		if (slxSplitShortFitsDyn(keyword2,_slxMAX_KEY_ELS, keyElements,
					 &noOfKeyEls, value,
					 comment, error) == FAILURE)
		    return FAILURE;
		found = FALSE;
		for (i = 0; (i < noOfKeyEls) && !found ; i++)
		    {
		    if (slxGetLastChar(keyElements[i]) != 'i')
			{
			strcat(keyElements[i], "i");
			strcpy(lookUpKey, keyElements[0]);
			for (j = 1; j < noOfKeyEls; j++)
			    {
			    strcat(lookUpKey, ".");
			    strcat(lookUpKey, keyElements[j]);
			    }			
			if (miscHashGetElExact(dicHashTable, lookUpKey,
					  (void **)dicInfo, 
					  &tmpError) == SUCCESS)
			    found = TRUE;
			else
			    errResetStack(&tmpError);
			
			/* Remove the "i" again
			 */
			if (slxGetLastChar(keyElements[i]) == 'i')
			    {
			    *(keyElements[i] + 
			      (strlen(keyElements[i]) - 1)) = '\0';
			    }
			}
		    }    /* end-for (i = 0; (i < noOfKeyEls) && !found ... */

		/* Last try: this is a 'dirty' trick to cover up for a 
		 * problem reported in SPR-970251, where a keyword is
		 * defined as "DETi WINi ASUIT1" in the Dictionary and
		 * looked up as "DET.WIN1.ASUIT1".
		 *
		 * A general algorithm should be developed covering all
		 * possible combinations.
		 */
		if (slxGetLastChar(keyElements[0]) != 'i')
		    {
		    if (slxSplitShortFitsDyn(keyword, _slxMAX_KEY_ELS, 
					     keyElementsOrg,
					     &noOfKeyEls, value,
					     comment, error) == FAILURE)
			return FAILURE;

		    strcat(keyElements[0], "i");
		    
		    /* The other keys we test as they are or with a possible
		     * index converted to "i"
		     */
		    for (n = 1; (n < noOfKeyEls && !found); n++)
			{
			for (m = 0; (m < 2 && !found); m++)
			    {
			    strcpy(lookUpKey, keyElements[0]);
			    for (l = 1; l < n; l++)
				{
				strcat(lookUpKey, ".");
				strcat(lookUpKey, keyElements[l]);
				}
			    strcat(lookUpKey, ".");
			    if (m == 0)
				strcat(lookUpKey, keyElementsOrg[n]);
			    else
				strcat(lookUpKey, keyElements[n]);
			    for (l = (n + 1); l < noOfKeyEls; l++)
				{
				strcat(lookUpKey, ".");
				strcat(lookUpKey, keyElements[l]);
				}
			    if (miscHashGetElExact(dicHashTable, lookUpKey,
					      (void **)dicInfo, 
					      &tmpError) == SUCCESS)
				found = TRUE;
			    else
				errResetStack(&tmpError);
			    }    /* end-for (m = 0; (m < 2 && !found); m++) */
			}    /* end-for (n = 1; (i < noOfKeyEls && ... */
		    *(keyElements[0] + (strlen(keyElements[0]) - 1)) = '\0';
		    }    /* end-if (slxGetLastChar(keyElements[0]) != 'i') */

		if (!found)
		    {
		    errResetStack(&tmpError);
		    goto notFound;
		    }
		}
	    }
	}

    strcpy((*dicInfo)->searchKeyword, keyword);
    return SUCCESS;
    
notFound:
    errAdd(error, slxMODULE_NAME, slxERR_KEYWORD_NOT_FOUND, __FILE__, NULL, 
	   keyword, "Dictionary loaded into Hash Table");
    return FAILURE;
}

ccsCOMPL_STAT slxConvMethNo2Id(const slxUNIT_CONVERSIONS number,
			       slxKEYW_LINE              id,
			       ccsERROR                  *error)
{

    switch (number)
	{
	case slxCONV_SECS_TO_TIME:
	    strcpy(id, slxCONV_SECS_TO_TIME_ID);
	    break;
	case slxCONV_SECS_TO_DAYTIM:
	    strcpy(id, slxCONV_SECS_TO_DAYTIM_ID);
	    break;
	case slxCONV_MJD_TO_DAYTIM:
	    strcpy(id, slxCONV_MJD_TO_DAYTIM_ID);
	    break;
	case slxCONV_DEG_TO_DEGREE:
	    strcpy(id, slxCONV_DEG_TO_DEGREE_ID);
	    break;
	case slxCONV_DEG_TO_HOURANG:
	    strcpy(id, slxCONV_DEG_TO_HOURANG_ID);
	    break;
	case slxCONV_RAD_TO_DEGREE:
	    strcpy(id, slxCONV_RAD_TO_DEGREE_ID);
	    break;
	case slxCONV_RAD_TO_HOURANG:
	    strcpy(id, slxCONV_RAD_TO_HOURANG_ID);
	    break;
	case slxCONV_ARCM_TO_DEGREE:
	    strcpy(id, slxCONV_ARCM_TO_DEGREE_ID);
	    break;
	case slxCONV_ARCM_TO_HOURANG:
	    strcpy(id, slxCONV_ARCM_TO_HOURANG_ID);
	    break;
	case slxCONV_KELVIN_TO_CELSIUS:
	    strcpy(id, slxCONV_KELVIN_TO_CELSIUS_ID);
	    break;
	default:
	    /* slxCONV_NONE */
	    strcpy(id, slxCONV_INACTIVE_ID);
	    break;
	}

    return SUCCESS;
}

ccsCOMPL_STAT slxGetDicRevision(const slxDICTIONARY   dictionary,
				const vltINT32        entryNo,
				slxDIC_RECORD_ENTRY   revision,
				ccsERROR              *error)
{
    char                 *bufPtr;
    slxDYN_BUF           dicBuf;
    slxDICTIONARY        dicFileName;
    slxDIC_RECORD_ENTRY  nextEntry;
    slxFILENAME          tmpBuf;
    vltINT32             entryCount;
    vltLOGICAL           run;

    if (slxInitDynBuf(&dicBuf, error) == FAILURE)
	return FAILURE;

    if (slxSearchDictionary((char *)dictionary, dicFileName, error) == FAILURE)
	return FAILURE;    
    if (slxLoadFile(dicFileName, "", &dicBuf, error) == FAILURE)
	return FAILURE;
    *revision = '\0';
    if (slxGetNextLineBuf(&dicBuf, TRUE, nextEntry, error) == FAILURE)
	goto errorCond;
    run = TRUE;
    entryCount = 0;
    while (run)
	{
	if (slxGetNextLineBuf(&dicBuf, FALSE, nextEntry, error) == FAILURE)
	    goto errorCond;
	if (strstr(nextEntry, "Revision:") == nextEntry)
	    {
	    if (slxGetNextField(nextEntry, sizeof(slxKEYW_LINE),
				revision, &bufPtr, error) == FAILURE)
		goto errorCond;
	    while ((*bufPtr == ' ') || (*bufPtr == '\t'))
		bufPtr++;
	    slxCOPY_MAX_N_BYTES(revision, bufPtr, sizeof(slxKEYW_LINE),error);
	    slxTrimString(revision, " \n");
	    entryCount++;
	    }
	else if (strstr(nextEntry, "Parameter Name:") == nextEntry)
	    run = FALSE;

	if ((entryNo != 0) && (entryCount == entryNo))
	    run = FALSE;
	}    
    if ((*revision == '\0') || ((entryNo != 0) && (entryCount < entryNo))) 
	{
	sprintf(tmpBuf, "The requested Revision record entry was not found in "
		"header of Dictionary: \"%s\".", dictionary);
	errAdd(error, slxMODULE_NAME, slxERR_GENERIC_W, __FILE__, NULL,tmpBuf);
	goto errorCond;
	}

    slxResetDynBuf(&dicBuf, error);

    return SUCCESS;

errorCond:
    slxResetDynBuf(&dicBuf, error);
    return FAILURE;
}

ccsCOMPL_STAT slxGetDicVersion(const slxDICTIONARY     dictionary,
			       slxDIC_RECORD_ENTRY     version,
			       ccsERROR                *error)
{
    char                 *bufPtr;
    slxDYN_BUF           dicBuf;
    slxDICTIONARY        dicFileName;
    slxDIC_RECORD_ENTRY  nextEntry;
    slxFILENAME          tmpBuf;
    vltLOGICAL           run;

    if (slxInitDynBuf(&dicBuf, error) == FAILURE)
	return FAILURE;

    if (slxSearchDictionary((char *)dictionary, dicFileName, error) == FAILURE)
	return FAILURE;    
    if (slxLoadFile(dicFileName, "", &dicBuf, error) == FAILURE)
	return FAILURE;
    *version = '\0';
    if (slxGetNextLineBuf(&dicBuf, TRUE, nextEntry, error) == FAILURE)
	goto errorCond;
    run = TRUE;
    while (run)
	{
	if (slxGetNextLineBuf(&dicBuf, FALSE, nextEntry, error) == FAILURE)
	    goto errorCond;
	if (strstr(nextEntry, "Version Control:") == nextEntry)
	    {
	    run = FALSE;
	    if (slxGetNextField(nextEntry, sizeof(slxKEYW_LINE),
				version, &bufPtr, error) == FAILURE)
		goto errorCond;
	    if (slxGetNextField(bufPtr, sizeof(slxKEYW_LINE),
				version, &bufPtr, error) == FAILURE)
		goto errorCond;
	    while ((*bufPtr == ' ') || (*bufPtr == '\t'))
		bufPtr++;
	    slxCOPY_MAX_N_BYTES(version, bufPtr, sizeof(slxKEYW_LINE),error);
	    slxTrimString(version, " \n");
	    }
	else if (strstr(nextEntry, "Parameter Name:") == nextEntry)
	    run = FALSE;
	}    
    if (*version == '\0')
	{
	sprintf(tmpBuf, "The Version Control record entry was not found in "
		"header of Dictionary: \"%s\".", dictionary);
	errAdd(error, slxMODULE_NAME, slxERR_GENERIC_W, __FILE__, NULL,tmpBuf);
	goto errorCond;
	}

    slxResetDynBuf(&dicBuf, error);

    return SUCCESS;

errorCond:
    slxResetDynBuf(&dicBuf, error);
    return FAILURE;
}   



/*
 *****************************************************************************
 */


/*___oOo___*/

