/******************************************************************************
* E.S.O. - VLT project
*
* slxShortToNormalFITS.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  27/07/94  Created 1. version
*/

/************************************************************************
*   NAME
*   slxShortToNormalFITS - converts from Short-FITS to normal FITS
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxShortToNormalFITS(const slxKEYW_LINE  shortFITS,
*				       slxKEYW_LINE        normalFITS,
*                                      ccsERROR            *error)
*
*
*   DESCRIPTION
*   NOTE: A new function slxShortToNormalFits2() is now supported providing 
*   a more convenient interface, and providing two sub-system keywords.
*   This new function should be used.
*
*   The function converts a set-up keyword from the Short-FITS format to the
*   normal FITS format.
* 
*   shortFITS	IN	Pointer to buffer containing the keyword string
*			in the Short-FITS format.
*			
*   normalFITS	IN	Pointer to buffer in which the keyword string in 
*    			normal FITS will be written.
*
*
*
*   RETURN VALUES 
*   SUCCESS     In case the operation was successful.
*
*   FAILURE     In case the operation went wrong. Could be a problem with the
*               format of the keyword line.
*
*               slxERR_FORMAT_KEYWORD  The format of the keyword (line) is 
*                                      wrong.
*
*               slxERR_CREATE_FITS     An error occurred while creating FITS
*                                      keyword - input parameters somehow wrong.
*
*
*               The function makes calls to slxSplitShortFITS() and 
*               slxCreateFITS() and may return errors generated by these.
*
*   SEE ALSO
*   slxSetupToNormalFITS, slxConvertToNormal(), slxNormalToShortFITS(), 
*   slxConvertToShort(), slxDetectFormat()
*
*------------------------------------------------------------------------
*/

#define _POSIX_SOURCE 1
#include "vltPort.h"


static char *rcsId="@(#) $Id: slxShortToNormalFITS.c,v 1.145 2005/07/27 09:29:25 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);


/* System Headers
 */
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

/* Local Headers
 */
#include "slx.h"

/* Function definition
 */
ccsCOMPL_STAT slxShortToNormalFITS(const slxKEYW_LINE   shortFITS,
                                   slxKEYW_LINE         normalFITS,
                                   ccsERROR             *error)

{
    char        category[4];
    char        subSystem[4];
    char        parameter[16];
    char        value[slxMAX_FITS_LI_LEN];
    char        comment[slxMAX_FITS_LI_LEN];
    char        stringValue[slxMAX_FITS_LI_LEN];
    char        *charPtr;
    long int    integerValue = 0;
    slxPAR_TYPE parameterType;
    vltDOUBLE   doubleValue = 0;
    vltLOGICAL  logicalValue = TRUE;
    vltLOGICAL  exponential = FALSE;
    vltUINT8    precision = 0;

   
    if (slxSplitShortFITS(shortFITS, category, subSystem, parameter,
                          value, comment, error) == FAILURE)
        return FAILURE;

    /* Determine parameter type; there are the following possibilities:
     * 1) "T" | "F", 2) "<string>", 3) +/- <integer>, 
     * 4) +/- <double, non-exp.>, 5) +/- <double, exp>
     */
    if ((*value == '\"') && ((*(value + 1) == 'T') || (*(value + 1) == 'F')) 
        && (*(value + 2)== '\"'))
        {
        parameterType = slxLOGICAL;    /* " */
        if (*(value + 1) == 'T')
            logicalValue = TRUE;
        else
            logicalValue = FALSE;
        }
    else if (*value == '\"')
        {
        parameterType = slxSTRING;
        memset((char *)stringValue, 0, (size_t)slxMAX_FITS_LI_LEN);
        strncpy((char *)stringValue, (char *)(value + 1), 
                (size_t)(strlen(value) - 2));
        }
    else if ((charPtr = strchr(value, '.')) == NULL)
        {
        parameterType = slxINTEGER;        
        integerValue = atol(value);
        }
    else
        {
        parameterType = slxDOUBLE; 
        charPtr++;
        while (isdigit((int)*(charPtr + precision)))
            precision++;
        if ((*(charPtr + precision) == 'E') || (*(charPtr + precision) == 'e'))
            exponential = TRUE;
        doubleValue = atof(value);
        }

    if (slxCreateFITS(category, subSystem, parameter, parameterType,
                      stringValue, logicalValue, integerValue, precision,
                      exponential, doubleValue, comment, 
                      normalFITS, error) == FAILURE)
        return FAILURE;

    return SUCCESS;
}


/*
 *****************************************************************************
 */


/*___oOo___*/


