/******************************************************************************
* E.S.O. - VLT project
*
* slxShortToNormalFits2.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  29/03/96  created
*/

/************************************************************************
*   NAME
*   slxShortToNormalFits2 - converts from Short-FITS to normal FITS
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxShortToNormalFits2(const slxKEYW_LINE    shortFits,
*				        slxKEYW_LINE          normalFits,
*                                       const miscHASH_TABLE  dictionary 
*                                       ccsERROR              *error)
*
*
*   DESCRIPTION
*   The function converts a set-up keyword from the Short-FITS format to the
*   normal FITS format.
* 
*   shortFits	IN	Buffer containing the keyword string in the Short-FITS
*                       format.
*			
*   normalFits	IN	Buffer in which the keyword string in normal FITS will
*                       be written.
*
*   dictionary  IN      Dictionary loaded into a MISC Hash Table. Use the
*                       function slxLoadDicHashX() to build up the Hash Table.
*
*
*   RETURN VALUES 
*   SUCCESS     
*   FAILURE     
*
*   SEE ALSO
*
*------------------------------------------------------------------------
*/

#define _POSIX_SOURCE 1
#include "vltPort.h"


static char *rcsId="@(#) $Id: slxShortToNormalFits2.c,v 1.145 2005/07/27 09:30:01 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);


/* System Headers
 */
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

/* Local Headers
 */
#include "slx.h"

/* Function definition
 */
ccsCOMPL_STAT slxShortToNormalFits2(const slxKEYW_LINE    shortFits,
				    slxKEYW_LINE          normalFits,
				    const miscHASH_TABLE  dictionary, 
                                    ccsERROR              *error)
{
    char           *keyword;
    slxDIC_HANDLE  *dicInfo;
    slxCATEGORY    category;
    slxSUBSYSTEM   subSystem1;
    slxSUBSYSTEM   subSystem2;
    slxPARAMETER   parameter;
    slxVALUE       value;
    slxCOMMENT     comment;
    slxFITS_VALUE  val;


    memset(&val, 0, sizeof(slxFITS_VALUE));
    if (slxSplitShortFits2(shortFits, category, subSystem1, subSystem2,
			   parameter, value, comment, error) == FAILURE)
        return FAILURE;

    keyword = slxAssKey(category, 0, subSystem1, 0, subSystem2, 0,
			parameter, 0, NULL);
    if (slxLookUpKeywordHashX(dictionary, keyword, &dicInfo, error) == FAILURE)
	return FAILURE;

    switch (dicInfo->dicParamType)
	{
	case slxSTRING:
	    val.s = value;
	    if (slxCreateFits2(dictionary, val, TRUE, normalFits, error, 
			       category, subSystem1, subSystem2, parameter, 
			       NULL) == FAILURE)
		return FAILURE;
	break;
      case slxLOGICAL:
	  if (strstr(value, "T") != NULL)
	      val.l = TRUE;
	  else
	      val.l = FALSE;
	    if (slxCreateFits2(dictionary, val, TRUE, normalFits, error, 
			       category, subSystem1, subSystem2, parameter, 
			       NULL) == FAILURE)
		return FAILURE;
	    break;
	case slxINTEGER:
	    val.i = atoi(value);
	    if (slxCreateFits2(dictionary, val, TRUE, normalFits, error, 
			       category, subSystem1, subSystem2, parameter, 
			       NULL) == FAILURE)
		return FAILURE;
	    break;
	default:
	    val.d = atof(value);
	    if (slxCreateFits2(dictionary, val, TRUE, normalFits, error, 
			       category, subSystem1, subSystem2, parameter, 
			       NULL) == FAILURE)
		return FAILURE;
	    break;
	}
    
    return SUCCESS;
}


/*
 *****************************************************************************
 */


/*___oOo___*/


