/******************************************************************************
* E.S.O. - VLT project
*
* slxWhichFormat.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  02/09/94  Created 1. version
*/

/************************************************************************
*   NAME
*   slxWhichFormat - detects the format of a set-up file
*
*   SYNOPSIS
*
*   slxWhichFormat <set-up file> [-t]
*
*
*   DESCRIPTION
*   The program locates the set-up file according to the rules specified in
*   "INS Common SW Specification/VLT-SPE-ESO-17240-0385", and writes the 
*   following report to standard output:
*
*   Set-Up File:  <<path>/<set-up file>>
*   Format: <format>
*
*   The <format can be either "Short-FITS", "Normal FITS" or "unknown".
*
*   <set-up file>   Name of the set-up file. Can be given with or without
*                   extension.
*
*   -t              Run the program in test mode.
* 
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxWhichFormat.c,v 1.145 2005/07/27 09:34:34 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

#include <string.h>
/* Local Headers 
 */
#include "slx.h"

/* Main
 */
int main (int argc, char *argv[])
{
    char            completeName[256]; 
    ccsERROR        error;
    ccsPROCNAME     procName;
    slxSETUP_TYPE   type;
    vltUINT8        test = 0;


    if (argc < 2)
        {
        printf("\nCorrect syntax is:\n\n");
        printf("slxWhichFormat <set-up file> [-t]\n\n\n");
        return(1);
        }

    if (argc ==3)
        test = 1;

    strcpy(procName, "slxWhichFormat");
    if (ccsInit(procName, 0, NULL, NULL, &error) == FAILURE)
        {
        fprintf(stderr, "\nError in intializing on RTAP!!\n\n");
        return(1);
        }

    if (slxLocateSetupFile(argv[1], completeName, &error) == FAILURE)
        {
        fprintf(stderr, "\nCould not locate the set-up file %s!\n\n", argv[1]);
        if (test)
            {
            fprintf(stderr, "Failure, error is:\n");
            fprintf(stderr, "   ModuleId   : %s\n", error.moduleId);
/*
#ifndef NOCCS
            fprintf(stderr, "   LocationId : %s\n", error.location);
            fprintf(stderr, "   ErrorNumber: %d\n", error.errorNumber);
#endif
*/
            fprintf(stderr, "   Parameters : %s\n", error.runTimePar);
            }
        ccsExit(&error);
        return(1);
        }

    if (slxDetectFormat(completeName, &type, &error) == FAILURE)
        {
        fprintf(stderr, "\nError occurred detecting the format!!\n\n");
        if (test)
            {
            fprintf(stderr, "Failure, error is:\n");
            fprintf(stderr, "   ModuleId   : %s\n", error.moduleId);
/*
#ifndef NOCCS
            fprintf(stderr, "   LocationId : %s\n", error.location);
            fprintf(stderr, "   ErrorNumber: %d\n", error.errorNumber);
#endif
*/
            fprintf(stderr, "   Parameters : %s\n", error.runTimePar);
            }
        ccsExit(&error);
        return(1);
        }

    if (!test)
        printf("\nSet-Up File: %s", completeName);
    if (type == slxNORMAL_FITS)
        {
        printf("\nFormat: Normal FITS\n\n");
        }
    else if (type == slxSHORT_FITS)
        {
        printf("\nFormat: Short-FITS\n\n");
        }
    else
        printf("\nFormat: unknown\n\n");

    ccsExit(&error);
    return(0);
}







