#!/usr/local/bin/tcl
#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxConvertToNormal.test.tcl,v 1.145 2005/07/27 10:40:45 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# jknudstr  13/10/94  created
#

#************************************************************************
#   NAME
#   slxConvertToNormal.test.prog - test script for slxConvertToNormal
# 
#   SYNOPSIS
#   slxConvertToNormal.test.prog
# 
#   DESCRIPTION
#   The script tests the function slxConvertToNormal.
#
#   The script carries out three tests:
#
#   1. Test of attempt to convert non-existing file.
#
#   2. Test of attempt to convert obscure file.
#
#   3. Attempt to convert rwx protected file
#
#   4. Test of conversion of set-up file from Short-FITS to normal FITS.
#      The file converted contains the most of the cases possible. It may
#      not be syntactically correct.
#
#   5. Test of handling of INS_SETUPPATH not properly set.
#
#
#   The script writes an error report to standard output. 
#
#
#   FILES
#   slxConvertToNormal.test.2.bs      Is an obscure set-up file.
#
#   slxConvertToNormal.test.3.bs      File to be rwx protected.
#
#   slxConvertToNormal.test.4.bs      Is a file in Short-FITS format 
#                                        containing a lot of various 
#                                        combinations.
#
#   slxConvertToNormal.test.4.rf  Is the.rf to compare with
#                                        the result of converting the .5.bs
#                                        file.
#
#
#   CAUTIONS
#   Should be executed from the ~slx/test directory.
#
#------------------------------------------------------------------------
#
source ../test/defs

#set verbose 1
set tf slxConvertToNormal.test
set env(INS_SETUPPATH) [pwd]



# PART 1: attempting to open non-existing set-up file
# ===================================================
test slxConvertToNormalFITS-1 {Non existing file} {
    if {[catch "exec -keepnewline slxSetupToNormalFITS NoNeXiStInGfIlE -v" \
        msg]} {
        set msg
    }
}   "\nCouldn't convert the file - check filename!\n"



# PART 2: format of file is wrong
# ================================
exec cp $tf.2.bs $tf.2.temp
catch "exec chmod ugo=+rw $tf.2.temp"
test slxConvertToNormalFITS-2 {File format wrong} {
    catch "exec -keepnewline slxSetupToNormalFITS $tf.2.temp -v" msg
    set msg
}    "\nCouldn't convert the file - check filename!\n"
exec rm $tf.2.temp



# PART 3: try to open full-protected (rwx) file
# =============================================
exec chmod -rwx $tf.3.bs
test slxConvertToNormalFITS-3 {File rwx protected} {
    if {[catch "exec -keepnewline slxSetupToNormalFITS $tf.3.bs -v" msg]} {
        set msg
    }
}    "\nCouldn't convert the file - check filename!\n"
exec chmod +rw $tf.3.bs



# PART 4: test basic functionality
# ================================
test slxConvertToNormalFITS-4 {Basic functionality test} {
    exec cp $tf.4.bs $tf.4.temp
    exec chmod ugo=+rwx $tf.4.temp
    exec slxSetupToNormalFITS $tf.4.temp -v
    set msg [exec diff ./$tf.4.temp $tf.4.rf]
    exec rm $tf.4.temp
    set msg
} ""




# PART 5: INS_SETUPPATH not properly set
# =============================================
unset env(INS_SETUPPATH)
test slxConvertToNormalFITS-5 {INS_SETUPPATH not properly set} {
    if {[catch "exec -keepnewline slxSetupToNormalFITS xxxx -v" msg]} {
        set msg
    }
}    "\nCouldn't convert the file - check filename!\n"






