
##!/usr/local/bin/tcl
#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxCreateFITS.test.tcl,v 1.145 2005/07/27 10:29:32 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ---------------------------------------------
# eallaert  15/10/94  initial version
#

#**************************************************************************
#   NAME
#   slxCreateFITS.test.prog - test program for slxCreateFITS()
#
#   SYNOPSIS
#   slxCreateFITS.test.prog
#
#   DESCRIPTION
#   This script tests the slxCreateFITS program, as follows:
#	1.1 arg2 (subsystem) is too long
#	1.2 arg4 (parameterType) out of range
#	1.3 arg5 (stringvalue) is too long
#       This test has been cancelled, since it is not checked if the string is 
#       too long.
#	1.4 arg8 (precision) is out of range, double in exp format
#
#	2.1 Normal execution of slxCreateFITS - put a string
#	2.2 Normal execution of slxCreateFITS - put a logical
#	2.3 Normal execution of slxCreateFITS - put an integer
#	2.4 Normal execution of slxCreateFITS - put a double, not-exp
#	2.5 Normal execution of slxCreateFITS - put a double, exponential
#
#   If any of these tests fail, an error report will be sent to stdout
#
#   CAUTIONS
#   To be executed from the ~slx/test directory
#
#-----------------------------------------------------------------------------

# Take over Ousterhout's test procedure 
source ../test/defs

#set VERBOSE 1

# testP is the name of the binary we use for testing our function
set testP ../bin/slxTestCreateFITS

# In case of errors, exec $testP does an exit(1), so from Tcl
# we have to catch this error

# PART 1: argument testing
# ========================

# ---------------------------------------------------------------------------
# Produce error for parameter type out of range
test slxCreateFITS-1.1 {arg2 (subsystem) is too long} {
	set cat cat
	set sub "subsystem containing many subs"
	set par parameter
	set typ 1
	set str stringvalue
	set log 1
	set int	32767
	set pre 255
	set exp	1
	set dbl 1234.56789
	set com comment
	catch "exec $testP $cat $sub $par $typ $str $log $int $pre\
	                   $exp $dbl $com 1" msg
	set msg
}  {slxERR_CREATE_FITS : Could not create FITS keyword -\
format of input parameters wrong}

test slxCreateFITS-1.2 {arg4 (parameterType) out of range} {
	set cat cat
	set sub subsystem
	set par parameter
	set typ 15;		# type should be from 1 - 4
	set str stringvalue
	set log 1
	set int	32767
	set pre 255
	set exp	1
	set dbl 1234.56789
	set com comment
	catch "exec $testP $cat $sub $par $typ $str $log $int $pre\
	                   $exp $dbl $com 1" msg
	set msg
}  {slxERR_CREATE_FITS : Could not create FITS keyword -\
format of input parameters wrong}

if {0} {
test slxCreateFITS-1.4 {arg8 (precision) is out of range, double in exp format} {
	set cat cat
	set sub subsystem
	set par parameter
	set typ 4;		# type 1: string, 2: logical, 3: int, 4: double
	set str stringvalue
	set log -1
	set int	32767
	set pre 127
	set exp	1
	set dbl 1234.56789
	set com comment
puts "exec $testP $cat $sub $par $typ $str $log $int $pre\
	                   $exp $dbl $com 1"
	catch "exec $testP $cat $sub $par $typ $str $log $int $pre\
	                   $exp $dbl $com 1" msg
	set msg
}  {slxERR_CREATE_FITS : Could not create FITS keyword -\
format of input parameters wrong}
}

# PART 2: functionality test
# ==========================
test slxCreateFITS-2.1 {put a string} {
	set cat cat
	set sub subsystem
	set par parameter
	set typ 1;		# type 1: string, 2: logical, 3: int, 4: double
	set str stringvalue
	set log 1
	set int	32767
	set pre 255
	set exp	1
	set dbl 1234.56789
	set com comment
	catch "exec $testP $cat $sub $par $typ $str $log $int $pre\
	                   $exp $dbl $com 1" msg
	set msg
}  {HIERARCH ESO cat subsystem parameter=    'stringvalue' / comment                }

test slxCreateFITS-2.2 {put a logical} {
	set cat cat
	set sub subsystem
	set par parameter
	set typ 2;		# type 1: string, 2: logical, 3: int, 4: double
	set str stringvalue
	set log 1
	set int	32767
	set pre 255
	set exp	1
	set dbl 1234.56789
	set com comment
	catch "exec $testP $cat $sub $par $typ $str $log $int $pre\
	                   $exp $dbl $com 1" msg
	set msg
}  {HIERARCH ESO cat subsystem parameter=               T / comment                 }

test slxCreateFITS-2.3 {put an integer} {
	set cat cat
	set sub subsystem
	set par parameter
	set typ 3;		# type 1: string, 2: logical, 3: int, 4: double
	set str stringvalue
	set log 1
	set int	123456789
	set pre 255
	set exp	1
	set dbl 1234.56789
	set com comment
	catch "exec $testP $cat $sub $par $typ $str $log $int $pre\
	                   $exp $dbl $com 1" msg
	set msg
}  {HIERARCH ESO cat subsystem parameter=       123456789 / comment                 }

test slxCreateFITS-2.4 {put a double, 10 digits precision, not exponential} {
	set cat cat
	set sub subsystem
	set par parameter
	set typ 4;		# type 1: string, 2: logical, 3: int, 4: double
	set str stringvalue
	set log 1
	set int	32767
	set pre 10
	set exp	0
	set dbl 1234.56789
	set com comment
	catch "exec $testP $cat $sub $par $typ $str $log $int $pre\
	                   $exp $dbl $com 1" msg
	set msg
}  {HIERARCH ESO cat subsystem parameter=    1234.5678900000 / comment              }

test slxCreateFITS-2.5 {put a double, 10 digits precision, exponential} {
	set cat cat
	set sub subsystem
	set par parameter
	set typ 4;		# type 1: string, 2: logical, 3: int, 4: double
	set str stringvalue
	set log 1
	set int	32767
	set pre 10
	set exp	1
	set dbl 1234.56789
	set com comment
	catch "exec $testP $cat $sub $par $typ $str $log $int $pre\
	                   $exp $dbl $com 1" msg
	set msg
}  {HIERARCH ESO cat subsystem parameter=    1.2345678900E+03 / comment             }
