
##!/usr/local/bin/tcl
#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxCreateFits2.test.tcl,v 1.145 2005/07/27 10:41:18 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ---------------------------------------------
# jknudstr  09/04/96  created
#

#**************************************************************************
#   NAME
#   slxCreateFits2.test.prog - test program for slxCreateFits2
#
#   SYNOPSIS
#   slxCreateFits2.test.prog
#
#   DESCRIPTION
#
#-----------------------------------------------------------------------------
# Take over Ousterhout's test procedure 
source ../test/defs

# testP is the name of the binary we use for testing our function
set testP         ../bin/slxTestCreateFits2
set dic           GROWING
set env(INS_ROOT) [pwd]/INS_ROOT
set env(INS_USER) SYSTEM

# PART 2: functionality test
# ==========================

# Strings
test slxCreateFITS-2.1.1 {put a short string - only parameter} {
	exec $testP $dic string DATE
} "|DATE    = 'string  '                    / Date this tape written dd/mm/yy       |"

test slxCreateFITS-2.1.2 {put a longer string - only parameter} {
	exec $testP $dic string_string_string DATE
} "|DATE    = 'string_string_string'        / Date this tape written dd/mm/yy       |"

test slxCreateFITS-2.1.3 {put a very long string - only parameter} {
	exec $testP $dic string_string_string_string_string_string_string DATE
} "|DATE    = 'string_string_string_string_string_string_string' / Date this tape wr|"

# Strings - hierarchical FITS
test slxCreateFITS-2.1.4 {put a string - category + parameter} {
	exec $testP $dic string ADA ID
} "|HIERARCH ESO ADA ID          = 'string  '   / Adapter Identification            |"

# Strings - hierarchical FITS
test slxCreateFITS-2.1.4 {put a string - category + subsystem + parameter} {
	exec $testP $dic string ADA LAMP-1 NAME
} "|HIERARCH ESO ADA LAMP-1 NAME = 'string  '   / Common name for lamp i            |"

# Strings - hierarchical FITS
test slxCreateFITS-2.1.4 {put a string - category + 2 * subsystem + parameter} {
	exec $testP $dic string ADA LAMP-1 LAMP NAME
} "|HIERARCH ESO ADA LAMP-1 LAMP NAME= 'string  ' / A test keyword                  |"

##########################################################################
# Logical
test slxCreateFITS-2.2.1 {put a logical - parameter} {
	exec $testP $dic T SIMPLE
} "|SIMPLE  =                    T          / Standard FITS format (rev feb 1985)   |"

test slxCreateFITS-2.2.2 {put a logical - parameter} {
	exec $testP $dic F SIMPLE
} "|SIMPLE  =                    F          / Standard FITS format (rev feb 1985)   |"

test slxCreateFITS-2.2.3 {put a logical - category + parameter} {
	exec $testP $dic F DET ST
} "|HIERARCH ESO DET ST          =            F / Test keyword                      |"

test slxCreateFITS-2.2.3 {put a logical - cat + subsyst + par} {
	exec $testP $dic F DET OUT-10 ST
} "|HIERARCH ESO DET OUT-10 ST   =            F / Output i status (T=enabled)       |"

test slxCreateFITS-2.2.3 {put a logical - cat + 2 * subsyst + par} {
	exec $testP $dic F DET OUT-10 XX ST
} "|HIERARCH ESO DET OUT-10 XX ST=            F / Test keyword                      |"

test slxCreateFITS-2.2.3 {put a logical - cat + 2 * subsyst + par} {
	exec $testP $dic F DET OUT-10 XXXXX ST
} "|HIERARCH ESO DET OUT-10 XXXXX ST=         F / Test keyword                      |"

##########################################################################
# Integer
test slxCreateFITS-2.3.1 {put an integer - par} {
	exec $testP $dic 12345 BITPIX
} "|BITPIX  =                12345          / No of bits storing pix values         |"

test slxCreateFITS-2.3.2 {put an integer - cat + par} {
	exec $testP $dic 123456 ARC CHECKSUM
} "|HIERARCH ESO ARC CHECKSUM    =       123456 / File CRC (32 bit CCITT)           |"

test slxCreateFITS-2.3.3 {put an integer - cat + subsyst + par} {
	exec $testP $dic 1234567 DET FRAM BITS
} "|HIERARCH ESO DET FRAM BITS   =      1234567 / Number of bits/pixel transfered   |"

test slxCreateFITS-2.3.4 {put an integer - cat + 2 * subsyst + par} {
	exec $testP $dic 1234567 DET FRAM XXXX BITS
} "|HIERARCH ESO DET FRAM XXXX BITS=    1234567 / Test keyword                      |"

##########################################################################
# Floating point
test slxCreateFITS-2.4.1 {put a double (!expo) - par} {
	exec $testP $dic 12345.12345 BSCALE
} "|BSCALE  =              12345.1          / Real=Tape*BSCALE+BZERO                |"

test slxCreateFITS-2.4.2 {put a double (!expo) - par} {
	exec $testP $dic 12345.12345 DBL-X
} "|DBL-X   =          1.23451E+04          / Test keyword                          |"

test slxCreateFITS-2.4.3 {put a double (!expo) - par} {
	exec $testP $dic 12345 DBL-X
} "|DBL-X   =          1.23450E+04          / Test keyword                          |"

test slxCreateFITS-2.4.4 {put a double (!expo) - par} {
	exec $testP $dic 12.12 DBL-XX
} "|DBL-XX  =            1.212E+01          / Test keyword                          |"

test slxCreateFITS-2.4.5 {put a double (!expo) - cat + par} {
	exec $testP $dic 12.12 DET DIT
} "|HIERARCH ESO DET DIT         =       12.120 / Defined integration time          |"

test slxCreateFITS-2.4.6 {put a double (!expo) - cat + subsyst + par} {
	exec $testP $dic 12. DET FRAM CDELT1
} "|HIERARCH ESO DET FRAM CDELT1 =         12.0 / Binning factor along X            |"

test slxCreateFITS-2.4.7 {put a double (!expo) - cat + 2 * subsyst + par} {
	exec $testP $dic 12. DET FRAM XXXX CDELT1
} "|HIERARCH ESO DET FRAM XXXX CDELT1=     12.0 / Binning factor along X            |"

# ==================================================================
# Test writing of converted units in the FITS Keyword Comments
# ------------------------------------------------------------------
test slxCreateFITS-2.5.1 {sec->time} {
	exec $testP $dic 12345 CAT1 SUB2 PAR_SB2
} "|HIERARCH ESO CAT1 SUB2 PAR_SB2=       12345 / UTC time is 03:25:45.000          |"

test slxCreateFITS-2.5.2 {sec->daytime} {
	exec $testP $dic 12345 CAT1 SUB2 PAR_SA2
} "|HIERARCH ESO CAT1 SUB2 PAR_SA2=       12345 / Time is \"1970-01-01T03:25:45.000\" |"

test slxCreateFITS-2.5.3 {mjd->daytime} {
	exec $testP $dic 49800.41345000 CAT1 SUB2 PAR_MJD1
} "|HIERARCH ESO CAT1 SUB2 PAR_MJD1= 49800.413450 / aaa \"1995-03-24T09:55:22.080\" bb|"

if {[exec uname] != "Linux"} {
    test slxCreateFITS-2.5.4 {deg->degree} {
	exec $testP $dic -54.345 CAT1 SUB2 PAR_DEGD1
    } "|HIERARCH ESO CAT1 SUB2 PAR_DEGD1= -54.345000 / aaa \"-54:20:42.0\" bbb            |"
    test slxCreateFITS-2.5.5 {deg->hourang} {
	exec $testP $dic 34.345 CAT1 SUB2 PAR_DEGH1
    } "|HIERARCH ESO CAT1 SUB2 PAR_DEGH1= 34.345000 / aaa \"02:17:22.8\" bbb              |"
} 

test slxCreateFITS-2.5.6 {rad->degree} {
	exec $testP $dic 1.234 CAT1 SUB2 PAR_RADD1
} "|HIERARCH ESO CAT1 SUB2 PAR_RADD1=  1.234000 / aaa \"70:42:10.7\" bbb              |"

test slxCreateFITS-2.5.7 {rad->hourang} {
	exec $testP $dic 1.234 CAT1 SUB2 PAR_RADH1
} "|HIERARCH ESO CAT1 SUB2 PAR_RADH1=  1.234000 / aaa \"04:42:48.7\" bbb              |"

test slxCreateFITS-2.5.8 {arcmin->degree} {
	exec $testP $dic 2.345 CAT1 SUB2 PAR_ARCMD1
} "|HIERARCH ESO CAT1 SUB2 PAR_ARCMD1= 2.345000 / aaa \"00:00:14.0\" bbb              |"

test slxCreateFITS-2.5.8 {arcmin->hourang} {
	exec $testP $dic 2.345 CAT1 SUB2 PAR_ARCMH1
} "|HIERARCH ESO CAT1 SUB2 PAR_ARCMH1= 2.345000 / aaa \"00:00:00.9\" bbb              |"

test slxCreateFITS-2.5.9 {kelvin->celsius} {
	exec $testP $dic 54.345 CAT1 SUB2 PAR_KELVIN1
} "|HIERARCH ESO CAT1 SUB2 PAR_KELVIN1=  54.345 / aaa \"-218.805C\" bbb               |"


