#!/usr/local/bin/tcl
#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxCreateShortFITS.test.tcl,v 1.145 2005/07/27 10:53:37 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ---------------------------------------------
# eallaert  15/10/94  initial version
#

#**************************************************************************
#   NAME
#   slxCreateShortFITS.test.prog - test program for slxCreateShortFITS()
#
#   SYNOPSIS
#   slxCreateShortFITS.test.prog
#
#   DESCRIPTION
#   This script tests the slxCreateShortFITS program, as follows:
#	1.1 arg4 (parameterType) out of range
#
#	2.1 Normal execution of slxCreateShortFITS - put a string
#	2.2 Normal execution of slxCreateShortFITS - put a logical
#	2.3 Normal execution of slxCreateShortFITS - put an integer
#	2.4 Normal execution of slxCreateShortFITS - put a double, not-exp
#	2.5 Normal execution of slxCreateShortFITS - put a double, exponential
#
#   To run these tests, the program slxTestCreateShortFITS is used.
#
#   If any of these tests fail, an error report will be sent to stdout
#
#   CAUTIONS
#   To be executed from the ~slx/test directory
#
#-----------------------------------------------------------------------------

# Take over Ousterhout's test procedure 
source ../test/defs

#set VERBOSE 1

# testP is the name of the binary we use for testing our function
set testP ../bin/slxTestCreateShortFITS


# In case of errors, exec $testP does an exit(1), so from Tcl
# we have to catch this error

# PART 1: argument testing
# ========================

# ---------------------------------------------------------------------------
# Produce error return from slxCreateShortFITS
test slxCreateShortFITS-1.1 {arg4 (parameterType) out of range} {
	set cat cat
	set sub subsystem
	set par parameter
	set typ 15;		# type 1: string, 2: logical, 3: int, 4: double
	set str stringvalue
	set log 1
	set int	32767
	set pre 255
	set exp	1
	set dbl 1234.56789
	set com comment
	catch "exec $testP $cat $sub $par $typ $str $log $int $pre\
	                   $exp $dbl $com" msg
	set msg
}  {slxERR_CREATE_FITS : Could not create FITS keyword -\
format of input parameters wrong}

# ---------------------------------------------------------------------------
# If the output of slxCreateShortFITS is OK, one cannot make 
# slxNormalToShortFITS fail

# PART 2: functionality test
# ==========================
# For debugger:
# r cat subsystem parameter 1 stringvalue 1 32767 255 1 1234.56789 comment 1
test slxCreateShortFITS-2.1 {put a string} {
	set cat cat
	set sub subsystem
	set par parameter
	set typ 1;		# type 1: string, 2: logical, 3: int, 4: double
	set str stringvalue
	set log 1
	set int	32767
	set pre 255
	set exp	1
	set dbl 1234.56789
	set com comment
	catch "exec $testP $cat $sub $par $typ $str $log $int $pre\
	                   $exp $dbl $com 1" msg
	set msg
}  {cat.subsystem.parameter    "stringvalue"; # comment                }

test slxCreateShortFITS-2.2 {put a logical} {
	set cat cat
	set sub subsystem
	set par parameter
	set typ 2;		# type 1: string, 2: logical, 3: int, 4: double
	set str stringvalue
	set log 1
	set int	32767
	set pre 255
	set exp	1
	set dbl 1234.56789
	set com comment
	catch "exec $testP $cat $sub $par $typ $str $log $int $pre\
	                   $exp $dbl $com 1" msg
	set msg
}  {cat.subsystem.parameter             "T"; # comment                 }

test slxCreateShortFITS-2.3 {put an integer} {
	set cat cat
	set sub subsystem
	set par parameter
	set typ 3;		# type 1: string, 2: logical, 3: int, 4: double
	set str stringvalue
	set log 1
	set int	123456789
	set pre 255
	set exp	1
	set dbl 1234.56789
	set com comment
	catch "exec $testP $cat $sub $par $typ $str $log $int $pre\
	                   $exp $dbl $com 1" msg
	set msg
}  {cat.subsystem.parameter       123456789; # comment                 }

test slxCreateShortFITS-2.4 {put a double, 10 digits precision, not exponential} {
	set cat cat
	set sub subsystem
	set par parameter
	set typ 4;		# type 1: string, 2: logical, 3: int, 4: double
	set str stringvalue
	set log 1
	set int	32767
	set pre 10
	set exp	0
	set dbl 1234.56789
	set com comment
	catch "exec $testP $cat $sub $par $typ $str $log $int $pre\
	                   $exp $dbl $com 1" msg
	set msg
}  {cat.subsystem.parameter    1234.5678900000; # comment              }

test slxCreateShortFITS-2.5 {put a double, 10 digits precision, exponential} {
	set cat cat
	set sub subsystem
	set par parameter
	set typ 4;		# type 1: string, 2: logical, 3: int, 4: double
	set str stringvalue
	set log 1
	set int	32767
	set pre 10
	set exp	1
	set dbl 1234.56789
	set com comment
	catch "exec $testP $cat $sub $par $typ $str $log $int $pre\
	                   $exp $dbl $com 1" msg
	set msg
}  {cat.subsystem.parameter    1.2345678900E+03; # comment             }
