#!/usr/local/bin/tcl
#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxCreateWriteFITS.test.tcl,v 1.145 2005/07/27 10:54:10 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ---------------------------------------------
# eallaert  15/10/94  initial version
#

#**************************************************************************
#   NAME
#   slxCreateWriteFITS.test.prog - test program for slxCreateWriteFITS()
#
#   SYNOPSIS
#   slxCreateWriteFITS.test.prog
#
#   DESCRIPTION
#   This script tests the slxCreateWriteFITS program, as follows:
#	1.1 file descriptor not open
#	1.2 arg4 (parameterType) out of range
#
#	2.1 Normal execution of slxCreateWriteFITS - put a string
#	2.2 Normal execution of slxCreateWriteFITS - put a logical
#	2.3 Normal execution of slxCreateWriteFITS - put an integer
#	2.4 Normal execution of slxCreateWriteFITS - put a double, not-exp
#	2.5 Normal execution of slxCreateWriteFITS - put a double, exponential
#
#   To run these tests, the program slxTestCreateWriteFITS is used.
#
#   If any of these tests fail, an error report will be sent to stdout
#
#   CAUTIONS
#   To be executed from the ~slx/test directory
#
#-----------------------------------------------------------------------------

# Take over Ousterhout's test procedure 
source ../test/defs

#set VERBOSE 1

# testP is the name of the binary we use for testing our function
set testP ../bin/slxTestCreateWriteFITS


# In case of errors, exec $testP does an exit(1), so from Tcl
# we have to catch this error

# PART 1: argument testing
# ========================

# ---------------------------------------------------------------------------
# Produce error for fd not open
test slxCreateWriteFITS-1.1 {file descriptor not open} {
	set cat cat
	set sub subsystem
	set par parameter
	set typ 1;		# type 1: string, 2: logical, 3: int, 4: double
	set str stringvalue
	set log 1
	set int	32767
	set pre 255
	set exp	1
	set dbl 1234.56789
	set com comment
	catch "exec $testP $cat $sub $par $typ $str $log $int $pre\
	                   $exp $dbl $com 32759" msg
	set msg
}  {slxERR_WRONG_FD : File corresponding to file descriptor "32759" does not exist}

# ---------------------------------------------------------------------------
# Produce error return from slxCreateFITS
test slxCreateWriteFITS-1.2 {arg4 (parameterType) out of range} {
	set cat cat
	set sub subsystem
	set par parameter
	set typ 15;		# type should be from 1 - 4
	set str stringvalue
	set log 1
	set int	32767
	set pre 255
	set exp	1
	set dbl 1234.56789
	set com comment
	catch "exec $testP $cat $sub $par $typ $str $log $int $pre\
	                   $exp $dbl $com 1" msg
	set msg
}  {slxERR_CREATE_FITS : Could not create FITS keyword -\
format of input parameters wrong}


# PART 2: functionality test
# ==========================
test slxCreateWriteFITS-2.1 {put a string} {
	set cat cat
	set sub subsystem
	set par parameter
	set typ 1;		# type 1: string, 2: logical, 3: int, 4: double
	set str stringvalue
	set log 1
	set int	32767
	set pre 255
	set exp	1
	set dbl 1234.56789
	set com comment
	catch "exec $testP $cat $sub $par $typ $str $log $int $pre\
	                   $exp $dbl $com 1" msg
	set msg
}  {HIERARCH ESO cat subsystem parameter=    'stringvalue' / comment                }

test slxCreateWriteFITS-2.2 {put a logical} {
	set cat cat
	set sub subsystem
	set par parameter
	set typ 2;		# type 1: string, 2: logical, 3: int, 4: double
	set str stringvalue
	set log 1
	set int	32767
	set pre 255
	set exp	1
	set dbl 1234.56789
	set com comment
	catch "exec $testP $cat $sub $par $typ $str $log $int $pre\
	                   $exp $dbl $com 1" msg
	set msg
}  {HIERARCH ESO cat subsystem parameter=               T / comment                 }

test slxCreateWriteFITS-2.3 {put an integer} {
	set cat cat
	set sub subsystem
	set par parameter
	set typ 3;		# type 1: string, 2: logical, 3: int, 4: double
	set str stringvalue
	set log 1
	set int	123456789
	set pre 255
	set exp	1
	set dbl 1234.56789
	set com comment
	catch "exec $testP $cat $sub $par $typ $str $log $int $pre\
	                   $exp $dbl $com 1" msg
	set msg
}  {HIERARCH ESO cat subsystem parameter=       123456789 / comment                 }

test slxCreateWriteFITS-2.4 {put a double, 10 digits precision, not exponential} {
	set cat cat
	set sub subsystem
	set par parameter
	set typ 4;		# type 1: string, 2: logical, 3: int, 4: double
	set str stringvalue
	set log 1
	set int	32767
	set pre 10
	set exp	0
	set dbl 1234.56789
	set com comment
	catch "exec $testP $cat $sub $par $typ $str $log $int $pre\
	                   $exp $dbl $com 1" msg
	set msg
}  {HIERARCH ESO cat subsystem parameter=    1234.5678900000 / comment              }

test slxCreateWriteFITS-2.5 {put a double, 10 digits precision, exponential} {
	set cat cat
	set sub subsystem
	set par parameter
	set typ 4;		# type 1: string, 2: logical, 3: int, 4: double
	set str stringvalue
	set log 1
	set int	32767
	set pre 10
	set exp	1
	set dbl 1234.56789
	set com comment
	catch "exec $testP $cat $sub $par $typ $str $log $int $pre\
	                   $exp $dbl $com 1" msg
	set msg
}  {HIERARCH ESO cat subsystem parameter=    1.2345678900E+03 / comment             }
