#!/usr/local/bin/tcl
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxGetKeyword.test.tcl,v 1.145 2005/07/27 10:43:01 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# jknudstr  13/10/94  created
#

#************************************************************************
#   NAME
#   slxGetKeyword.test.prog - test script for slxGetKeyword()
# 
#   SYNOPSIS
#   slxGetKeyword.test.prog
# 
#   DESCRIPTION
#   Used for testing the SLX function slxGetKeyword().
#
#   The following tests are carried out
#   
#   1.1 Search for keywords according to name (existing)
#
#   1.2 Search for keywords according to name (non-existing)
#
#   1.3 Search for keywords according to number (existing)
#
#   1.4 Search for keywords according to number (non-existing)
#
#
#   CAUTIONS
#   Should be executed from the ~slx/test directory.
#
#------------------------------------------------------------------------
#
source ../test/defs
source ../test/slxTestCommon.tcl

set tf slxGetKeyword.test
set testPath [pwd]
set env(INS_SETUPPATH) [pwd]
set env(INS_ROOT) [pwd]/INS_ROOT
set env(INS_USER) SYSTEM
set slxInfoKeywordSetup ../bin/slxInfoKeywordSetup

# PART 1.1: Search for keywords according to name (existing)
# ==========================================================
test slxGetKeyword-1.1 {Search for keywords by names} {
    exec echo Begin > $tf.1.1.temp
    exec $slxInfoKeywordSetup -k SUP.INST.ID $tf.1.1.bs.rf  -t >> $tf.1.1.temp
    exec $slxInfoKeywordSetup -k GEN.TEMPVAR $tf.1.1.bs.rf   -t >> $tf.1.1.temp
    exec $slxInfoKeywordSetup -k SUP.CHCK.NAME $tf.1.1.bs.rf   -t >> $tf.1.1.temp
    exec $slxInfoKeywordSetup -k TEL.ID $tf.1.1.bs.rf   -t >> $tf.1.1.temp
    exec $slxInfoKeywordSetup -k INS.COMP.ID $tf.1.1.bs.rf   -t >> $tf.1.1.temp
    exec $slxInfoKeywordSetup -k ADA.GUID-1.Y $tf.1.1.bs.rf   -t >> $tf.1.1.temp
    exec $slxInfoKeywordSetup -k DET.PIXSIZE $tf.1.1.bs.rf   -t >> $tf.1.1.temp
    exec $slxInfoKeywordSetup -k ARC.CHECKSUM $tf.1.1.bs.rf   -t >> $tf.1.1.temp
    exec $slxInfoKeywordSetup -k DET.OUTPUTS $tf.1.1.bs.rf   -t >> $tf.1.1.temp
    exec $slxInfoKeywordSetup -k DET.TEMPMEAN $tf.1.1.bs.rf   -t >> $tf.1.1.temp
    exec $slxInfoKeywordSetup -k DET.FRAM.NAXIS2 $tf.1.1.bs.rf   -t >> $tf.1.1.temp
    exec $slxInfoKeywordSetup -k DET.DKTIME $tf.1.1.bs.rf   -t >> $tf.1.1.temp
    exec $slxInfoKeywordSetup -k DET.COMP.ID $tf.1.1.bs.rf   -t >> $tf.1.1.temp
    exec $slxInfoKeywordSetup -k ARC.DATAMEAN $tf.1.1.bs.rf   -t >> $tf.1.1.temp
    exec $slxInfoKeywordSetup -k SUP.MODE $tf.1.1.bs.rf   -t >> $tf.1.1.temp
    exec $slxInfoKeywordSetup -k SUP.LCHG.UT $tf.1.1.bs.rf   -t >> $tf.1.1.temp
    exec $slxInfoKeywordSetup -k GEN.TEMP $tf.1.1.bs.rf   -t >> $tf.1.1.temp

    set msg [exec diff $tf.1.1.temp $tf.1.1.rf]
    exec rm $tf.1.1.temp
    set msg
} ""



# PART 1.2 Search for keyword according to name (non-existing)
# ============================================================
test slxGetKeyword-1.2 {Search for non-exist. keyword by name} {
    if {[catch "exec -keepnewline $slxInfoKeywordSetup -k crkrsdjk $tf.1.2.bs.rf" msg]} {
	set testString "The keyword \"crkrsdjk\" was not found"
	set msg [slxTestTestString $testString $msg]
    } else {
	set msg FAILURE
    }
    set msg
} ""


# PART 1.3 Search for keywords according to number (existing)
# ===========================================================
test slxGetKeyword-1.3 {Search for keywords by numbers} {
    exec echo Begin > $tf.1.3.temp
    exec $slxInfoKeywordSetup -n 1 $tf.1.3.bs.rf -t >> $tf.1.3.temp
    exec $slxInfoKeywordSetup -n 5 $tf.1.3.bs.rf  -t >> $tf.1.3.temp
    exec $slxInfoKeywordSetup -n 10 $tf.1.3.bs.rf  -t >> $tf.1.3.temp
    exec $slxInfoKeywordSetup -n 11 $tf.1.3.bs.rf  -t >> $tf.1.3.temp
    exec $slxInfoKeywordSetup -n 20 $tf.1.3.bs.rf  -t >> $tf.1.3.temp
    exec $slxInfoKeywordSetup -n 31 $tf.1.3.bs.rf  -t >> $tf.1.3.temp
    exec $slxInfoKeywordSetup -n 32 $tf.1.3.bs.rf  -t >> $tf.1.3.temp
    exec $slxInfoKeywordSetup -n 40 $tf.1.3.bs.rf  -t >> $tf.1.3.temp
    exec $slxInfoKeywordSetup -n 45 $tf.1.3.bs.rf  -t >> $tf.1.3.temp
    exec $slxInfoKeywordSetup -n 50 $tf.1.3.bs.rf  -t >> $tf.1.3.temp
    exec $slxInfoKeywordSetup -n 53 $tf.1.3.bs.rf  -t >> $tf.1.3.temp
    exec $slxInfoKeywordSetup -n 61 $tf.1.3.bs.rf  -t >> $tf.1.3.temp
    exec $slxInfoKeywordSetup -n 70 $tf.1.3.bs.rf  -t >> $tf.1.3.temp

    set msg [exec diff $tf.1.3.temp $tf.1.3.rf]
    exec rm $tf.1.3.temp
    set msg
} ""



# PART 1.4 Search for keywords according to number (non-existing)
# ===============================================================
test slxGetKeyword-1.4 {Search for non-exist. keyword by number} {
    if {[catch "exec -keepnewline $slxInfoKeywordSetup -n 10000 $tf.1.4.bs.rf" msg]} {
	set testString "The keyword \"\" was not found"
	set msg [slxTestTestString $testString $msg]
    } else {
	set msg FAILURE
    }
    set msg
} ""

# --- oOo ---
