/******************************************************************************
* E.S.O. - VLT project
*
* slxInfoKeywordSetup.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  04/08/94  Created 1. version
* epozna    30/07/02  changed exit->return to avoid warnings
*/

/************************************************************************
*   NAME
*   slxInfoKeywordSetup - extracts features for keyword in set-up file
* 
*   SYNOPSIS
*   slxInfoKeywordSetup -k|-n  <keyword>|<number> <set-up filename> [-t]
*
*   DESCRIPTION
*   The program searches for a keyword in a specified set-up file. The keyword
*   can either be searched for according to the name (in Short-FITS format)
*   or according to the number in the set-up file.
*
*   -k|-n         Determines if there should be searched for the keyword 
*                 according to the name (-k) or number in the set-up file.
*
*   <keyword>|    The name of the keyword or the number. The name must be 
*   <number>      given in the Short-FITS format, CAT.[SUB.]PAR. The number
*                 is the number of the keyword as listed in the set-up file. 
*                 First keyword has number 1. Note that the number may
*                 change when the set-up file is updated.
*
*   <set-up file> The set-up file that should be searched in. Can be given in
*                 with or without the extension (".ref", ".ins", ".targ", 
*                 ".det"), and with or without the complete path.
*
*   -t            Used for running the program in test mode. Should not be used
*                 during normal operation.
*
* 
*   ENVIRONMENT
*   INS_SETUPPATH  The variable defines places where set-up files and set-up
*                  related files are situated.
*                  If not set, uses the current directory.
* 
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h" 

static char *rcsId="@(#) $Id: slxInfoKeywordSetup.c,v 1.145 2005/07/27 12:23:10 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers 
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* Local Headers 
 */
#include "slx.h"

/* Main
 */
int main(int argc, char *argv[])

{
    ccsERROR        error;
    char            completeName[256];
    ccsPROCNAME     procName;
    slxKEYW_INFO    keywordInfo;
    slxSETUP_FORMAT type;
    vltUINT8        test = 0;

 
    if (argc < 4)
        {
        printf("\nCorrect syntax is:\n\n");
        printf("slxInfoKeywordSetup -k|-n <keyword>|<number> <set-up>\n\n");
        return(1);
        }

    if (argc == 5)
        test = 1;

    if (slxLocateSetupFile(argv[3], completeName, &error) == FAILURE)
        {
        printf("\nError in locating set-up file (%s)\n\n", argv[3]); 
        return(1);
        }

    if (slxDetectFormat(completeName, &type, &error) == FAILURE)
        {
        printf("\nError in accessing set-up file (%s)\n\n", completeName);
        return(1);
        }
    if (type != slxSHORT_FITS)
	{
        printf("\nFormat of set-up file wrong (%s)\n\n", completeName);
        return(1);
        }
    strcpy(procName, "slxInfoKeywSUP");
    if (ccsInit(procName, 0, NULL, NULL, &error) == FAILURE)
        {
        printf("\nError in initializing on RTAP!\n\n");
        return(1);
        }

    if (strcmp(argv[1], "-k") == 0)
        {
        if (slxGetKeyword(argv[3], argv[2], 0, &keywordInfo, 
                          &error) == FAILURE)
            {
            fprintf(stderr, "\nError in looking up keyword!!\n\n");
            fprintf(stderr, "Failure, error is:\n");
            fprintf(stderr, "   ModuleId   : %s\n", error.moduleId);
            fprintf(stderr, "   Parameters : %s\n", error.runTimePar);
            ccsExit(&error);
            return(1);
            }
        }
    else
        {
        if (slxGetKeyword(argv[3], "", atoi(argv[2]), &keywordInfo, 
                          &error) == FAILURE)
            {
            fprintf(stderr, "\nError in looking up keyword!!\n\n");
            fprintf(stderr, "Failure, error is:\n");
            fprintf(stderr, "   ModuleId   : %s\n", error.moduleId);
            fprintf(stderr, "   Parameters : %s\n", error.runTimePar);
            ccsExit(&error);
            return(1);
            }
        }

    printf("\nFeatures for the keyword: \n");
    if (!test)
        printf("\nSet-up file searched: %s", completeName);
    printf("\nLine Number: %d", keywordInfo.number);
    printf("\nKeyword Name: %s", keywordInfo.keyword);
    printf("\nCategory: %s", keywordInfo.category);
    printf("\nSubsystem: %s", keywordInfo.subSystem);
    printf("\nParameter: %s", keywordInfo.parameter);
    printf("\nValue: %s", keywordInfo.value);
    printf("\nComment: %s\n\n", keywordInfo.comment);

    ccsExit(&error);
    return(0);
}















