#!/usr/local/bin/tcl
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxLoadSetup.test.tcl,v 1.145 2005/07/27 10:55:48 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# jknudstr  13/10/94  created
#

#************************************************************************
#   NAME
#   slxLoadSetup.test.prog - test script for slxLoadSetup()
# 
#   SYNOPSIS
#   slxLoadSetup.test.prog
# 
#   DESCRIPTION
#   The script tests the function slxConvertToNormal.
#
#   The script carries out three tests:
#
#   1. Test of attempt to convert non-existing file.
#
#   2. Test of attempt to convert obscure file.
#
#   3. Attempt to convert rwx protected file.
#
#   4. Functionality test.
#
#
#   The script writes an error report to standard output. 
#
#
#   FILES
#   slxLoadSetup.test.2.bs      Is an obscure set-up file.
#
#   slxLoadSetup.test.3.bs      File to be rwx protected.
#
#   slxLoadSetup.test.4.bs      Is a file in Short-FITS format 
#                                  containing a lot of various 
#                                  combinations.
#
#   slxLoadSetup.test.4.rf  Is the.rf to compare with
#                                  the result of loading and saving again
#                                  the slxLoadSetup.test.4.bs file.
#
#
#   CAUTIONS
#   Should be executed from the ~slx/test directory.
#
#------------------------------------------------------------------------
#
source ../test/defs
source ../test/slxTestCommon.tcl

set tf slxLoadSetup.test
set testPath [pwd]
set env(INS_SETUPPATH) [pwd]
set env(INS_ROOT) [pwd]/INS_ROOT
set env(INS_USER) SYSTEM
set slxTestLoadSetup ../bin/slxTestLoadSetup


# PART 1: attempting to open non-existing set-up file
# ===================================================
test slxLoadSetup-1 {Non existing file} {
    if {[catch "exec -keepnewline $slxTestLoadSetup NoNeXiStInGfIlE slxLoadSetup.1.temp" msg]} {
	set testString "Could not find \"NoNeXiStInGfIlE\""
	set msg [slxTestTestString $testString $msg]
    } else {
	set msg FAILURE
    }
    set msg
}   ""


# PART 2: format of file is wrong
# ================================
test slxLoadSetup-2 {File format wrong} {
    catch "exec -keepnewline $slxTestLoadSetup $tf.2.bs $tf.2.temp" msg
    set testString "File \"slxLoadSetup.test.2.bs\" does"
    set msg [slxTestTestString $testString $msg]
    set msg
}  ""


# PART 3: try to open full-protected (rwx) file
# =============================================
exec chmod -rwx $tf.3.bs
test slxLoadSetup-3 {File rwx protected} {
    if {[catch "exec -keepnewline $slxTestLoadSetup $tf.3.bs $tf.3.temp" msg]} {
	set testString "Could not open file \"slxLoadSetup.test.3.bs\""
	set msg [slxTestTestString $testString $msg]
    } else {
	set msg FAILURE
    }
    set msg
    
}    ""
exec chmod +rwx $tf.3.bs


# PART 4: test basic functionality
# ================================
test slxLoadSetup-4 {Basic functionality test} {
    exec $slxTestLoadSetup $testPath/$tf.4.bs $testPath/$tf.4.temp
    set msg [exec diff $testPath/$tf.4.temp $testPath/$tf.4.rf]
    exec rm $testPath/$tf.4.temp
    set msg
} ""

# --- oOo ---






