#!/usr/local/bin/tcl
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxLookUpKeyword.test.tcl,v 1.145 2005/07/27 11:12:24 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# jknudstr  13/10/94  created
#

#************************************************************************
#   NAME
#   slxLookUpKeyword.test.prog - test script for slxLookUpKeyword()
# 
#   SYNOPSIS
#   slxLookUpKeyword.test.prog
# 
#   DESCRIPTION
#   Used for testing the SLX function slxLookUpKeyword().
#
#   The following tests are carried out
#   
#   1.1 Search for keywords according to name (existing)
#
#   1.2 Search for keywords according to name (non-existing)
#
#   1.3 Search for keywords according to number (existing)
#
#   1.4 Search for keywords according to number (non-existing)
#
#   2.1 Non-existing dictionary specified 
#
#   2.2 Inconsistent dictionary specified
#
#
#   CAUTIONS
#   Should be executed from the ~slx/test directory.
#
#------------------------------------------------------------------------
#
source ../test/defs

set tf slxLookUpKeyword.test
set testPath [pwd]
set env(INS_SETUPPATH) [pwd]

set env(INS_SETUPPATH) [pwd]
set env(INS_ROOT) [pwd]/INS_ROOT
set env(INS_USER) SYSTEM



# PART 1.1: Search for keywords according to name (existing)
# ==========================================================
test slxLookUpKeyword-1.1 {Search for keywords by names} {
    exec echo Begin > $tf.1.1.temp
    exec slxInfoKeywordDic -k ADA.ABSROT.START ISAAC  >> $tf.1.1.temp
    exec slxInfoKeywordDic -k ADA.ABSROT.END ISAAC  >> $tf.1.1.temp
    exec slxInfoKeywordDic -k ADA.ABSROT.MIN ISAAC  >> $tf.1.1.temp
    exec slxInfoKeywordDic -k ADA.ABSROT.MAX ISAAC  >> $tf.1.1.temp
    exec slxInfoKeywordDic -k ADA.GUID-1.X ISAAC  >> $tf.1.1.temp
    exec slxInfoKeywordDic -k ADA.POSANG ISAAC  >> $tf.1.1.temp
    exec slxInfoKeywordDic -k DET.COMP.ID ISAAC  >> $tf.1.1.temp
    exec slxInfoKeywordDic -k DET.DKTIME ISAAC  >> $tf.1.1.temp
    exec slxInfoKeywordDic -k DET.FRAM.CRPIX1 ISAAC  >> $tf.1.1.temp
    exec slxInfoKeywordDic -k DET.FRAM.NAXIS2 ISAAC  >> $tf.1.1.temp
    exec slxInfoKeywordDic -k GEN.CAT.NAME ISAAC  >> $tf.1.1.temp
    exec slxInfoKeywordDic -k INS.ADIC.ID ISAAC  >> $tf.1.1.temp
    exec slxInfoKeywordDic -k INSTRUME ISAAC  >> $tf.1.1.temp
    exec slxInfoKeywordDic -k OBSERVER ISAAC  >> $tf.1.1.temp
    exec slxInfoKeywordDic -k SUP.ID ISAAC  >> $tf.1.1.temp
    exec slxInfoKeywordDic -k TEL.TRAK.STATUS ISAAC  >> $tf.1.1.temp
    exec slxInfoKeywordDic -k TM-START ISAAC  >> $tf.1.1.temp

    set msg [exec diff $tf.1.1.temp $tf.1.1.rf]
    exec rm $tf.1.1.temp
    set msg
} ""



# PART 1.2 Search for keyword according to name (non-existing)
# ============================================================
test slxLookUpKeyword-1.2 {Search for non-exist. keyword by name} {
   if {[catch "exec -keepnewline slxInfoKeywordDic -k ADA.ABSROT ISAAC" msg]} {
        set msg
    }
} "\nError in looking up keyword!!

Failure, error is:
   ModuleId   : slx
   LocationId : slxLookUpKeyword.c
   ErrorNumber: 35
   Parameters : slxERR_KEYWORD_NOT_FOUND : The keyword \"ADA.ABSROT\" was not found\n"


# PART 1.3 Search for keywords according to number (existing)
# ===========================================================
test slxLookUpKeyword-1.3 {Search for keywords by numbers} {
    exec echo Begin > $tf.1.3.temp
    exec slxInfoKeywordDic -n 1 ISAAC  >> $tf.1.3.temp
    exec slxInfoKeywordDic -n 5 ISAAC  >> $tf.1.3.temp
    exec slxInfoKeywordDic -n 10 ISAAC  >> $tf.1.3.temp
    exec slxInfoKeywordDic -n 11 ISAAC  >> $tf.1.3.temp
    exec slxInfoKeywordDic -n 20 ISAAC  >> $tf.1.3.temp
    exec slxInfoKeywordDic -n 31 ISAAC  >> $tf.1.3.temp
    exec slxInfoKeywordDic -n 32 ISAAC  >> $tf.1.3.temp
    exec slxInfoKeywordDic -n 40 ISAAC  >> $tf.1.3.temp
    exec slxInfoKeywordDic -n 45 ISAAC  >> $tf.1.3.temp
    exec slxInfoKeywordDic -n 50 ISAAC  >> $tf.1.3.temp
    exec slxInfoKeywordDic -n 53 ISAAC  >> $tf.1.3.temp
    exec slxInfoKeywordDic -n 61 ISAAC  >> $tf.1.3.temp
    exec slxInfoKeywordDic -n 70 ISAAC  >> $tf.1.3.temp

    set msg [exec diff $tf.1.3.temp $tf.1.3.rf]
    exec rm $tf.1.3.temp
    set msg
} ""



# PART 1.4 Search for keywords according to number (non-existing)
# ===============================================================
test slxLookUpKeyword-1.4 {Search for non-exist. keyword by number} {
   if {[catch "exec -keepnewline slxInfoKeywordDic -n 10000 ISAAC" msg]} {
        set msg
    }
} "\nError in looking up keyword!!

Failure, error is:
   ModuleId   : slx
   LocationId : slxLookUpKeyword.c
   ErrorNumber: 35
   Parameters : slxERR_KEYWORD_NOT_FOUND : The keyword \"\" was not found\n"


# PART 2.1 Non-existing dictionary specified 
# ==========================================
test slxLookUpKeyword-2.1 {Search for non-exist. keyword by number} {
   if {[catch "exec -keepnewline slxInfoKeywordDic -n 1 zzzZZZ" msg]} {
        set msg
    }
} "\nError in looking up keyword!!

Failure, error is:
   ModuleId   : slx
   LocationId : slxLookUpKeyword.c
   ErrorNumber: 24
   Parameters : slxERR_OPEN_DIC : Could not open dictionary \"$env(INS_ROOT)/SYSTEM/Dictionary/ESO-VLT-DIC.zzzZZZ\"\n"


# --- oOo ---
