#!/usr/local/bin/tcl
#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxNormalToShortFITS.test.tcl,v 1.145 2005/07/27 11:20:47 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ---------------------------------------------
# eallaert  15/10/94  initial version
#

#**************************************************************************
#   NAME
#   slxNormalToShortFITS.test.tcl - test program for slxNormalToShortFITS()
#
#   SYNOPSIS
#   slxNormalToShortFITS.test.tcl
#
#   DESCRIPTION
#   This script tests the slxNormalToShortFITS program, as follows:
#	1.1 "setupfile" is not in FITS format
#
#	2.1 Normal execution of slxNormalToShortFITS - file is to be found in
#	    $INS_SETUPPATH
#	2.2 Normal execution of slxNormalToShortFITS - absolute file path is 
#	    given
#
#   To run these tests, the program slxSetupToShortFITS is used.  The tests
#   for the latter program should therefore be run before this one.
#
#   If any of these tests fail, an error report will be sent to stdout
#
#   FILES
#   The following files are required and accessed under ~slx/test:
#	- slxNormalToShortFITS.test.tcl (test 1.1)
#	- slxFitsHeader.test.bs (tests 2.1, 2.2)
#
#   CAUTIONS
#   To be executed from the ~slx/test directory
#
#-----------------------------------------------------------------------------

# Take over Ousterhout's test procedure 
source ../test/defs

#set VERBOSE 1
catch {unset env(INS_SETUPPATH)} msg

# This test program will use the services of the program "slxSetupToShortFITS".
# It is therefore expected that the test suite for the latter was run 
# successfully before.
set testP slxSetupToShortFITS


# For more info, see also the test suite for "slxSetupToShortFITS".

# PART 1: produce errors
# ======================

# ---------------------------------------------------------------------------
# file is not in appropriate format
# This is an error produced by slxNormalToShortFITS
set tf slxNormalToShortFITS.test.tcl
test slxNormalToShortFITS-1.1 {"setupfile" is not in FITS format} {
    set env(INS_SETUPPATH) [pwd]
    exec cp $tf $tf.temp
    exec chmod ugo=+rwx $tf.temp
    catch "exec $testP $tf.temp -t" msg
    unset env(INS_SETUPPATH)
    exec rm $tf.temp
    set msg
}   "\nCouldn't convert the file - check if filename is correct!

slxERR_FILE_FORMAT : File \"slxNormalToShortFITS.test.tcl.temp\" does not have the correct format"


# PART 2: test functionality
# ==========================

# ---------------------------------------------------------------------------
# file is to be found in $INS_SETUPPATH
test slxNormalToShortFITS-2.1 {file is to be found in $INS_SETUPPATH} {
    set tf slxFitsHeader.test
    set env(INS_SETUPPATH) [pwd]
    exec cp $tf.bs $tf.temp
    exec chmod ugo=+rwx $tf.temp
    if {[catch "exec -keepnewline $testP $tf.temp -t" msg] == 0} {
	set msg [exec diff $tf.temp $tf.rf]
    } 
    unset env(INS_SETUPPATH)
    exec rm $tf.temp
    set msg
}   ""

# ---------------------------------------------------------------------------
# file path is given, $INS_SETUPPATH not set
test slxNormalToShortFITS-2.2 {absolute file path is given} {
    set tf slxFitsHeader.test
    exec cp $tf.bs $tf.temp
    exec chmod ugo=+rwx $tf.temp
    if {[catch "exec -keepnewline $testP [pwd]/$tf.temp -t" msg] == 0} {
	set msg [exec diff $tf.temp $tf.rf]
    } 
    exec rm $tf.temp
    set msg
}   ""
