#!/usr/local/bin/tcl
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxPrepImageFile.test.tcl,v 1.145 2005/07/27 11:21:19 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# jknudstr  13/10/94  created
#

#************************************************************************
#   NAME
#   slxPrepImageFile.test.prog - test script for slxPrepImageFile()
# 
#   SYNOPSIS
#   slxPrepImageFile.test.prog
# 
#   DESCRIPTION
#   Used for testing the SLX function slxPrepImageFile().
#
#   The following tests are carried out
#   
#   1.1 Normal behaviour - create one block
#
#   1.2 Normal behaviour - create one block + add "END"
#
#   1.3 Normal behaviour - create 3 blocks
#
#   1.4 Normal behaviour - create 6 blocks + write "END"
#
#   2.1 File specified exists
#
#
#   CAUTIONS
#   Should be executed from the ~slx/test directory.
#
#------------------------------------------------------------------------
#
source ../test/defs
source ../test/slxTestCommon.tcl

set tf slxPrepImageFile.test
set testPath [pwd]
set slxTestPrepImFile ../bin/slxTestPrepImFile


# PART 1.1 Normal behaviour - create one block
# ============================================
test slxPrepImageFile-1.1 {Normal behaviour create one block} {
    exec $slxTestPrepImFile $testPath/$tf.1.1.temp 1 NO
    set msg [exec diff $tf.1.1.temp $tf.1.1.rf]
    exec rm $tf.1.1.temp
    set msg
} ""


# PART 1.2 Normal behaviour - create one block + add "END"
# ========================================================
test slxPrepImageFile-1.2 {Normal behaviour create one block, write END} {
    exec $slxTestPrepImFile $testPath/$tf.1.2.temp 1 YES
    set msg [exec diff $tf.1.2.temp $tf.1.2.rf]
    exec rm $tf.1.2.temp
    set msg
} ""



# PART 1.3 Normal behaviour - create 3 blocks
# ===========================================
test slxPrepImageFile-1.3 {Normal behaviour create 3 blocks} {
    exec $slxTestPrepImFile $testPath/$tf.1.3.temp 3 NO
    set msg [exec diff $tf.1.3.temp $tf.1.3.rf]
    exec rm $tf.1.3.temp
    set msg
} ""



# PART 1.4 Normal behaviour - create 6 blocks + write "END"
# ========================================================
test slxPrepImage-1.4 {Normal behaviour create 6 blocks} {
    exec $slxTestPrepImFile $testPath/$tf.1.4.temp 6 YES
    set msg [exec diff $tf.1.4.temp $tf.1.4.rf]
    exec rm $tf.1.4.temp
    set msg
} ""



# PART 2.1 File specified exists
# ==============================
exec chmod -rw $tf.2.1.bs
test slxPrepImageFile-2.1 {File specified exists} {
    if {[catch "exec -keepnewline $slxTestPrepImFile $testPath/$tf.2.1.bs 1 YES" msg]} {
	set testString "Could not create file \"$testPath/slxPr"
	set msg [slxTestTestString $testString $msg]
    } else {
	set msg FAILURE
    }
    set msg
} ""
exec chmod +rw $tf.2.1.bs 

# --- oOo ---
