#!/usr/local/bin/tcl
#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxSetupToShortFITS.test.tcl,v 1.145 2005/07/27 11:26:19 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ---------------------------------------------
# eallaert  15/10/94  initial version
#

#**************************************************************************
#   NAME
#   slxSetupToShortFITS.test.prog - test program for slxSetupToShortFITS
#
#   SYNOPSIS
#   slxSetupToShortFITS.test.prog
#
#   DESCRIPTION
#   This script tests the slxSetupToShortFITS program, as follows:
#	1.1 number of arguments: none
#	1.2 number of arguments: two, but second is not -t
#	1.3 not existing FITS header file
#
#	2.1 Normal execution of slxSetupToShortFITS
#
#   If any of these tests fail, an error report will be sent to stdout
#
#   FILES
#   The following files are required and accessed under ~slx/test:
#	- slxFitsHeader.test.bs (test 2.1)
#
#   CAUTIONS
#   To be executed from the ~slx/test directory
#
#-----------------------------------------------------------------------------


# Take over Ousterhout's test procedure 
source ../test/defs

set env(INS_SETUPPATH) [pwd]
set env(INS_ROOT) [pwd]/INS_ROOT
set env(INS_USER) SYSTEM

# In case of errors, slxSetupToShortFITS does an exit(1), so from Tcl
# we have to catch this error
catch {unset env(INS_SETUPPATH)} msg
set slxSetupToShortFITS slxSetupToShortFITS

# PART 1: argument testing
# ========================

# ---------------------------------------------------------------------------
# Produce error for wrong number of parameters (must be one or two)
test slxSetupToShortFITS-1.1 {number of arguments: none} {
	catch "exec $slxSetupToShortFITS " msg
	set msg
}   "\nCorrect syntax is:\n\nslxSetupToShortFITS  <setup filename> \[-t\]\n"

# ---------------------------------------------------------------------------
# Now two arguments, but the second is wrong
test slxSetupToShortFITS-1.2 {number of arguments: two, but second is not -t} {
	catch "exec $slxSetupToShortFITS arg1 arg2" msg
	set msg
}   "\nCorrect syntax is:\n\nslxSetupToShortFITS  <setup filename> \[-t\]\n"

# ---------------------------------------------------------------------------
# Now get any failure from slxConvertToShort, e.g. for a not existing file...
# ... and even INS_SETUPPATH is not set
test slxSetupToShortFITS-1.3 {not existing FITS header file} {
	catch "exec $slxSetupToShortFITS sureUcantFindThis1 -t" msg
	set msg
}   "\nCouldn't convert the file - check if filename is correct!\n
slxERR_LOCATE_FILE : Could not find \"sureUcantFindThis1\" according to INS_SETUPPATH or path given with filename";

# PART 2: functionality test
# ==========================
# use as testfile "fitsHeader"
# if all is well, slxSetupToShortFITS should take an exit(0) without 
# additional messages.
# ---------------------------------------------------------------------------
# Keep the output of "diff", to restore after removing temporary files
test slxSetupToShortFITS-2.1 {the real thing ...} {
    set tf slxFitsHeader.test
    set env(INS_SETUPPATH) [pwd]
    exec cp $tf.bs $tf.temp
    exec chmod ugo=+rwx $tf.temp
    exec $slxSetupToShortFITS $tf.temp -t
    set msg [exec diff $tf.temp $tf.rf]
    exec rm $tf.temp
    set msg
}   ""

