#!/usr/local/bin/tcl
#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxShortToNormalFits2.test.tcl,v 1.145 2005/07/27 11:35:53 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ---------------------------------------------
# jknudstr  09/04/96  created


#**************************************************************************
#   NAME
#   slxShortToNormalFits2.test.prog - test program for slxCreateFits2
#
#   SYNOPSIS
#   slxShortToNormalFits2.test.prog
#
#   DESCRIPTION
#
#-----------------------------------------------------------------------------
# Take over Ousterhout's test procedure 
source ../test/defs

set env(INS_SETUPPATH) [pwd]
set env(INS_ROOT) [pwd]/INS_ROOT
set env(INS_USER) SYSTEM

# testP is the name of the binary we use for testing our function
set testP ../bin/slxTestShortToNormalFits2
set shFitsFile [pwd]/../test/shortFits.temp
set dic   GROWING


# PART 1: functionality test
# ==========================
test slxShortToNormalFits2-1.1.1 {string - par} {
	catch "exec chmod ugo=+rwx $shFitsFile"
	catch "exec rm $shFitsFile"
	set shortFitsLine "DATE     string;       # Date tape written"
	exec echo $shortFitsLine > $shFitsFile
	exec $testP $dic $shFitsFile
} "|DATE    = 'string  '                    / Date this tape written dd/mm/yy       |"

test slxShortToNormalFits2-1.1.2 {string - cat + par} {
	catch "exec chmod ugo=+rwx $shFitsFile"
	catch "exec rm $shFitsFile"
	set shortFitsLine "ADA.ID    string;   # Adapter Identification"
	exec echo $shortFitsLine > $shFitsFile
	exec $testP $dic $shFitsFile
} "|HIERARCH ESO ADA ID          = 'string  '   / Adapter Identification            |"

test slxShortToNormalFits2-1.1.3 {string - cat + sub + par} {
	catch "exec chmod ugo=+rwx $shFitsFile"
	catch "exec rm $shFitsFile"
	set shortFitsLine "ADA.LAMP-1.NAME string; # Common name for lamp i  "
	exec echo $shortFitsLine > $shFitsFile
	exec $testP $dic $shFitsFile
} "|HIERARCH ESO ADA LAMP-1 NAME = 'string  '   / Common name for lamp i            |"


test slxShortToNormalFits2-1.1.4 {string - cat + 2 * sub + par} {
	catch "exec chmod ugo=+rwx $shFitsFile"
	catch "exec rm $shFitsFile"
	set shortFitsLine "ADA.LAMP-1.LAMP.NAME string;  # A test keyword      "
	exec echo $shortFitsLine > $shFitsFile
	exec $testP $dic $shFitsFile
} "|HIERARCH ESO ADA LAMP-1 LAMP NAME= 'string  ' / A test keyword                  |"

test slxShortToNormalFits2-1.2.1 {logical - par} {
	catch "exec chmod ugo=+rwx $shFitsFile"
	catch "exec rm $shFitsFile"
	set shortFitsLine "SIMPLE          T; \# Test comment"
	exec echo $shortFitsLine > $shFitsFile
	exec $testP $dic $shFitsFile
} "|SIMPLE  =                    T          / Standard FITS format (rev feb 1985)   |"

test slxShortToNormalFits2-1.2.2 {logical - cat + par} {
	catch "exec chmod ugo=+rwx $shFitsFile"	
	catch "exec rm $shFitsFile"
	set shortFitsLine "DET.ST                   F; \# Test keyword   "
	exec echo $shortFitsLine > $shFitsFile
	exec $testP $dic $shFitsFile
} "|HIERARCH ESO DET ST          =            F / Test keyword                      |"

test slxShortToNormalFits2-1.2.3 {logical - cat + sub + par} {
	catch "exec chmod ugo=+rwx $shFitsFile"
	catch "exec rm $shFitsFile"
	set shortFitsLine "DET.OUT-10.ST            F; \# Output i status (T=enabled) "
	exec echo $shortFitsLine > $shFitsFile
	exec $testP $dic $shFitsFile
} "|HIERARCH ESO DET OUT-10 ST   =            F / Output i status (T=enabled)       |"

test slxShortToNormalFits2-1.2.4 {logical - cat + 2 * sub par} {
	catch "exec chmod ugo=+rwx $shFitsFile"	
	catch "exec rm $shFitsFile"
	set shortFitsLine "DET.OUT-10.XX.ST          F; \# Test keyword          "
	exec echo $shortFitsLine > $shFitsFile
	exec $testP $dic $shFitsFile
} "|HIERARCH ESO DET OUT-10 XX ST=            F / Test keyword                      |"

test slxShortToNormalFits2-1.3.1 {integer - par} {
	catch "exec chmod ugo=+rwx $shFitsFile"
	catch "exec rm $shFitsFile"
	set shortFitsLine "BITPIX   12345 ;                        # xxxx"  
	exec echo $shortFitsLine > $shFitsFile
	exec $testP $dic $shFitsFile
} "|BITPIX  =                12345          / No of bits storing pix values         |"

test slxShortToNormalFits2-1.3.2 {integer - cat + par} {
	catch "exec chmod ugo=+rwx $shFitsFile"
	catch "exec rm $shFitsFile"
	set shortFitsLine "ARC.CHECKSUM         123456; # fjfhkj"
	exec echo $shortFitsLine > $shFitsFile
	exec $testP $dic $shFitsFile
} "|HIERARCH ESO ARC CHECKSUM    =       123456 / File CRC (32 bit CCITT)           |"

test slxShortToNormalFits2-1.3.3 {integer - cat + sub + par} {
	catch "exec chmod ugo=+rwx $shFitsFile"
	catch "exec rm $shFitsFile"
	set shortFitsLine "DET.FRAM.BITS       1234567; "
	exec echo $shortFitsLine > $shFitsFile
	exec $testP $dic $shFitsFile
} "|HIERARCH ESO DET FRAM BITS   =      1234567 / Number of bits/pixel transfered   |"

test slxShortToNormalFits2-1.3.4 {integer - cat + 2 * sub + par} {
	catch "exec chmod ugo=+rwx $shFitsFile"
	catch "exec rm $shFitsFile"
	set shortFitsLine "DET.FRAM.XXXX.BITS    123456 "
	exec echo $shortFitsLine > $shFitsFile
	exec $testP $dic $shFitsFile
} "|HIERARCH ESO DET FRAM XXXX BITS=     123456 / Test keyword                      |"

##########################################################################
# Floating point
test slxShortToNormalFits2-1.4.1 {double - par} {
	catch "exec chmod ugo=+rwx $shFitsFile"
	catch "exec rm $shFitsFile"
	set shortFitsLine "BSCALE   12345.1                      "
	exec echo $shortFitsLine > $shFitsFile
	exec $testP $dic $shFitsFile
} "|BSCALE  =              12345.1          / Real=Tape*BSCALE+BZERO                |"

test slxShortToNormalFits2-1.4.2 {double - cat + par} {
	catch "exec chmod ugo=+rwx $shFitsFile"
	catch "exec rm $shFitsFile"
	set shortFitsLine "DET.DIT             12.12; # kjs"
	exec echo $shortFitsLine > $shFitsFile
	exec $testP $dic $shFitsFile
} "|HIERARCH ESO DET DIT         =       12.120 / Defined integration time          |"

test slxShortToNormalFits2-1.4.3 {double - cat + sub + par} {
	catch "exec chmod ugo=+rwx $shFitsFile"
	catch "exec rm $shFitsFile"
	set shortFitsLine "DET.FRAM.CDELT1         12.0"
	exec echo $shortFitsLine > $shFitsFile
	exec $testP $dic $shFitsFile
} "|HIERARCH ESO DET FRAM CDELT1 =         12.0 / Binning factor along X            |"

test slxShortToNormalFits2-1.4.4 {double - cat + 2 * sub + par} {
	catch "exec chmod ugo=+rwx $shFitsFile"
	catch "exec rm $shFitsFile"
	set shortFitsLine "DET.FRAM.XXXX.CDELT1     12.0    "
	exec echo $shortFitsLine > $shFitsFile
	exec $testP $dic $shFitsFile
} "|HIERARCH ESO DET FRAM XXXX CDELT1=     12.0 / Binning factor along X            |"



