
#!/usr/local/bin/tcl
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxStoreLineBuf.test.tcl,v 1.145 2005/07/27 11:37:45 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# jknudstr  24/01/95  created
#

#************************************************************************
#   NAME
#   slxStoreLineBuf.test.prog - test script for slxStoreLineBuf()
# 
#   SYNOPSIS
#   slxStoreLineBuf.test.prog
# 
#   DESCRIPTION
#   Used for testing the SLX function slxStoreLineBuf().
#
#   The following tests are carried out
#   
#   1.1 Normal behaviour - saving non-exisiting keyword in file
#
#   1.2 Normal behaviour - saving existing keyword in file
#
#   1.3 Normal behaviour - saving existing keyword including '\n'
#
#   2.1 Non-existing file specified
#
#   2.2 Read protected file specified (file could not be opened)
#
#   2.3 Format of keyword line is wrong - SKIPPED!!!
#
#
#   CAUTIONS
#   Should be executed from the ~slx/test directory.
#
#------------------------------------------------------------------------
#
source ../test/defs

set tf slxStoreLineBuf.test
set tempPath [pwd]
set testPath "$tempPath/"
set env(INS_SETUPPATH) [pwd]

# IMPL.: This test is cancelled temporarily
return


# PART 1.1 Normal behaviour - saving non-exisiting keyword in file
# ================================================================
test slxStoreLineBuf-1.1 {Saving non-existing keyword} {
    exec slxTestStoreLineBuf "NEW.KEYW.STOREIT        12345; # A comment"  $tf.1.1.bs.ins $testPath$tf.1.1.temp.ins 

    set msg [exec diff $testPath$tf.1.1.temp.ins $testPath$tf.1.1.rf.ins]
    exec rm $tf.1.1.temp.ins
    set msg
} ""


# PART 1.2 Normal behaviour - saving existing keyword in file
# ===========================================================
test slxStoreLineBuf-1.2 {Saving existing keyword} {
    exec slxTestStoreLineBuf "INS.FOCU.ENC           1234; # A comment"  $tf.1.1.bs.ins $testPath$tf.1.2.temp.ins

    set msg [exec diff $testPath$tf.1.2.temp.ins $testPath$tf.1.2.rf]
    exec rm $tf.1.2.temp.ins
    set msg
} ""


# PART 1.3 Normal behaviour - saving existing keyword including '\n'
# ==================================================================
test slxStoreLineBuf-1.3 {Saving existing keyword including '\n'} {
    exec slxTestStoreLineBuf "INS.FOCU.ENC           1234; # A comment\n"  $tf.1.3.bs.ins $tf.1.3.temp.ins

    set msg [exec diff $tf.1.3.temp.ins $tf.1.3.rf]
    exec rm $tf.1.3.temp.ins
    set msg
} ""



# PART 2.1 Non-existing file specified
# ====================================
test slxStoreLineBuf-2.1 {Non-existing set-up file} {
   if {[catch "exec -keepnewline slxTestStoreLineBuf \"dont care\" nOnExIsT dontCare" msg]} {
        set msg
    }
} "
Error occurred!!

Failure, error is:
   ModuleId   : slx
   LocationId : slxLocateSetupFile.c
   ErrorNumber: 37
   Parameters : slxERR_LOCATE_FILE : Could not find \"nOnExIsT\" according to INS_SETUPPATH or path given with filename\n\n"



# PART 2.2 read protected file specified (file could not be opened)
# =================================================================
exec chmod -rw $tf.2.2.bs 
test slxStoreLineBuf-2.2 {Read protected set-up file} {
   if {[catch "exec -keepnewline slxTestStoreLineBuf \"dont care\" $tf.2.2.bs dontCare" msg]} {
        set msg
    }
} "\nError occurred!!

Failure, error is:
   ModuleId   : slx
   LocationId : slxDetectFormat.c
   ErrorNumber: 1
   Parameters : slxERR_OPEN_FILE : Could not open file \"$testPath/slxStoreLineBuf.test.2.2.bs\"\n\n"
exec chmod +rw $tf.2.2.bs 

if {1 == 0} {
# PART 2.3 Format of keyword line is wrong
# ========================================
test slxStoreLineBuf-2.3 {Format of keyword line is wrong} {
   if {[catch "exec -keepnewline slxTestStoreLineBuf \"thisIsWrongOhSoWrongWrongItIs\" $tf.2.3.bs.ins dontCare" msg]} {
        set msg
    }
} "\nError occurred!!

Failure, error is:
   ModuleId   : slx
   LocationId : slxStoreLineBuf.c
   ErrorNumber: 23
   Parameters : slxERR_FORMAT_KEYWORD : Format of keyword \"thisIsWrongOhSoWrongWrongItIs
\" is wrong\n\n"
}


