/*******************************************************************************
* E.S.O. - VLT project
*
* slxTestGetKeywordBuf.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  24/01/95  Created 1. version
* epozna    30/07/02  comp warnings: exit->return
*/

/************************************************************************
*   NAME
*   slxTestGetKeywordBuf - test program for slxGetKeywordBuf()
* 
*   SYNOPSIS
*   slxTestGetKeywordBuf -k|-n  <keyword>|<number> <set-up filename> [-t]
*
*   DESCRIPTION
*   The program searches for a keyword in a specified set-up file which is
*   loaded into a SLX Dynamic Buffer. The keyword can either be searched for 
*   according to the name (in Short-FITS format) or according to the number 
*   in the set-up file.
*
*   -k|-n         Determines if there should be searched for the keyword 
*                 according to the name (-k) or number in the set-up file.
*
*   <keyword>|    The name of the keyword or the number. The name must be 
*   <number>      given in the Short-FITS format, CAT.[SUB.]PAR. The number
*                 is the number of the keyword as listed in the set-up file. 
*                 First keyword has number 1. Note that the number may
*                 change when the set-up file is updated.
*
*   <set-up file> The set-up file that should be searched in. Can be given in
*                 with or without the extension (".ref", ".ins", ".targ", 
*                 ".det"), and with or without the complete path.
*
*   -t            Used for running the program in test mode. Should not be used
*                 during normal operation.
*
* 
*   ENVIRONMENT
*   INS_SETUPPATH  The variable defines places where set-up files and set-up
*                  related files are situated.
* 
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h" 

static char *rcsId="@(#) $Id: slxTestGetKeywordBuf.c,v 1.145 2005/07/27 11:56:37 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers 
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* Local Headers 
 */
#include "slx.h"
#include "slxTest.h"

/* Main
 */
int main(int argc, char *argv[])

{
    ccsERROR           error;
    ccsPROCNAME        procName;
    char               completeName[256];
    slxKEYW_INFO       keywordInfo;
    slxSETUP_FORMAT    type;
    slxDYN_BUF         setupHandle;
    slxCHANGE_RECORD   changeRec;
    vltUINT8           test = 0;
 
    if (argc < 4)
        {
        printf("\nCorrect syntax is:\n\n");
        printf("slxInfoKeywordSetup -k|-n <keyword>|<number> <set-up>\n\n");
        return(1);
        }

    if (argc == 5)
        test = 1;
    
    strcpy(procName, "slxInfoKeywSUP");
    if (ccsInit(procName, 0, NULL, NULL, &error) == FAILURE)
        {
	slxDumpError(&error);
        return(1);
        }

    if (slxLocateSetupFile(argv[3], completeName, &error) == FAILURE)
        {
	slxDumpError(&error);
	ccsExit(&error);
        return(1);
        }

    if (slxDetectFormat(completeName, &type, &error) == FAILURE)
        {
	slxDumpError(&error);
	ccsExit(&error);
        return(1);
        }
    if (type != slxSHORT_FITS)
	{
	slxDumpError(&error);
	ccsExit(&error);
        return(1);
        }

    slxInitDynBuf(&setupHandle, &error);
    if (slxLoadSetup(completeName, &setupHandle, changeRec, &error) == FAILURE)
	{
	slxDumpError(&error);
	slxResetDynBuf(&setupHandle, &error);
	ccsExit(&error);
        return(1);
	}

    if (strcmp(argv[1], "-k") == 0)
        {
	if (slxGetKeywordBuf(&setupHandle, argv[2], 0, &keywordInfo, 
			     &error) == FAILURE)
            {
	    slxDumpError(&error);
	    slxResetDynBuf(&setupHandle, &error);
            ccsExit(&error);
            return(1);
            }
        }
    else
        {
	if (slxGetKeywordBuf(&setupHandle, "", atoi(argv[2]), &keywordInfo, 
			     &error) == FAILURE)
            {
	    slxDumpError(&error);
	    slxResetDynBuf(&setupHandle, &error);
            ccsExit(&error);
            return(1);
            }
        }

    printf("\nFeatures for the keyword: \n");
    if (!test)
        printf("\nSet-up file searched: %s", completeName);
    printf("\nLine Number: %d", keywordInfo.number);
    printf("\nKeyword Name: %s", keywordInfo.keyword);
    printf("\nCategory: %s", keywordInfo.category);
    printf("\nSubsystem: %s", keywordInfo.subSystem);
    printf("\nParameter: %s", keywordInfo.parameter);
    printf("\nValue: %s", keywordInfo.value);
    printf("\nComment: %s\n\n", keywordInfo.comment);
 
    slxResetDynBuf(&setupHandle, &error);
    ccsExit(&error);
    return(0);
}















