/*******************************************************************************
* E.S.O. - VLT project
*
* slxTestMergeSetupBuf.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  24/01/94  created
* epozna    30/07/02  comp warnings: exit->return,include string.h
*/

/************************************************************************
*   NAME
*   slxTestMergeSetupBuf - test program for slxMergeSetupBuf()
* 
*   SYNOPSIS
*   slxTestMergeSetupBuf <dump filename>
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"


static char *rcsId="@(#) $Id: slxTestMergeSetupBuf.c,v 1.145 2005/07/27 12:00:38 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* Local Headers 
 */
#include "slx.h"
#include "slxTest.h"
#include "string.h"

/* Main
 */
int main (int argc, char *argv[])
{
    ccsERROR          error;
    ccsPROCNAME       procName;
    slxDYN_BUF        setupHandle;
    slxDYN_BUF        setupHandle_1;
    slxDYN_BUF        setupHandle_2;
    slxDYN_BUF        setupHandle_3;
    slxCHANGE_RECORD  changeRec;


    if (argc < 2)
	{
	printf("\nCorrect syntax is:");
	printf("\n\nslxTestMergeSetupBuf <dump filename>\n\n");
	return(1);
	}

    strcpy(procName, "slxTstMrgSUP");
    if (ccsInit(procName, 0, NULL, NULL, &error) == FAILURE)
        {
	slxDumpError(&error);
        return(1);
        }

    slxInitDynBuf(&setupHandle, &error);
    slxInitDynBuf(&setupHandle_1, &error);
    slxInitDynBuf(&setupHandle_2, &error);
    slxInitDynBuf(&setupHandle_3, &error);

    /* Load a couple of setup files
     */
    if (slxLoadSetup("slxMergeSetupBuf.test.1.bs", &setupHandle_1, 
		     changeRec, &error) == FAILURE)
	{
	slxDumpError(&error);
	slxResetDynBuf(&setupHandle_1, &error);
	ccsExit(&error);
        return(1);
	}
    if (slxLoadSetup("slxMergeSetupBuf.test.2.bs", &setupHandle_2, 
		     changeRec, &error) == FAILURE)
	{
	slxDumpError(&error);
	slxResetDynBuf(&setupHandle_1, &error);
	slxResetDynBuf(&setupHandle_2, &error);
	ccsExit(&error);
        return(1);
	}
    if (slxLoadSetup("slxMergeSetupBuf.test.3.bs", &setupHandle_3, 
		     changeRec, &error) == FAILURE)
	{
	slxDumpError(&error);
	slxResetDynBuf(&setupHandle_1, &error);
	slxResetDynBuf(&setupHandle_2, &error);
	slxResetDynBuf(&setupHandle_3, &error);
	ccsExit(&error);
        return(1);
	}

    /* Test 1.1: Non-existing setup file
     */
    fprintf(stderr, "\n1.1: Handling of non-existing setup file\n\n");
    if (slxMergeSetupBuf(&setupHandle, "ISAAC", changeRec, &error, 
			 "-f", "nOTeXiSt", NULL) == FAILURE)
	{
	slxDumpError(&error);
	errResetStack(&error);
	}
    slxResetDynBuf(&setupHandle, &error);

    /* Test 1.2: File not in Short-FITS format
     */
    fprintf(stderr, "\n1.2: Setup file not in Short-FITS format\n\n");
    if (slxMergeSetupBuf(&setupHandle, "ISAAC", changeRec, &error, 
			 "-f", "slxMergeSetupBuf.test.4.bs",
			 "-f", "slxMergeSetupBuf.test.1.bs",
			 NULL) == FAILURE)
	{
	slxDumpError(&error);
	errResetStack(&error);
	}
    slxResetDynBuf(&setupHandle, &error);

    /* Test 2: Functionality test
     */
    if (slxMergeSetupBuf(&setupHandle, "ISAAC", changeRec, &error, 
			 "-f", "slxMergeSetupBuf.test.1.bs", 
			 "slxMergeSetupBuf.test.2.bs",
			 "-b", &setupHandle_1, &setupHandle_2,
			 "-f", "slxMergeSetupBuf.test.1.bs", 
			 "-k", "DET.ID           \"ccd$36\";     # Single keyword merge",
			 "TEL.FOCU.LEN           38.482; # Single keyword merge",
			 "-f", "slxMergeSetupBuf.test.2.bs", 
			 "slxMergeSetupBuf.test.3.bs",
			 "-b", &setupHandle_1, &setupHandle_2, &setupHandle_3,
			 "-k", "INS.PATH         \"RED     \";   # Single keyword merge", "INS.ID           \"EMMI #1 \";   # Single keyword merge",
			 NULL) == FAILURE)
	{
	slxDumpError(&error);
	}

/*    if (slxMergeSetupBuf(&setupHandle, "ISAAC", &error, 
			 "-f", "slxMergeSetupBuf.test.1.bs", 
			 "slxMergeSetupBuf.test.2.bs",
			 "-b", &setupHandle_1, &setupHandle_2,
			 "-f", "slxMergeSetupBuf.test.1.bs", 
			 "-k", "DET.ID           \"ccd$36\";     # Single keyword merge",
			 "TEL.FOCU.LEN           38.482; # Single keyword merge",
			 "-f", "slxMergeSetupBuf.test.2.bs", 
			 "slxMergeSetupBuf.test.3.bs",
			 "-b", &setupHandle_1, &setupHandle_2, &setupHandle_3,
			 "-k", "INS.PATH         \"RED     \";   # Single keyword merge", "INS.ID           \"EMMI #1 \";   # Single keyword merge",
			 NULL) == FAILURE)
	{
	slxDumpError(&error);
	}*/

    slxDumpBuf(&setupHandle, argv[1], &error);
    slxResetDynBuf(&setupHandle, &error);   
    slxResetDynBuf(&setupHandle_1, &error);
    slxResetDynBuf(&setupHandle_2, &error);
    slxResetDynBuf(&setupHandle_3, &error);

    ccsExit(&error);
    return (0);
}


