/*******************************************************************************
* E.S.O. - VLT project
*
* slxTestStoreLine.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  04/08/94  Created 1. version
* epozna    30/07/02  comp warnings: exit->return,incl string
*/

/************************************************************************
*   NAME
*   slxTestStoreLine - stores a line in a set-up file
* 
*   SYNOPSIS
*   slxTestStoreLine "<FITS line>" <set-up filename> 
* 
*   DESCRIPTION
*   The program searches through the set-up file specified for the 
*   <FITS line>; if it finds it, it overwrites the old line with the contents
*   of the new line. Else the new line is appended at the end of the set-up
*   file. Note that also the (possible) old comment will be overwritten with
*   the possible new FITS line comment. 
*   
* 
*   <FITS line>        The FITS keyword line top be stored in the set-u file. 
*                      The FITS line should be given in within quotes 
*                      "<FITS line>".
*   
*   <set-up filename>  The set-up file in which the new FITS keyword line 
*                      should be stored. The set-up file can be either
*                      a Reference Set-Up or a Partial Set-Up file (".ref",
*                      ".ins", ".targ").
*
*
*   ENVIRONMENT
*   INS_SETUPPATH      Specifies paths where set-up files can be located.
*
*
*   RETURN VALUES
*   0  Success
*   1  Success
*
*
*   EXAMPLES
*   slxTestStoreLine "INS.LAMP-2.ST T # Turn Lamp-2 on" mySetUp.ref
*
*------------------------------------------------------------------------
*/

#define _POSIX_SOURCE 1
#include "vltPort.h"


static char *rcsId="@(#) $Id: slxTestStoreLine.c,v 1.145 2005/07/27 12:09:11 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/*
 * Headers
 */

#include <string.h>


/*
 * Local Headers 
 */
#include "slx.h"

/* 
 * Main
 */

int main (int argc, char *argv[])
{
    ccsERROR      error;
    ccsPROCNAME   procName;


    /*
     * Check input parameters
     */

    if (argc < 3)
	{
        printf("\nCorrect syntax is: \n\n");
        printf("slxTestStoreLine \"<FITS line>\" <set-up filename>\n\n");
        return(1);
        }

    errResetStack(&error);

    strcpy(procName, "slxTestStoreLi");
    if (ccsInit(procName, 0, NULL, NULL, &error) == FAILURE)
        {
        printf("\nError in initializing on RTAP!\n\n");
        return(1);
        }

    /*
     * Save the FITS line
     */

    if (slxStoreLine(argv[1], argv[2], &error) != SUCCESS)
        {
        fprintf(stderr, "\nError occurred while storing line!!\n\n");
        fprintf(stderr, "Failure, error is:\n");
        fprintf(stderr, "   ModuleId   : %s\n", error.moduleId);
/*
        fprintf(stderr, "   LocationId : %s\n", error.location);
        fprintf(stderr, "   ErrorNumber: %d\n", error.errorNumber);
*/
        fprintf(stderr, "   Parameters : %s\n", error.runTimePar);
        ccsExit(&error);
        return(1);
        }

    ccsExit(&error);
    return(0);
}

