/*******************************************************************************
* E.S.O. - VLT project
*
* slxTestStoreLineBuf.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  04/08/94  Created 1. version
* epozna    30/07/02  comp warnings: exit->return, incl string.h
*/

/************************************************************************
*   NAME
*   slxTestStoreLineBuf - test program for slxTestStoreLineBuf
* 
*   SYNOPSIS
*   slxTestStoreLineBuf "<FITS line>" <basis file> <dump filename> 
* 
*   DESCRIPTION
*   The program searches through the set-up file specified for the 
*   <FITS line>; if it finds it, it overwrites the old line with the contents
*   of the new line. Else the new line is appended at the end of the set-up
*   file. Note that also the (possible) old comment will be overwritten with
*   the possible new FITS line comment. 
*   
* 
*   <FITS line>        The FITS keyword line top be stored in the set-u file. 
*                      The FITS line should be given in within quotes 
*                      "<FITS line>".
*   
*   <dump filename>    The file in which the keywords will be dumped
*                      should be stored. The set-up file can be either
*                      a Reference Set-Up or a Partial Set-Up file (".ref",
*                      ".ins", ".targ").
*
*
*   ENVIRONMENT
*   INS_SETUPPATH      Specifies paths where set-up files can be located.
*
*
*   RETURN VALUES
*   0  Success
*   1  Success
*
*
*   EXAMPLES
*   slxTestStoreLineBuf "INS.LAMP-2.ST T # Turn Lamp-2 on" \
*                       /home/user1/test/testFile1
*
*------------------------------------------------------------------------
*/

#define _POSIX_SOURCE 1
#include "vltPort.h"


static char *rcsId="@(#) $Id: slxTestStoreLineBuf.c,v 1.145 2005/07/27 12:09:52 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/*
 * Headers
 */
#include <string.h>
/*
 * Local Headers 
 */

#include "slx.h"
#include "slxTest.h"


/* 
 * Main
 */

int main (int argc, char *argv[])
{
    ccsERROR         error;
    ccsPROCNAME      procName;
    slxDYN_BUF       setupHandle;
    slxCHANGE_RECORD changeRec;
    vltINT32         lineNo;

    /* Check input parameters
     */
    if (argc < 4)
	{
        printf("\nCorrect syntax is: \n\n");
        printf("slxTestStoreLineBuf \"<FITS line>\" <basis file> <dump filename>\n\n");
        return(1);
        }

    strcpy(procName, "slxTestStoLiBuf");
    if (ccsInit(procName, 0, NULL, NULL, &error) == FAILURE)
        {
        printf("\nError in initializing on RTAP!\n\n");
        return(1);
        }

    slxInitDynBuf(&setupHandle, &error);

    if (slxLoadSetup(argv[2], &setupHandle, changeRec, &error) == FAILURE)
	{
	slxDumpError(&error);
	slxResetDynBuf(&setupHandle, &error);
        ccsExit(&error);
	return FAILURE;
	}

    /* Save the FITS line
     */
    if (slxStoreLineBuf(argv[1], &setupHandle, &lineNo, &error) == FAILURE)
	{
	slxDumpError(&error);
	slxResetDynBuf(&setupHandle, &error);
        ccsExit(&error);
        return(1);
        }

    if (slxDumpBuf(&setupHandle, argv[3], &error) == FAILURE)
	{
	slxDumpError(&error);
	slxResetDynBuf(&setupHandle, &error);
        ccsExit(&error);
        return(1);
	}

    slxResetDynBuf(&setupHandle, &error);
    ccsExit(&error);
    return(0);
}

